@@ -18,4 +18,4 @@
  * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
  * USA, or visit http://www.gnu.org.
  */
-char **glob(register char *);
+char **glob(char *);
