@@ -19,8 +19,10 @@
  * USA, or visit http://www.gnu.org.
  */
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 /*
  * Squeeze OFF files.
@@ -29,9 +31,9 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
-#include <stdlib.h>	/* for qsort(), malloc() */
+#include <stdlib.h>	/* for qsort(), malloc(), atof() */
 
-#include <ooglutil.h>	/* variable-length arrays */
+#include <ooglutil.h>	/* variable-length arrays, getopt() and stuff */
 
 #ifdef NeXT
 extern char *strdup(const char *);
@@ -123,23 +125,34 @@ static int debug = 0;
 static int vdim = 3;
 vvec geods;				/* List of geodesics to be traced */
 
-Fe *fedge( F *f, V *v0, V *v1 );
+static Fe *fedge( F *f, V *v0, V *v1 );
+static void vmerge(void);
+static void trace_geodesic(Geod *g);
+static E *advance_geodesic(Geod *g, E *avoid);
+static void add_geodesic(Geod *g, char *str);
+static void reflectline(P *axis, P *vector, P *result);
+static void rotation(P *va, P *vb, float T[3][3]);
+static int vcmp(V **p, V **q);
+static void deface(F *f);
+static void normal_ize(F *f);
+/*static E *echeck(int v0, int v1);*/
+/*static void fecheck(Fe *fe);*/
+static void femerge(Fe *fe1, Fe *fe2);
+static void unfedge(Fe *fe);
+static Fe *fedge(F *f, V *v0, V *v1);
 
 #define	New(t)		(t *) malloc(sizeof(t))
 #define NewN(t, N)	(t *) malloc((N)*sizeof(t))
 
-void add_geodesic(Geod *g, char *str);
-void trace_geodesic(Geod *g);
-
-main(argc, argv)
-    char *argv[];
+int main(int argc, char *argv[])
 {
-    register int i;
+    int i;
     int tnv, tne, any;
     int binary = 0, vertsize = 0;
     char *C = "", *N = "", *four = "";
     extern int optind;
     extern char *optarg;
+    IOBFILE *iobf;
 
     VVINIT(geods, Geod, 5);
     vvzero(&geods);
@@ -183,9 +196,11 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
     if(VVCOUNT(geods) > 0)
 	flags |= F_EDGE;
 
+    iobf = iobfileopen(stdin);
+
     vdim = 3;
     for(;;) {
-	switch( fnextc(stdin, 0) ) {
+	switch( iobfnextc(iobf, 0) ) {
 	case 'N': N = "N"; vertsize += 3; goto swallow;
 	case 'C': C = "C"; vertsize += 4; goto swallow;
 	case '4':
@@ -194,13 +209,13 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 		flags &= ~(F_EDGE|F_FACE);
 		goto swallow;
       swallow:
-	case '{': case '=': fgetc(stdin); break;
+	case '{': case '=': iobfgetc(iobf); break;
 	case 'a':	/* Appearance?  Silently swallow keyword */
-	    fexpectstr(stdin, "appearance");
-	    if(fnextc(stdin, 0) == '{') {
+	    iobfexpectstr(iobf, "appearance");
+	    if(iobfnextc(iobf, 0) == '{') {
 		int c, brack = 0;
 		do {
-		    if((c = fgetc(stdin)) == '{') brack++;
+		    if((c = iobfgetc(iobf)) == '{') brack++;
 		    else if(c == '}') brack--;
 		} while(brack > 0 && c != EOF);
 	    }
@@ -210,16 +225,16 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 	}
     }
   done:
-    fexpecttoken(stdin, "OFF");	/* swallow optional OFF prefix */
-    if(fexpecttoken(stdin, "BINARY") == 0) {
+    iobfexpecttoken(iobf, "OFF");	/* swallow optional OFF prefix */
+    if(iobfexpecttoken(iobf, "BINARY") == 0) {
 	binary = 1;
-	fnextc(stdin, 1);
-	fgetc(stdin);
+	iobfnextc(iobf, 1);
+	iobfgetc(iobf);
     }
 
-    if(fgetni(stdin, 1, &nv, 0) <= 0 ||
-	fgetni(stdin, 1, &nf, 0) <= 0 ||
-	fgetni(stdin, 1, &nhash, 0) <= 0 || nv <= 0 || nf <= 0) {
+    if(iobfgetni(iobf, 1, &nv, 0) <= 0 ||
+	iobfgetni(iobf, 1, &nf, 0) <= 0 ||
+	iobfgetni(iobf, 1, &nhash, 0) <= 0 || nv <= 0 || nf <= 0) {
 	   fprintf(stderr, "polymerge: Input not an OFF file.\n");
 	   exit(1);
     }
@@ -237,14 +252,14 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 	V *vp = &Vs[i];
 	
 	vp->p.w = 1;
-	if(fgetnf(stdin, vdim, (float *)&vp->p, binary) < vdim) {
+	if(iobfgetnf(iobf, vdim, (float *)&vp->p, binary) < vdim) {
 	  badvert:
 	    fprintf(stderr, "polymerge: error reading vertex %d/%d\n", i,nv);
 	    exit(1);
 	}
 	if(vertsize) {
 	    Vs[i].more = NewN(float, vertsize);
-	    if(fgetnf(stdin, vertsize, Vs[i].more, binary) < vertsize)
+	    if(iobfgetnf(iobf, vertsize, Vs[i].more, binary) < vertsize)
 		goto badvert;
 	}
 	Vs[i].ref = 0;
@@ -265,29 +280,29 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 
     tossedf = 0;
     for(i = 0; i < nf; i++) {
-	register F *fp = &Fs[i];
-	register Fe *fe;
+	F *fp = &Fs[i];
+	Fe *fe;
 	int nfv, k, c;
 	int v, ov, v0;
 	Fe head;
 	char *cp;
 	char aff[512];
 
-	if(fgetni(stdin, 1, &nfv, binary) <= 0 || nfv <= 0) {
+	if(iobfgetni(iobf, 1, &nfv, binary) <= 0 || nfv <= 0) {
 	    fprintf(stderr, "polymerge: error reading face %d/%d\n",
 		i, nf);
 	    exit(1);
 	}
 	head.prev = head.next = &head;
 	fp->fedges = &head;
-	fgetni(stdin, 1, &v, binary);
+	iobfgetni(iobf, 1, &v, binary);
 	if(v < 0 || v >= nv) {
 	    fprintf(stderr, "polymerge: bad vertex %d on face %d\n", v, i);
 	    exit(1);
 	}
 	v0 = ov = Vs[v].index;		/* Use common vertex if merged */
 	for(k = 1; k < nfv; k++, ov = v) {
-	    fgetni(stdin, 1, &v, binary);
+	    iobfgetni(iobf, 1, &v, binary);
 	    if(v < 0 || v >= nv) {
 		fprintf(stderr, "polymerge: bad vertex %d on face %d\n", v, i);
 		exit(1);
@@ -314,7 +329,8 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 	     */
 	    fp->fedges = NULL;
 	    tossedf++;
-	    debug && printf("# Face %d degenerate already\n", i);
+	    if (debug)
+		    printf("# Face %d degenerate already\n", i);
 	} else {
 	    head.prev->next = fp->fedges = head.next;
 	}
@@ -322,18 +338,18 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 	if(binary) {
 	    int nfc;
 	    float c;
-	    fgetni(stdin, 1, &nfc, binary);
+	    iobfgetni(iobf, 1, &nfc, binary);
 	    if(nfc > 4) {
 		fprintf(stderr, "Bad face color count 0x%x", nfc);
 		exit(1);
 	    }
 	    for(cp = aff; --nfc >= 0; cp += strlen(cp)) {
-		fgetnf(stdin, 1, &c, binary);
+		iobfgetnf(iobf, 1, &c, binary);
 		sprintf(cp, " %g", c);
 	    }
 	} else {
-	    (void) fnextc(stdin, 1);
-	    for(cp = aff; (c = getc(stdin)) != EOF && c != '\n' && cp < &aff[511]; )
+	    (void) iobfnextc(iobf, 1);
+	    for(cp = aff; (c = iobfgetc(iobf)) != EOF && c != '\n' && cp < &aff[511]; )
 		*cp++ = c;
 	}
 	*cp = '\0';
@@ -367,11 +383,11 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 	    E *e;
 
 	    for(e = ehash[i]; e != NULL; e = e->link) {
-		register Fe *fe, *fee;
+		Fe *fe, *fee;
 
 		for(fe = e->feds; fe != NULL; fe = fe->elink) {
 		    for(fee = fe->elink; fee != NULL; fee = fee->elink) {
-			register float dn;
+			float dn;
 
 			dn = VDOT(&fe->n, &fee->n);
 			if(fabs(1 - dn*dn) < EPS_NORMAL) {
@@ -387,7 +403,8 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
 	      another: ;
 	    }
 	}
-	debug && printf("# %d faces merged this pass.\n", any);
+	if (debug)
+		printf("# %d faces merged this pass.\n", any);
       } while(any);
     }
 
@@ -421,7 +438,7 @@ Merges coincident vertices, collinear edges, coplanar faces of an OFF object.\n\
     if(VVCOUNT(geods) > 0) {
 	/* Emit all geodesics as a VECT object. */
 	/* Don't emit anything else. */
-	int i, j, totv, totpl;
+	int i, j, totv = 0, totpl = 0;
 	Geod *g;
 
 	for(i = 0; i < VVCOUNT(geods); i++) {
@@ -462,8 +479,7 @@ if (flags & F_EVOLVER) /* Produce Brakke's evolver .fe format */
     if (vdim == 4) printf("space_dimension 4\n");
     printf("vertices\n");
     for(i = 0; i < nv; i++) {
-	register V *v = &Vs[i];
-	int k;
+	V *v = &Vs[i];
 	if(v->ref || !(flags & 1)) {
 	    v->index = ++j;
 	    printf("%d\t%#g %#g %#g", v->index, v->p.x, v->p.y, v->p.z);
@@ -482,7 +498,7 @@ if (flags & F_EVOLVER) /* Produce Brakke's evolver .fe format */
     printf("\nfaces\n");
     j=0;
     for(i=0; i<nf; i++) {
-	register Fe *fe, *fee;
+	Fe *fe, *fee;
 	int k, nfv;
 
 	fe = Fs[i].fedges;
@@ -502,7 +518,7 @@ else    /* Produce OFF format */
 {
     printf("%s%s%sOFF\n%d %d %d\n", C, N, four, tnv, nf - tossedf, tne);
     for(i = 0; i < nv; i++) {
-	register V *v = &Vs[i];
+	V *v = &Vs[i];
 	int k;
 	if(v->ref || !(flags & F_VERT)) {
 	    printf("%#g %#g %#g", v->p.x, v->p.y, v->p.z);
@@ -520,7 +536,7 @@ else    /* Produce OFF format */
     printf("\n");
     /* ho hum */
     for(i=0; i<nf; i++) {
-	register Fe *fe, *fee;
+	Fe *fe, *fee;
 	int k, nfv;
 
 	fe = Fs[i].fedges;
@@ -536,7 +552,7 @@ else    /* Produce OFF format */
 	if(debug) {
 	    printf(" #");
 	    for(fee = fe, k = nfv; --k >= 0; fee = fee->next)
-		printf(" %d", fee->edge->v[fee->sign] - Vs);
+		    printf(" %d", (int)(fee->edge->v[fee->sign] - Vs));
 	}
 	printf("\n");
     }
@@ -547,8 +563,7 @@ else    /* Produce OFF format */
 /*
  * Add a new faceedge
  */
-Fe *
-fedge(F *f, V *v0, V *v1)
+static Fe *fedge(F *f, V *v0, V *v1)
 {
     Fe *fe;
     E *e;
@@ -585,8 +600,8 @@ fedge(F *f, V *v0, V *v1)
     if(r != 0) {
 	r = 1/sqrt(r);
 	e->to.x *= r;  e->to.y *= r;  e->to.z *= r;
-    } else 
-	debug && printf("# Coincident: %d == %d [%g %g %g]\n", v0->index, v1->index, v0->p.x,v0->p.y,v0->p.z);
+    } else if (debug)
+	printf("# Coincident: %d == %d [%g %g %g]\n", v0->index, v1->index, v0->p.x,v0->p.y,v0->p.z);
     e->link = ehash[t];
     ehash[t] = e;
  gotit:
@@ -599,10 +614,9 @@ fedge(F *f, V *v0, V *v1)
 /*
  * Remove a faceedge from its edge list
  */
-unfedge(fe)
-    register Fe *fe;
+static void unfedge(Fe *fe)
 {
-    register Fe **fepp;
+    Fe **fepp;
     
     for(fepp = &fe->edge->feds; *fepp != NULL; fepp = &(*fepp)->elink) {
 	if(*fepp == fe) {
@@ -617,15 +631,15 @@ unfedge(fe)
  * Merge two faces
  * We delete these face-edges from both faces
  */
-femerge(fe1, fe2)
-    register Fe *fe1, *fe2;
+static void femerge(Fe *fe1, Fe *fe2)
 {
     F *f1, *f2;
-    register Fe *tfe;
+    Fe *tfe;
 
     if(fe1->face == fe2->face) {
-	debug && printf("# Merging two edges of face %d -- tossing it.\n",
-		fe1->face - Fs);
+	if (debug)
+		printf("# Merging two edges of face %d -- tossing it.\n",
+		       (int)(fe1->face - Fs));
 	deface(fe1->face);
 	return;
     }
@@ -656,10 +670,10 @@ femerge(fe1, fe2)
 	f1->fedges = fe2->next;
 	if(debug)
 	    printf("# Merged face %d into %d (vertices %d %d) n1.n2 %g\n",
-		f2-Fs, f1-Fs,
-		fe1->edge->v[fe1->sign]->index,
-		fe1->edge->v[1 - fe1->sign]->index,
-		VDOT(&fe1->n, &fe2->n));
+		   (int)(f2-Fs), (int)(f1-Fs),
+		   fe1->edge->v[fe1->sign]->index,
+		   fe1->edge->v[1 - fe1->sign]->index,
+		   VDOT(&fe1->n, &fe2->n));
     tfe = fe2->next;
     if(tfe == NULL) {
 	fprintf(stderr, "polymerge: face f2 already deleted?\n");
@@ -681,19 +695,17 @@ femerge(fe1, fe2)
 
 #define PRETTY(x)  ((int)(x) - 0x10000000)
 
-fecheck(fe)
-    Fe *fe;
+#if 0
+static void fecheck(Fe *fe)
 {
-    register Fe *fee;
-    int ne;
-    int onface = 0;
-    register F *f;
-    register E *e;
+    Fe *fee;
+    F *f;
+    E *e;
 
     if(fe == NULL)
 	return;
     f = fe->face;
-    fprintf(stderr,"0x%x: on face %d (%x); ", fe, f - Fs, f);
+    fprintf(stderr,"0x%p: on face %d (%p); ", (void *)fe, f - Fs, (void *)f);
     fee = fe;
     do {
 	fprintf(stderr," %s%x[%d%s%d] ", (f->fedges == fee) ? "*" : "",
@@ -703,8 +715,8 @@ fecheck(fe)
 		fee->edge->v[1]->index);
 
 	if(fee->face != f)
-	    fprintf(stderr," Fe %x: face %d (%x) != %x\n",
-		PRETTY(fee), fee->face - Fs, fee->face, f);
+	    fprintf(stderr," Fe %p: face %d (%p) != %p\n",
+		    (void *)PRETTY(fee), fee->face - Fs, (void *)fee->face, (void *)f);
 	if(fee->next->prev != fee)
 	    fprintf(stderr," Fe %x: next %x next->prev %x\n",
 		PRETTY(fee), PRETTY(fee->next), PRETTY(fee->next->prev));
@@ -713,21 +725,21 @@ fecheck(fe)
     } while(fee != fe);
     fprintf(stderr, "\n");
 }
+#endif
 
-E *
-echeck(v0, v1)
-    register int v0, v1;
+#if 0
+static E *echeck(int v0, int v1)
 {
-    register E *e;
+    E *e;
 
     int t = (v0 + v1 + v0*v1) % nhash;
 					/* Symmetric hash function */
     if(v0 > v1) v0 ^= v1, v1 ^= v0, v0 ^= v1;
     for(e = ehash[t]; e != NULL; e = e->link) {
 	if(e->v[0] == &Vs[v0] && e->v[1] == &Vs[v1]) {
-	    register Fe *fe;
-	    fprintf(stderr, "E 0x%x %d-%d (%d-%d)  %x...\n",
-		e, v0,v1, e->v[0]->index, e->v[1]->index, PRETTY(e->feds));
+	    Fe *fe;
+	    fprintf(stderr, "E 0x%p %d-%d (%d-%d)  %p...\n",
+		    (void *)e, v0,v1, e->v[0]->index, e->v[1]->index, (void *)PRETTY(e->feds));
 	    for(fe = e->feds; fe != NULL; fe = fe->elink) {
 		fecheck(fe);
 	    }
@@ -736,19 +748,19 @@ echeck(v0, v1)
     
     return e;
 }
+#endif
 
-
-
-normal_ize(f)
-   F *f;
+static void normal_ize(F *f)
 {
-    register Fe *fe;
+    Fe *fe;
 
     fe = f->fedges;
     if(fe == NULL)
 	return;
     if(fe->prev == fe->next) {
-	debug && printf("# Face %d already degenerate -- tossing it.\n", f - Fs);
+	if (debug)
+	    printf("# Face %d already degenerate -- tossing it.\n",
+		   (int)(f - Fs));
 	deface(f);
 	return;
     }
@@ -758,8 +770,8 @@ normal_ize(f)
 	float r;
 
 	for(;;) {
-	    register Fe *fn, *fee;
-	    register P *pp, *qp;
+	    Fe *fn, *fee;
+	    P *pp, *qp;
 
 	    pp = &fe->edge->to;
 	    fee = fe->next;
@@ -784,7 +796,8 @@ normal_ize(f)
 		/*
 		 * This face became degenerate -- toss it.
 		 */
-		debug && printf("# degenerate face %d\n", f - Fs);
+		if (debug)
+			printf("# degenerate face %d\n", (int)(f - Fs));
 		deface(f);
 		return;
 	    }
@@ -792,7 +805,7 @@ normal_ize(f)
 	}
 
 	r = 1/sqrt(r);
-	if(n.x < 0 ||  (n.x == 0 && n.y < 0 || (n.y == 0 && n.z < 0)))
+	if(n.x < 0 ||  (n.x == 0 && n.y < 0) || (n.y == 0 && n.z < 0))
 	    r = -r;		/* Canonicalize */
 	fe->n.x = n.x*r;  fe->n.y = n.y*r;  fe->n.z = n.z*r;
     } while((fe = fe->next) != f->fedges);
@@ -801,10 +814,9 @@ normal_ize(f)
 /*
  * Delete a face, erasing all edges.
  */
-deface(f)
-    F *f;
+static void deface(F *f)
 {
-    register Fe *fe, *fee;
+    Fe *fe, *fee;
 
     fe = f->fedges;
     if(fe == NULL)
@@ -819,11 +831,10 @@ deface(f)
     tossedf++;
 }
 
-vcmp(p, q)
-    V **p, **q;
+static int vcmp(V **p, V **q)
 {
-    register V *vp, *vq;
-    register float d;
+    V *vp, *vq;
+    float d;
 
     vp = *p;
     vq = *q;
@@ -840,11 +851,11 @@ vcmp(p, q)
     return(vp->p.w < vq->p.w ? -1 : 1);
 }
 
-vmerge()
+static void vmerge(void)
 {
     V **vp;
     int i, j;
-    register V *a, *b;
+    V *a, *b;
     int nexti;
 
     vp = NewN(V *, nv);
@@ -872,7 +883,8 @@ vmerge()
 	    if(fabs(a->p.y - b->p.y) < EPS_POINT
 			&& fabs(a->p.z - b->p.z) < EPS_POINT
 			&& (vdim == 3 || fabs(a->p.w - b->p.w) < EPS_POINT)) {
-		debug && printf("# Vtx %d->%d\n", b->index, a->index);
+		if (debug)
+		    printf("# Vtx %d->%d\n", b->index, a->index);
 		b->index = a->index;
 		b->ref++;
 	    } else if(!nexti)
@@ -885,8 +897,7 @@ vmerge()
 /*
  * Add a geodesic, described by a string.
  */
-void
-add_geodesic(Geod *g, char *str)
+static void add_geodesic(Geod *g, char *str)
 {
     int ok;
     float r;
@@ -913,7 +924,7 @@ and faceno is the integer index of the face on which the base-point lies.\n");
 /* Generate 3x3 rotation matrix which takes va -> vb
  * Assumes va and vb both unit vectors.
  */
-rotation(P *va, P *vb, float T[3][3])
+static void rotation(P *va, P *vb, float T[3][3])
 {
     float adotb = VDOT(va, vb);
     float ab_1 = adotb - 1;
@@ -925,7 +936,6 @@ rotation(P *va, P *vb, float T[3][3])
     apb2 = VDOT(&aperpb, &aperpb);
     if(apb2 == 0) {
 	float dot;
-	P t;
 	if(adotb >= 0) {
 	    /* Either some vector is zero, or they're identical.  No rot'n. */
 	    memset(T, 0, 9*sizeof(float));
@@ -956,7 +966,7 @@ rotation(P *va, P *vb, float T[3][3])
 /*
  * result = vector reflected in axis
  */
-reflectline(P *axis, P *vector, P *result)
+static void reflectline(P *axis, P *vector, P *result)
 {
     float mag = VDOT(axis, axis);
     float along = VDOT(axis, vector);
@@ -976,13 +986,13 @@ reflectline(P *axis, P *vector, P *result)
  *
  * Return a pointer to the edge we found the intersection with, NULL if none.
  */
-E *
+static E *
 advance_geodesic(Geod *g, E *avoid)
 {
     Fe *fed, *bestfed = NULL;
     E *edge;
     P n;
-    P lb, bestbase, bestdir;
+    P lb, bestbase = { 0, }, bestdir = { 0, };
     float r, u, t, bestt;
     int i;
     struct point2 {
@@ -1098,7 +1108,7 @@ advance_geodesic(Geod *g, E *avoid)
 		dir[i]  = bestdir.x*T[0][i] + bestdir.y*T[1][i]
 			+ bestdir.z*T[2][i];
 	    }
-	    g->dir = *(P *)dir;
+	    g->dir = *(P *)(void *)dir;
 	} else {
 	    /* There's no neighboring face.  Must have reached a boundary. */
 	    g->face = NULL;
@@ -1119,11 +1129,10 @@ advance_geodesic(Geod *g, E *avoid)
     }
 }
    
-void
+static void
 trace_geodesic(Geod *g)
 {
     E *edge, *nextedge;
-    int nsteps;
     int maxsteps = 1000000;
     float maxlength = 1e10;
 
