@@ -104,6 +104,15 @@ int gv_load(char * a1, char * a2)
   return retval;
 }
 
+void gv_hdelete(char * a1, char * a2)
+{
+  LObject *val = LEvalFunc("hdelete",
+			   LSTRING, a1,
+			   LSTRING, a2,
+			   LEND);
+  LFree(val);
+}
+
 void gv_name_object(int a1, char * a2)
 {
   LObject *val = LEvalFunc("name-object",
@@ -539,11 +548,11 @@ void gv_pick(int a1, int a2, float *a3, int a3n, float *a4, int a4n, float *a5,
   LObject *val = LEvalFunc("pick",
 			   LID, a1,
 			   LID, a2,
-			   LARRAY, LFLOAT, a3, a3n,
-			   LARRAY, LFLOAT, a4, a4n,
-			   LARRAY, LFLOAT, a5, a5n,
-			   LARRAY, LFLOAT, a6, a6n,
-			   LARRAY, LINT, a7, a7n,
+			   LVARARRAY, LFLOAT, a3, a3n,
+			   LVARARRAY, LFLOAT, a4, a4n,
+			   LVARARRAY, LFLOAT, a5, a5n,
+			   LVARARRAY, LFLOAT, a6, a6n,
+			   LVARARRAY, LINT, a7, a7n,
 			   LINT, a8,
 			   LARRAY, LINT, a9, a9n,
 			   LINT, a10,
@@ -576,6 +585,22 @@ void gv_dither(int a1, int a2)
   LFree(val);
 }
 
+void gv_ui_html_browser(char * a1)
+{
+  LObject *val = LEvalFunc("ui-html-browser",
+			   LSTRING, a1,
+			   LEND);
+  LFree(val);
+}
+
+void gv_ui_pdf_viewer(char * a1)
+{
+  LObject *val = LEvalFunc("ui-pdf-viewer",
+			   LSTRING, a1,
+			   LEND);
+  LFree(val);
+}
+
 void gv_shell(char * a1)
 {
   LObject *val = LEvalFunc("shell",
@@ -593,6 +618,23 @@ void gv_write_sexpr(char * a1, LObject * a2)
   LFree(val);
 }
 
+void gv_write_handle(char * a1, char * a2, char * a3)
+{
+  LObject *val = LEvalFunc("write-handle",
+			   LSTRING, a1,
+			   LSTRING, a2,
+			   LSTRING, a3,
+			   LEND);
+  LFree(val);
+}
+
+void gv_dump_handles()
+{
+  LObject *val = LEvalFunc("dump-handles",
+			   LEND);
+  LFree(val);
+}
+
 char * gv_geomview_version()
 {
   LObject *val = LEvalFunc("geomview-version",
@@ -660,6 +702,18 @@ TmNStruct * gv_ND_xform(int a1, TmNStruct * a2)
   return retval;
 }
 
+TmNStruct * gv_ND_xform_set(int a1, TmNStruct * a2)
+{
+  LObject *val = LEvalFunc("ND-xform-set",
+			   LID, a1,
+			   LTRANSFORMN, a2,
+			   LEND);
+  TmNStruct * retval;
+  LFROMOBJ(LTRANSFORMN)(val, &retval);
+  LFree(val);
+  return retval;
+}
+
 TmNStruct * gv_ND_xform_get(int a1, int a2)
 {
   LObject *val = LEvalFunc("ND-xform-get",
@@ -1022,6 +1076,23 @@ char * gv_bgimagefile(int a1, char * a2)
   return retval;
 }
 
+void gv_ui_motion(int a1, int a2)
+{
+  LObject *val = LEvalFunc("ui-motion",
+			   LKEYWORD, a1,
+			   LKEYWORD, a2,
+			   LEND);
+  LFree(val);
+}
+
+void gv_ui_cam_focus(int a1)
+{
+  LObject *val = LEvalFunc("ui-cam-focus",
+			   LKEYWORD, a1,
+			   LEND);
+  LFree(val);
+}
+
 void gv_ui_target(int a1, int a2)
 {
   LObject *val = LEvalFunc("ui-target",
@@ -1039,6 +1110,14 @@ void gv_ui_center(int a1)
   LFree(val);
 }
 
+void gv_ui_center_origin(int a1)
+{
+  LObject *val = LEvalFunc("ui-center-origin",
+			   LKEYWORD, a1,
+			   LEND);
+  LFree(val);
+}
+
 void gv_ui_panel(char * a1, int a2, WindowStruct * a3)
 {
   LObject *val = LEvalFunc("ui-panel",
@@ -1094,6 +1173,7 @@ extern LObject *Lclock(Lake *, LList *);
 extern LObject *Lecho(Lake *, LList *);
 extern LObject *Lemodule_transmit(Lake *, LList *);
 extern LObject *Lload(Lake *, LList *);
+extern LObject *Lhdelete(Lake *, LList *);
 extern LObject *Lname_object(Lake *, LList *);
 extern LObject *Lupdate_draw(Lake *, LList *);
 extern LObject *Ldraw(Lake *, LList *);
@@ -1144,14 +1224,19 @@ extern LObject *Lpick(Lake *, LList *);
 extern LObject *Levent_keys(Lake *, LList *);
 extern LObject *Levent_pick(Lake *, LList *);
 extern LObject *Ldither(Lake *, LList *);
+extern LObject *Lui_html_browser(Lake *, LList *);
+extern LObject *Lui_pdf_viewer(Lake *, LList *);
 extern LObject *Lshell(Lake *, LList *);
 extern LObject *Lwrite_sexpr(Lake *, LList *);
+extern LObject *Lwrite_handle(Lake *, LList *);
+extern LObject *Ldump_handles(Lake *, LList *);
 extern LObject *Lgeomview_version(Lake *, LList *);
 extern LObject *Lrib_display(Lake *, LList *);
 extern LObject *Lrib_snapshot(Lake *, LList *);
 extern LObject *LND_axes(Lake *, LList *);
 extern LObject *Ldimension(Lake *, LList *);
 extern LObject *LND_xform(Lake *, LList *);
+extern LObject *LND_xform_set(Lake *, LList *);
 extern LObject *LND_xform_get(Lake *, LList *);
 extern LObject *LND_color(Lake *, LList *);
 extern LObject *Lzoom(Lake *, LList *);
@@ -1187,8 +1272,11 @@ extern LObject *Lcamera_prop(Lake *, LList *);
 extern LObject *Lwrite(Lake *, LList *);
 extern LObject *Lsnapshot(Lake *, LList *);
 extern LObject *Lbgimagefile(Lake *, LList *);
+extern LObject *Lui_motion(Lake *, LList *);
+extern LObject *Lui_cam_focus(Lake *, LList *);
 extern LObject *Lui_target(Lake *, LList *);
 extern LObject *Lui_center(Lake *, LList *);
+extern LObject *Lui_center_origin(Lake *, LList *);
 extern LObject *Lui_panel(Lake *, LList *);
 extern LObject *Lui_freeze(Lake *, LList *);
 extern LObject *Lui_emodule_define(Lake *, LList *);
@@ -1207,6 +1295,7 @@ extern char Hclock[];
 extern char Hecho[];
 extern char Hemodule_transmit[];
 extern char Hload[];
+extern char Hhdelete[];
 extern char Hname_object[];
 extern char Hupdate_draw[];
 extern char Hdraw[];
@@ -1257,14 +1346,19 @@ extern char Hpick[];
 extern char Hevent_keys[];
 extern char Hevent_pick[];
 extern char Hdither[];
+extern char Hui_html_browser[];
+extern char Hui_pdf_viewer[];
 extern char Hshell[];
 extern char Hwrite_sexpr[];
+extern char Hwrite_handle[];
+extern char Hdump_handles[];
 extern char Hgeomview_version[];
 extern char Hrib_display[];
 extern char Hrib_snapshot[];
 extern char HND_axes[];
 extern char Hdimension[];
 extern char HND_xform[];
+extern char HND_xform_set[];
 extern char HND_xform_get[];
 extern char HND_color[];
 extern char Hzoom[];
@@ -1300,8 +1394,11 @@ extern char Hcamera_prop[];
 extern char Hwrite[];
 extern char Hsnapshot[];
 extern char Hbgimagefile[];
+extern char Hui_motion[];
+extern char Hui_cam_focus[];
 extern char Hui_target[];
 extern char Hui_center[];
+extern char Hui_center_origin[];
 extern char Hui_panel[];
 extern char Hui_freeze[];
 extern char Hui_emodule_define[];
@@ -1323,6 +1420,7 @@ void clang_init()
   LDefun("echo", Lecho, Hecho);
   LDefun("emodule-transmit", Lemodule_transmit, Hemodule_transmit);
   LDefun("load", Lload, Hload);
+  LDefun("hdelete", Lhdelete, Hhdelete);
   LDefun("name-object", Lname_object, Hname_object);
   LDefun("update-draw", Lupdate_draw, Hupdate_draw);
   LDefun("draw", Ldraw, Hdraw);
@@ -1373,14 +1471,19 @@ void clang_init()
   LDefun("event-keys", Levent_keys, Hevent_keys);
   LDefun("event-pick", Levent_pick, Hevent_pick);
   LDefun("dither", Ldither, Hdither);
+  LDefun("ui-html-browser", Lui_html_browser, Hui_html_browser);
+  LDefun("ui-pdf-viewer", Lui_pdf_viewer, Hui_pdf_viewer);
   LDefun("shell", Lshell, Hshell);
   LDefun("write-sexpr", Lwrite_sexpr, Hwrite_sexpr);
+  LDefun("write-handle", Lwrite_handle, Hwrite_handle);
+  LDefun("dump-handles", Ldump_handles, Hdump_handles);
   LDefun("geomview-version", Lgeomview_version, Hgeomview_version);
   LDefun("rib-display", Lrib_display, Hrib_display);
   LDefun("rib-snapshot", Lrib_snapshot, Hrib_snapshot);
   LDefun("ND-axes", LND_axes, HND_axes);
   LDefun("dimension", Ldimension, Hdimension);
   LDefun("ND-xform", LND_xform, HND_xform);
+  LDefun("ND-xform-set", LND_xform_set, HND_xform_set);
   LDefun("ND-xform-get", LND_xform_get, HND_xform_get);
   LDefun("ND-color", LND_color, HND_color);
   LDefun("zoom", Lzoom, Hzoom);
@@ -1416,8 +1519,11 @@ void clang_init()
   LDefun("write", Lwrite, Hwrite);
   LDefun("snapshot", Lsnapshot, Hsnapshot);
   LDefun("background-image", Lbgimagefile, Hbgimagefile);
+  LDefun("ui-motion", Lui_motion, Hui_motion);
+  LDefun("ui-cam-focus", Lui_cam_focus, Hui_cam_focus);
   LDefun("ui-target", Lui_target, Hui_target);
   LDefun("ui-center", Lui_center, Hui_center);
+  LDefun("ui-center-origin", Lui_center_origin, Hui_center_origin);
   LDefun("ui-panel", Lui_panel, Hui_panel);
   LDefun("ui-freeze", Lui_freeze, Hui_freeze);
   LDefun("emodule-define", Lui_emodule_define, Hui_emodule_define);
