@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 /*
  * geomview custom lisp object types
@@ -47,84 +49,107 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "transform.h"
 #include "fsa.h"
 #include "lang.h"
+#include "freelist.h"
 
-extern HandleOps TransOps, GeomOps, CamOps, WindowOps;
+extern HandleOps GeomOps, CamOps, WindowOps;
+extern HandleOps TransOps, NTransOps, ImageOps, AppearanceOps;
 
 LObject *L0, *L1;
 
 static Fsa lang_fsa = NULL;
 #define REJECT -1
 
+static char **keyword_names;
+static int n_keywords;
+
+static DEF_FREELIST(HandleRefStruct);
+static DEF_FREELIST(TransformStruct);
+
+#ifdef NEW
+# undef NEW
+#endif
+#define NEW(type, name)				\
+{						\
+  HandleRefStruct *hrname;			\
+  FREELIST_NEW(HandleRefStruct, hrname);	\
+  name = (type##Struct *)hrname;		\
+}
+#ifdef DELETE
+# undef DELETE
+#endif
+#define DELETE(name) FREELIST_FREE(HandleRefStruct, name)
+
 /************************************************************************
  CAMERA LISP OBJECT
  ************************************************************************/
 static CameraStruct *camcopy(CameraStruct *old)
 {
-  CameraStruct *new = OOGLNew(CameraStruct);
-  if(old) *new = *old;
-  else new->cam = NULL, new->h = NULL;
-  if (new->cam) RefIncr((Ref*)(new->cam));
-  if (new->h) RefIncr((Ref*)(new->h));
-  return new;
+  CameraStruct *newc;
+  
+  NEW(Camera, newc);
+
+  if(old) *newc = *old;
+  else newc->cam = NULL, newc->h = NULL;
+  if (newc->cam) RefIncr((Ref*)(newc->cam));
+  if (newc->h) RefIncr((Ref*)newc->h);
+  return newc;
 }
 
-int camfromobj(obj, x)
-    LObject *obj;
-    CameraStruct * *x;
+static bool camfromobj(LObject *obj, CameraStruct **x)
 {
   if (obj->type != LCAMERA) return 0;
   *x = LCAMERAVAL(obj);
   return 1;
 }
 
-LObject *cam2obj(x)
-    CameraStruct * *x;
+static LObject *cam2obj(CameraStruct **x)
 {
   CameraStruct *copy = camcopy(*x);
   return LNew( LCAMERA, &copy );
 }
 
-void camfree(x)
-    CameraStruct * *x;
+static void camfree(CameraStruct **x)
 {
   if (*x) {
     if ((*x)->cam) CamDelete( (*x)->cam );
     if ((*x)->h) HandleDelete( (*x)->h );
+    DELETE(*x);
   }
-  OOGLFree(*x);
 }
 
-int cammatch(a, b)
-    CameraStruct **a,**b;
+static bool cammatch(CameraStruct **a, CameraStruct **b)
 { 
   if ((*a)->h && ((*a)->h == (*b)->h)) return 1;
   if ((*a)->cam && ((*a)->cam == (*b)->cam)) return 1;
   return 0;
 }
 
-void camwrite(fp, x)
-    FILE *fp;
-    CameraStruct * *x;
+static void camwrite(FILE *fp, CameraStruct **x)
 {
   CamFSave( (*x)->cam, fp, "lisp output stream" );
 }
 
-void campull(a_list, x)
-    va_list *a_list;
-    CameraStruct * *x;
+static void campull(va_list *a_list, CameraStruct **x)
 {
   *x = va_arg(*a_list, CameraStruct*);
 }
 
-LObject *camparse(Lake *lake)
+static LObject *camparse(Lake *lake)
 {
-  CameraStruct *new = OOGLNew(CameraStruct);
-  new->h = NULL; new->cam = NULL;
-  if (CamOps.strmin(POOL(lake), (Handle **)&(new->h),
-                                 (Ref **)&(new->cam)) == 0) {
+  CameraStruct *newc;
+
+  NEW(Camera, newc);
+
+  newc->h = NULL; newc->cam = NULL;
+  if (CamOps.strmin(POOL(lake), (Handle **)&newc->h,
+		    (Ref **)(void *)&(newc->cam)) == 0) {
     return Lnil;
   } else
-    return LNew( LCAMERA, &new );
+    if(newc->h && !newc->h->permanent) {
+      HandleDelete(newc->h);
+      newc->h = NULL;
+    }
+    return LNew( LCAMERA, &newc );
 }
 
 LType LCamerap = {
@@ -138,7 +163,7 @@ LType LCamerap = {
   campull,
   camparse,
   LTypeMagic
-  };
+};
 
 /************************************************************************
  * WINDOW LISP OBJECT 							*
@@ -146,75 +171,76 @@ LType LCamerap = {
 
 static WindowStruct *wncopy(WindowStruct *old)
 {
-  WindowStruct *new = OOGLNew(WindowStruct);
-  if(old) *new = *old;
-  else new->wn = NULL, new->h = NULL;
-  if (new->wn) RefIncr((Ref*)(new->wn));
-  if (new->h) RefIncr((Ref*)(new->h));
-  return new;
+  WindowStruct *neww;
+
+  NEW(Window, neww);
+
+  if(old) *neww = *old;
+  else neww->wn = NULL, neww->h = NULL;
+  if (neww->wn) RefIncr((Ref*)(neww->wn));
+  if (neww->h) RefIncr((Ref*)neww->h);
+  return neww;
 }
 
-int wnfromobj(obj, x)
-    LObject *obj;
-    WindowStruct * *x;
+static bool wnfromobj(LObject *obj, WindowStruct **x)
 {
   if (obj->type != LWINDOW) return 0;
   *x = LWINDOWVAL(obj);
   return 1;
 }
 
-LObject *wn2obj(x)
-    WindowStruct * *x;
+static LObject *wn2obj(WindowStruct **x)
 {
   WindowStruct *copy = wncopy(*x);
   return LNew( LWINDOW, &copy );
 }
 
-void wnfree(x)
-    WindowStruct * *x;
+static void wnfree(WindowStruct **x)
 {
   if (*x) {
     if ((*x)->wn) WnDelete( (*x)->wn );
     if ((*x)->h) HandleDelete( (*x)->h );
+    DELETE(*x);
   }
-  OOGLFree(*x);
 }
 
-int wnmatch(a, b)
-    WindowStruct **a,**b;
+static bool wnmatch(WindowStruct **a, WindowStruct **b)
 { 
   if ((*a)->h && ((*a)->h == (*b)->h)) return 1;
   if ((*a)->wn && ((*a)->wn == (*b)->wn)) return 1;
   return 0;
 }
 
-void wnwrite(fp, x)
-    FILE *fp;
-    WindowStruct * *x;
+static void wnwrite(FILE *fp, WindowStruct * *x)
 {
-  Pool *p = PoolStreamTemp("", fp, 1, &WindowOps);
+  Pool *p = PoolStreamTemp("", NULL, fp, 1, &WindowOps);
   if(p == NULL)
     return;
   (void) WnStreamOut(p, (*x)->h, (*x)->wn);
   PoolDelete(p);
 }
 
-void wnpull(a_list, x)
-    va_list *a_list;
-    WindowStruct * *x;
+static void wnpull(va_list *a_list, WindowStruct **x)
 {
   *x = va_arg(*a_list, WindowStruct*);
 }
 
-LObject *wnparse(Lake *lake)
+static LObject *wnparse(Lake *lake)
 {
-  WindowStruct *new = OOGLNew(WindowStruct);
-  new->h = NULL; new->wn = NULL;
-  if (WindowOps.strmin(POOL(lake),(Handle **)&(new->h),
-  			       (Ref **)&(new->wn)) == 0) {
+  WindowStruct *neww;
+
+  NEW(Window, neww);
+
+  neww->h = NULL; neww->wn = NULL;
+  if (WindowOps.strmin(POOL(lake),(Handle **)&neww->h,
+		       (Ref **)(void *)&(neww->wn)) == 0) {
     return Lnil;
   } else
-    return LNew( LWINDOW, &new );
+    if(neww->h && !neww->h->permanent) {
+      HandleDelete(neww->h);
+      neww->h = NULL;
+    }
+    return LNew( LWINDOW, &neww );
 }
 
 LType LWindowp = {
@@ -228,7 +254,7 @@ LType LWindowp = {
   wnpull,
   wnparse,
   LTypeMagic
-  };
+};
 
 /************************************************************************
  * GEOM LISP OBJECT 							*
@@ -236,75 +262,71 @@ LType LWindowp = {
 
 static GeomStruct *geomcopy(GeomStruct *old)
 {
-  GeomStruct *new = OOGLNew(GeomStruct);
-  if(old) *new = *old;
-  else new->geom = NULL, new->h = NULL;
-  if (new->geom) RefIncr((Ref*)(new->geom));
-  if (new->h) RefIncr((Ref*)(new->h));
-  return new;
+  GeomStruct *newg;
+
+  NEW(Geom, newg);
+  if(old) *newg = *old;
+  else newg->geom = NULL, newg->h = NULL;
+  if (newg->geom) RefIncr((Ref*)(newg->geom));
+  if (newg->h) RefIncr((Ref*)newg->h);
+  return newg;
 }
 
-int geomfromobj(obj, x)
-    LObject *obj;
-    GeomStruct * *x;
+static bool geomfromobj(LObject *obj, GeomStruct **x)
 {
   if (obj->type != LGEOM) return 0;
   *x = LGEOMVAL(obj);
   return 1;
 }
 
-LObject *geom2obj(x)
-    GeomStruct * *x;
+static LObject *geom2obj(GeomStruct **x)
 {
   GeomStruct *copy = geomcopy(*x);
   return LNew( LGEOM, &copy );
 }
 
-void geomfree(x)
-    GeomStruct * *x;
+static void geomfree(GeomStruct **x)
 {
   if (*x) {
     if ((*x)->geom) GeomDelete( (*x)->geom );
     if ((*x)->h) HandleDelete( (*x)->h );
+    DELETE(*x);
   }
-  OOGLFree(*x);
 }
 
-int geommatch(a, b)
-    GeomStruct **a,**b;
+static bool geommatch(GeomStruct **a, GeomStruct **b)
 { 
   if ((*a)->h && ((*a)->h == (*b)->h)) return 1;
   if ((*a)->geom && ((*a)->geom == (*b)->geom)) return 1;
   return 0;
 }
 
-void geomwrite(fp, x)
-    FILE *fp;
-    GeomStruct * *x;
+static void geomwrite(FILE *fp, GeomStruct **x)
 {
   GeomFSave( (*x)->geom, fp, "lisp output stream" );
 }
 
-void geompull(a_list, x)
-    va_list *a_list;
-    GeomStruct * *x;
+static void geompull(va_list *a_list, GeomStruct **x)
 {
   *x = va_arg(*a_list, GeomStruct*);
 }
 
-LObject *geomparse(Lake *lake)
+static LObject *geomparse(Lake *lake)
 {
-  GeomStruct *new = OOGLNew(GeomStruct);
-  new->h = NULL; new->geom = NULL;
-  if (GeomOps.strmin(POOL(lake), (Handle **)&(new->h), 
-  			    (Ref **)&(new->geom)) == 0) {
+  GeomStruct *newg;
+
+  NEW(Geom, newg);
+
+  newg->h = NULL; newg->geom = NULL;
+  if (GeomOps.strmin(POOL(lake), (Handle **)&newg->h, 
+		     (Ref **)(void *)&(newg->geom)) == 0) {
     return Lnil;
   } else {
-    /* the following line should appear in all our parse routines.
-       Actually, this whole procedure and others like it should
-       be consolidated into one "ooglparse" thing... */
-    if(new->h && !new->h->permanent) new->h = NULL;
-    return LNew( LGEOM, &new );
+    if(newg->h && !newg->h->permanent) {
+      HandleDelete(newg->h);
+      newg->h = NULL;
+    }
+    return LNew( LGEOM, &newg );
   }
 }
 
@@ -319,7 +341,7 @@ LType LGeomp = {
   geompull,
   geomparse,
   LTypeMagic
-  };
+};
 
 /************************************************************************
  * AP LISP OBJECT							*
@@ -327,79 +349,70 @@ LType LGeomp = {
 
 static ApStruct *apcopy(ApStruct *old)
 {
-  ApStruct *new = OOGLNew(ApStruct);
-  if(old) *new = *old;
-  else new->ap = NULL, new->h = NULL;
-  if (new->ap) RefIncr((Ref*)(new->ap));
-#if 0
-  /* no one uses ap handles yet, and callers don't bother
-     to initialize the 'h' member to NULL, so don't do this
-     yet */
-  if (new->h) RefIncr((Ref*)(new->h));
-#endif
-  return new;
+  ApStruct *newap;
+  
+  NEW(Ap, newap);
+  if(old) *newap = *old;
+  else newap->ap = NULL, newap->h = NULL;
+  if (newap->ap) RefIncr((Ref*)(newap->ap));
+  if (newap->h) RefIncr((Ref*)newap->h);
+  return newap;
 }
 
-int apfromobj(obj, x)
-    LObject *obj;
-    ApStruct * *x;
+static bool apfromobj(LObject *obj, ApStruct **x)
 {
   if (obj->type != LAP) return 0;
   *x = LAPVAL(obj);
   return 1;
 }
 
-LObject *ap2obj(x)
-    ApStruct * *x;
+static LObject *ap2obj(ApStruct **x)
 {
   ApStruct *copy = apcopy(*x);
   return LNew( LAP, &copy );
 }
 
-void apfree(x)
-    ApStruct * *x;
+static void apfree(ApStruct **x)
 {
   if (*x) {
     if ((*x)->ap) ApDelete( (*x)->ap );
-#if 0
-    /* don't do the handle; see comment above */
     if ((*x)->h) HandleDelete( (*x)->h );
-#endif
+    DELETE(*x);
   }
-  OOGLFree(*x);
 }
 
-int apmatch(a, b)
-    ApStruct **a,**b;
+static bool apmatch(ApStruct **a, ApStruct **b)
 { 
-#if 0
-  /* don't do the handle; see comment above */
   if ((*a)->h && ((*a)->h == (*b)->h)) return 1;
-#endif
   if ((*a)->ap && ((*a)->ap == (*b)->ap)) return 1;
   return 0;
 }
 
-void apwrite(FILE *fp, ApStruct * *x)
+static void apwrite(FILE *fp, ApStruct * *x)
 {
-  ApFSave( (*x)->ap, (*x)->ap->handle, fp, "lisp output stream" );
+  ApFSave((*x)->ap, fp, "lisp output stream");
 }
 
-void appull(a_list, x)
-    va_list *a_list;
-    ApStruct * *x;
+static void appull(va_list *a_list, ApStruct **x)
 {
   *x = va_arg(*a_list, ApStruct*);
 }
 
-LObject *apparse(Lake *lake)
+static LObject *apparse(Lake *lake)
 {
-  ApStruct *new = OOGLNew(ApStruct);
-  new->h = NULL; new->ap = NULL;
-  if (ApStreamIn(POOL(lake), &(new->h), &(new->ap)) == 0) {
+  ApStruct *newap;
+
+  NEW(Ap, newap);
+
+  newap->h = NULL; newap->ap = NULL;
+  if (ApStreamIn(POOL(lake), &newap->h, &(newap->ap)) == 0) {
     return Lnil;
   } else
-    return LNew( LAP, &new );
+    if(newap->h && !newap->h->permanent) {
+      HandleDelete(newap->h);
+      newap->h = NULL;
+    }
+    return LNew( LAP, &newap );
 }
 
 LType LApp = {
@@ -413,7 +426,7 @@ LType LApp = {
   appull,
   apparse,
   LTypeMagic
-  };
+};
 
 
 /************************************************************************
@@ -422,68 +435,69 @@ LType LApp = {
 
 static TransformStruct *tmcopy(TransformStruct *old)
 {
-  TransformStruct *new = OOGLNew(TransformStruct);
-  if(old) *new = *old;
-  else new->h = NULL;
-  if (new->h) RefIncr((Ref*)(new->h));
-  return new;
+  TransformStruct *newt;
+
+  FREELIST_NEW(TransformStruct, newt);
+
+  if (old) *newt = *old;
+  else newt->h = NULL;
+  if (newt->h) RefIncr((Ref*)newt->h);
+  return newt;
 }
 
-int tmfromobj(obj, x)
-    LObject *obj;
-    TransformStruct * *x;
+static bool tmfromobj(LObject *obj, TransformStruct **x)
 {
   if (obj->type != LTRANSFORM) return 0;
   *x = LTRANSFORMVAL(obj);
   return 1;
 }
 
-LObject *tm2obj(x)
-    TransformStruct * *x;
+static LObject *tm2obj(TransformStruct **x)
 {
   TransformStruct *copy = tmcopy(*x);
-  return LNew( LTRANSFORM, &copy );
+  return LNew(LTRANSFORM, &copy);
 }
 
-void tmfree(x)
-    TransformStruct * *x;
+static void tmfree(TransformStruct **x)
 {
   if (*x) {
-    if ((*x)->h) HandleDelete( (*x)->h );
+    if ((*x)->h) HandleDelete((*x)->h);
+    FREELIST_FREE(TransformStruct, *x);
   }
-  OOGLFree(*x);
 }
 
-int tmmatch(a, b)
-    TransformStruct **a,**b;
+static bool tmmatch(TransformStruct **a, TransformStruct **b)
 { 
   if ((*a)->h && ((*a)->h == (*b)->h)) return 1;
   if ((*a)->tm && ((*a)->tm == (*b)->tm)) return 1;
-  return TmCompare( (*a)->tm, (*b)->tm, (float)0.0 );
+  return TmCompare((*a)->tm, (*b)->tm, (float)0.0);
 }
 
-void tmwrite(fp, x)
-    FILE *fp;
-    TransformStruct * *x;
+static void tmwrite(FILE *fp, TransformStruct **x)
 {
-  fputtransform(fp, 1, (float*)((*x)->tm), 0);
+  TransFSave((*x)->tm, fp, "lisp output stream");
 }
 
-void tmpull(a_list, x)
-    va_list *a_list;
-    TransformStruct * *x;
+static void tmpull(va_list *a_list, TransformStruct **x)
 {
   *x = va_arg(*a_list, TransformStruct*);
 }
 
-LObject *tmparse(Lake *lake)
+static LObject *tmparse(Lake *lake)
 {
-  TransformStruct *new = OOGLNew(TransformStruct);
-  new->h = NULL;
-  if (TransOps.strmin(POOL(lake), (Handle **)&(new->h), (Ref **)&(new->tm)) == 0) {
+  TransformStruct *newt;
+  
+  FREELIST_NEW(TransformStruct, newt);
+
+  newt->h = NULL;
+  if (TransStreamIn(POOL(lake), (Handle **)&newt->h, newt->tm) == false) {
     return Lnil;
   } else
-    return LNew( LTRANSFORM, &new );
+    if(newt->h && !newt->h->permanent) {
+      HandleDelete(newt->h);
+      newt->h = NULL;
+    }
+    return LNew( LTRANSFORM, &newt );
 }
 
 LType LTransformp = {
@@ -497,7 +511,7 @@ LType LTransformp = {
   tmpull,
   tmparse,
   LTypeMagic
-  };
+};
 
 /************************************************************************
  * N-D TRANSFORM LISP OBJECT						*
@@ -505,68 +519,74 @@ LType LTransformp = {
 
 static TmNStruct *tmncopy(TmNStruct *old)
 {
-  TmNStruct *new = OOGLNew(TmNStruct);
-  if(old) *new = *old;
-  else new->tm = NULL, new->h = NULL;
-  if (new->tm) RefIncr((Ref*)(new->tm));
-  if (new->h) RefIncr((Ref*)(new->h));
-  return new;
+  TmNStruct *newt;
+
+  NEW(TmN, newt);
+
+  if(old) *newt = *old;
+  else newt->tm = NULL, newt->h = NULL;
+  if (newt->tm) RefIncr((Ref*)(newt->tm));
+  if (newt->h) RefIncr((Ref*)newt->h);
+  return newt;
 }
 
-int tmnfromobj(obj, x)
-    LObject *obj;
-    TmNStruct * *x;
+static bool tmnfromobj(LObject *obj, TmNStruct **x)
 {
   if (obj->type != LTRANSFORMN) return 0;
   *x = LTRANSFORMNVAL(obj);
   return 1;
 }
 
-LObject *tmn2obj( TmNStruct **x )
+static LObject *tmn2obj( TmNStruct **x )
 {
   TmNStruct *copy = tmncopy(*x);
   return LNew( LTRANSFORMN, &copy );
 }
 
-void tmnfree(TmNStruct **x)
+static void tmnfree(TmNStruct **x)
 {
   if (*x) {
     if ((*x)->tm) TmNDelete( (*x)->tm );
     if ((*x)->h) HandleDelete( (*x)->h );
+    DELETE(*x);
   }
-  OOGLFree(*x);
 }
 
-int tmnmatch(register TmNStruct **a, register TmNStruct **b)
+static bool tmnmatch(TmNStruct **a, TmNStruct **b)
 { 
   if ((*a)->h && ((*a)->h == (*b)->h)) return 1;
   if ((*a)->tm && ((*a)->tm == (*b)->tm)) return 1;
   return 0;
 }
 
-void tmnwrite(FILE *fp, TmNStruct **x)
+static void tmnwrite(FILE *fp, TmNStruct **x)
 {
   TmNPrint( fp, (*x)->tm );
 }
 
-void tmnpull(a_list, x)
-    va_list *a_list;
-    TmNStruct * *x;
+static void tmnpull(va_list *a_list, TmNStruct **x)
 {
   *x = va_arg(*a_list, TmNStruct*);
 }
 
-LObject *tmnparse(Lake *lake)
+static LObject *tmnparse(Lake *lake)
 {
-  TransformN *T = TmNRead(lake->streamin);
-  TmNStruct *new;
-  if(T) {
-    new = OOGLNew(TmNStruct);
-    new->h = NULL;
-    new->tm = T;
-    return LNew( LTRANSFORMN, &new );
+  TmNStruct *newt;
+
+  NEW(TmN, newt);
+
+  newt->h = NULL;
+  newt->tm = NULL;
+  if (NTransOps.strmin(POOL(lake), (Handle **)&newt->h,
+		       (Ref **)(void *)(&newt->tm)) == 0) {
+    return Lnil;
+  } else {
+    if(newt->h && !newt->h->permanent) {
+      HandleDelete(newt->h);
+      newt->h = NULL;
+    }
+    return LNew( LTRANSFORMN, &newt );
   }
-  return Lnil;
 }
 
 LType LTransformNp = {
@@ -580,16 +600,102 @@ LType LTransformNp = {
   tmnpull,
   tmnparse,
   LTypeMagic
-  };
+};
 
+/************************************************************************
+ * IMAGE LISP OBJECT						        *
+ ************************************************************************/
+
+static ImgStruct *imgcopy(ImgStruct *old)
+{
+  ImgStruct *newi;
+
+  NEW(Img, newi);
+
+  if(old) *newi = *old;
+  else newi->img = NULL, newi->h = NULL;
+  if (newi->img) RefIncr((Ref*)(newi->img));
+  if (newi->h) RefIncr((Ref*)newi->h);
+  return newi;
+}
+
+static bool imgfromobj(LObject *obj, ImgStruct **x)
+{
+  if (obj->type != LIMAGE) return 0;
+  *x = LIMAGEVAL(obj);
+  return 1;
+}
+
+static LObject *img2obj( ImgStruct **x )
+{
+  ImgStruct *copy = imgcopy(*x);
+  return LNew( LIMAGE, &copy );
+}
+
+static void imgfree(ImgStruct **x)
+{
+  if (*x) {
+    if ((*x)->img) ImgDelete( (*x)->img );
+    if ((*x)->h) HandleDelete( (*x)->h );
+    DELETE(*x);
+  }
+}
+
+static bool imgmatch(ImgStruct **a, ImgStruct **b)
+{ 
+  if ((*a)->h && ((*a)->h == (*b)->h)) return true;
+  if ((*a)->img && ((*a)->img == (*b)->img)) return true;
+  return false;
+}
+
+static void imgwrite(FILE *fp, ImgStruct **x)
+{
+  ImgFSave((*x)->img, fp, "lisp output stream");
+}
+
+static void imgpull(va_list *a_list, ImgStruct **x)
+{
+  *x = va_arg(*a_list, ImgStruct*);
+}
+
+static LObject *imgparse(Lake *lake)
+{
+  ImgStruct *newi;
+
+  NEW(Img, newi);
+  
+  newi->h = NULL;
+  newi->img = NULL;
+  if (ImageOps.strmin(POOL(lake), (Handle **)&newi->h,
+		      (Ref **)(void *)(&newi->img)) == 0) {
+    return Lnil;
+  } else {
+    if(newi->h && !newi->h->permanent) {
+      HandleDelete(newi->h);
+      newi->h = NULL;
+    }
+    return LNew( LIMAGE, &newi );
+  }
+}
+
+LType LImagep = {
+  "image",
+  sizeof(ImgStruct*),
+  imgfromobj,
+  img2obj,
+  imgfree,
+  imgwrite,
+  imgmatch,
+  imgpull,
+  imgparse,
+  LTypeMagic
+};
 
 /************************************************************************
  * ID LISP OBJECT							*
  ************************************************************************/
 
-int idfromobj(obj, x)
-    LObject *obj;
-    int *x;
+static bool idfromobj(LObject *obj, int *x)
 {
   if (obj->type == LSTRING) {
     *x = drawer_idbyname(LSTRINGVAL(obj));
@@ -600,8 +706,7 @@ int idfromobj(obj, x)
   return 1;
 }
 
-LObject *id2obj(x)
-    int *x;
+static LObject *id2obj(int *x)
 {
   return LNew( LID, x );
 }
@@ -609,20 +714,18 @@ LObject *id2obj(x)
 static void idfree(int *x)
 {}
 
-int idmatch(a, b)
-    int *a,*b;
+static bool idmatch(int *a, int *b)
 {
   return drawer_idmatch(*a,*b);
 }
 
-void idwrite(fp, x)
-    FILE *fp;
-    int *x;
+static void idwrite(FILE *fp, int *x)
 {
-  fprintf(fp, "%s", drawer_id2name(*x));
+  fprintf(fp, "\"%s\"", drawer_id2name(*x));
 }
 
-LObject *idparse(Lake *lake)
+#if 0
+static LObject *idparse(Lake *lake)
 {
   LObject *obj = LSexpr(lake);
   int id;
@@ -638,6 +741,7 @@ LObject *idparse(Lake *lake)
     return Lnil;
   }
 }
+#endif
     
 static void idpull(a_list, x)
     va_list *a_list;
@@ -664,12 +768,10 @@ LType LIdp = {
  * KEYWORD LISP OBJECT							*
  ************************************************************************/
 
-int keywordfromobj(obj, x)
-    LObject *obj;
-    int *x;
+static bool keywordfromobj(LObject *obj, int *x)
 {
   if (obj->type == LSTRING) {
-    *x = (int)fsa_parse(lang_fsa, LSTRINGVAL(obj));
+    *x = (int)(long)fsa_parse(lang_fsa, LSTRINGVAL(obj));
     if (*x == REJECT) return 0;
   } else if (obj->type == LKEYWORD) {
     *x = LKEYWORDVAL(obj);
@@ -677,21 +779,17 @@ int keywordfromobj(obj, x)
   return 1;
 }
 
-LObject *keyword2obj(x)
-    int *x;
+static LObject *keyword2obj(int *x)
 {
   return LNew( LKEYWORD, x );
 }
 
-static int keywordmatch(a, b)
-    int *a,*b;
+static bool keywordmatch(int *a, int *b)
 {
   return *a == *b;
 }
 
-void keywordwrite(fp, x)
-    FILE *fp;
-    int *x;
+static void keywordwrite(FILE *fp, Keyword *x)
 {
   fprintf(fp, "%s", keywordname(*x));
 }
@@ -706,12 +804,13 @@ static void keywordpull(a_list, x)
   *x = va_arg(*a_list, int);
 }
 
-LObject *keywordparse(Lake *lake)
+#if 0
+static LObject *keywordparse(Lake *lake)
 {
   LObject *obj = LSexpr(lake);
   int key;
   if (obj->type == LSTRING) {
-    key  = (int)fsa_parse(lang_fsa, LSTRINGVAL(obj));
+    key  = (int)(long)fsa_parse(lang_fsa, LSTRINGVAL(obj));
     if (key == REJECT) return Lnil;
     OOGLFree(LSTRINGVAL(obj));
     obj->type = LKEYWORD;
@@ -722,6 +821,7 @@ LObject *keywordparse(Lake *lake)
     return Lnil;
   }
 }
+#endif
 
 LType LKeywordp = {
   "keyword",
@@ -742,9 +842,7 @@ LType LKeywordp = {
  * (a "strings" object is a string with possibly embedded spaces)	*
  ************************************************************************/
 
-int stringsfromobj(obj, x)
-    LObject *obj;
-    char * *x;
+static bool stringsfromobj(LObject *obj, char * *x)
 {
   if (obj->type != LSTRING  && obj->type != LSTRINGS) return 0;
   *x = LSTRINGVAL(obj);
@@ -762,7 +860,7 @@ static LObject *stringsparse(Lake *lake)
 
   *VVINDEX(svv, char, 0) = '\0';
   while ( LakeMore(lake,c) ) {
-    tok = fdelimtok( delims, lake->streamin, 0 );
+    tok = iobfdelimtok( delims, lake->streamin, 0 );
     toklen = strlen(tok);
     vvneeds(&svv, strlen(VVEC(svv,char))+toklen+2);
     if (!first) strcat(VVEC(svv,char), " ");
@@ -797,13 +895,43 @@ void lispext_init()
   return;
 }
 
-void define_keyword(char *word, int value)
+void define_keyword(char *word, Keyword value)
 {
-  fsa_install(lang_fsa, word, (void*)value);
+  if (value+1 > n_keywords) {
+    char **newwords = OOGLNewNE(char *, value+1, "New keyword list");
+    memset(newwords, 0, sizeof(char *)*(value+1));
+    memcpy(newwords, keyword_names, n_keywords*sizeof(char *));
+    OOGLFree(keyword_names);
+    keyword_names = newwords;
+    keyword_names[value] = word;
+    n_keywords = value+1;
+  } else if (keyword_names[value] == NULL) {
+    /* We allow aliases, but the name-list just picks up the first
+     * name.
+     */
+    keyword_names[value] = word;
+  }
+  fsa_install(lang_fsa, word, (void*)(long)value);
 }
 
 /* returns < 0 if asked to parse something that isn't a keyword. */
-int parse_keyword(char *word)
+Keyword parse_keyword(char *word)
 {
-  return (int)fsa_parse(lang_fsa, word);
+  return (Keyword)(long)fsa_parse(lang_fsa, word);
 }
+
+char *keywordname(Keyword keyword)
+{
+  if (keyword < n_keywords && keyword_names[keyword] != NULL) {
+    return keyword_names[keyword];
+  } else {
+    return "???";
+  }
+}
+
+/*
+ * Local Variables: ***
+ * mode: c ***
+ * c-basic-offset: 2 ***
+ * End: ***
+ */
