@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 #include "mibload.h"
 #include "../common/drawer.h"
@@ -48,6 +50,7 @@ static void file_menu_callbacks(Widget, char *);
 static void edit_menu_callbacks(Widget, char *);
 static void inspect_menu_callbacks(Widget, XtPointer);
 static void motion_menu_callbacks(Widget, char *);
+static void help_menu_callbacks(Widget, XtPointer);
 static void choose_space(Widget, int);
 static void BuildBrowserMenu();
 static int id2menuindex(int);
@@ -97,11 +100,10 @@ static MenuItem inspect_menu[] = {
         inspect_menu_callbacks, (XtPointer)P_CAMERA, (MenuItem *)NULL},
   { "Commands       ", &xmPushButtonGadgetClass, 'O', "Ctrl<Key>P", "[PC]",
         inspect_menu_callbacks, (XtPointer)P_COMMANDS, (MenuItem *)NULL},
-  { "Credits        ", &xmPushButtonGadgetClass, 'r', "Ctrl<Key>R", "[PA]",
-        inspect_menu_callbacks, (XtPointer)P_CREDITS, (MenuItem *)NULL},
   { NULL, NULL }
 };
 
+#if 0
 static MenuItem space_menu[] = {
   { "Euclidean",   &xmPushButtonGadgetClass, 'E', NULL, "[me]",
 	choose_space, (XtPointer)EUCLIDEAN, (MenuItem *)NULL},
@@ -111,6 +113,7 @@ static MenuItem space_menu[] = {
 	choose_space, (XtPointer)SPHERICAL, (MenuItem *)NULL},
   { NULL, NULL }
 };
+#endif
 
 static MenuItem motion_menu[] = {
   { "Tools", &xmPushButtonGadgetClass, 'T', "Ctrl<Key>T", "[Pt]",
@@ -126,6 +129,18 @@ static MenuItem motion_menu[] = {
   { NULL, NULL }
 };
 
+static MenuItem help_menu[] = {
+  { "About", &xmPushButtonGadgetClass, 'a', "Alt<Key>A", "[PA]",
+    help_menu_callbacks, (XtPointer)P_CREDITS, (MenuItem *)NULL },
+  { "Manual (PDF)", &xmPushButtonGadgetClass, 'h', "Alt<Key>P", "[PP]",
+    help_menu_callbacks, "p", (MenuItem *)NULL },
+  { "Manual (HTML)", &xmPushButtonGadgetClass, 'h', "Alt<Key>H", "[PH]",
+    help_menu_callbacks, "h", (MenuItem *)NULL },
+  { "Credits", &xmPushButtonGadgetClass, 'A', "Alt<Key>A", "[PA]",
+    help_menu_callbacks, (XtPointer)P_CREDITS, (MenuItem *)NULL },
+  { NULL, NULL }
+};
+
 void ui_refresh_mainpanel(int id)
 {
   int newspace, i;
@@ -157,7 +172,7 @@ void ui_refresh_mainpanel(int id)
 
 static void motion_menu_callbacks(Widget widget, char *text)
 {
-  int val;
+  DrawerKeyword val = DRAWER_NOKEYWORD;
 
   switch (text[0])
   {
@@ -180,7 +195,113 @@ static void motion_menu_callbacks(Widget widget, char *text)
   }
 
   drawer_int(GEOMID(uistate.targetgeom), val,
-		XmToggleButtonGadgetGetState(widget));
+	     XmToggleButtonGadgetGetState(widget));
+}
+
+/* make the motion menu state accessible through GCL commands */
+LDEFINE(ui_motion, LVOID,
+	"(ui-motion {inertia|constrain|own-coordinates} {on|off|yes|no})\n"
+	"Enable or disable certain properties of mouse-controlled motion. "
+	"The purpose of this command is to give access to the toggles of the "
+	"`motion'-menu of the main panels through GCL commands. "
+	"\n\n"
+	"`inertia': "
+	"Normally, moving objects have inertia: if the mouse is still moving "
+	"when the button is released, the selected object continues to move. "
+	"When `inertia' is off, objects cease to move as soon as you release "
+	"the mouse."
+	"\n\n"
+	"`constrain': "
+	"It's sometimes handy to move an object in a direction aligned with a "
+	"coordinate axis: exactly horizontally or vertically. Calling "
+	"`(ui-motion constrain on)' changes the interpretation of mouse "
+	"motions to allow this; approximately-horizontal or "
+	"approximately-vertical mouse dragging becomes exactly horizontal "
+	"or vertical motion. Note that the motion is still along the "
+	"X or Y axes of the camera in which you move the mouse, not "
+	"necessarily the object's own coordinate system."
+	"\n\n"
+	"`own-coordinates': "
+	"It's sometimes handy to move objects with respect to the coordinate "
+	"system where they were defined, rather than with respect to some "
+	"camera's view.  When `(ui-motion own-coordinates on) has been called, "
+	"all motions are interpreted that way: dragging the mouse rightward "
+	"in translate mode moves the object in its own +X direction, and so "
+	"on.  May be especially useful in conjunction with the "
+	"(ui-motion constrain on) command.")
+{
+  Keyword enabled = NO_KEYWORD, flagkw = NO_KEYWORD;
+  DrawerKeyword motionflag = DRAWER_NOKEYWORD;
+  bool onoff;
+  int btnidx = -1;
+  
+  LDECLARE(("ui-motion", LBEGIN,
+	    LKEYWORD, &flagkw,
+	    LKEYWORD, &enabled,
+	    LEND));
+
+  switch (flagkw) {
+  case MOTION_INERTIA_KEYWORD:
+    motionflag = DRAWER_INERTIA;
+    btnidx = 0;
+    break;
+  case MOTION_CONSTRAIN_KEYWORD:
+    motionflag = DRAWER_CONSTRAIN;
+    btnidx = 1;
+    break;
+  case MOTION_OWN_COORDS_KEYWORD:
+    motionflag = DRAWER_OWNMOTION;
+    btnidx = 2;
+    break;
+  default:
+    OOGLError(1,
+	      "Wrong keyword, expected one out of "
+	      "{\"%s\", \"%s\", \"%s\"}\n",
+	      keywordname(MOTION_INERTIA_KEYWORD),
+	      keywordname(MOTION_CONSTRAIN_KEYWORD),
+	      keywordname(MOTION_OWN_COORDS_KEYWORD));
+    break;
+    return Lnil;
+  }
+
+  switch (enabled) {
+  case YES_KEYWORD:
+  case ON_KEYWORD:
+    onoff = true;
+    break;
+  case NO_KEYWORD:
+  case OFF_KEYWORD:
+    onoff = false;
+    break;
+  default:
+    OOGLError(1,
+	      "Wrong keyword, expected one out of "
+	      "{\"%s\", \"%s\", \"%s\", \"%s\"}\n",
+	      keywordname(YES_KEYWORD),
+	      keywordname(ON_KEYWORD),
+	      keywordname(NO_KEYWORD),
+	      keywordname(OFF_KEYWORD));
+    return Lnil;
+  }
+
+  drawer_int(GEOMID(uistate.targetgeom), motionflag, onoff);
+  
+  XtVaSetValues(MotionToggle[btnidx], XmNset, onoff ? True : False, NULL);
+
+  return Lt;
+}
+
+char htmlbrowser[PATH_MAX] = { '\0', }, pdfviewer[PATH_MAX] = { '\0', };
+
+static void help_menu_callbacks(Widget widget, XtPointer ptr)
+{
+  if ((int)(long)ptr == P_CREDITS) {
+    ui_showpanel(P_CREDITS, 1);
+  } else if (*(char *)ptr == 'h') {
+    ui_manual_browser("html");
+  } else { /* if (*(char *)ptr == 'p') { */
+    ui_manual_browser("pdf");
+  }
 }
 
 static void choose_space(Widget w, int item)
@@ -219,8 +340,8 @@ void ui_load_mainpanel()
 	     mainform,
 	     cascade,
 	     space,
-	     inspect,
-	     motion,
+	  /*inspect,*/
+             help,
 	     title,
 	     HideButton;
 
@@ -228,7 +349,6 @@ void ui_load_mainpanel()
   static char gvtitle[100];
   XmString    GVtitle;
   mib_Widget *mainload;
-  Atom       AProtocol;
   XmString   str[3];
   XFontStruct *font;
   XmFontList   fontlist;
@@ -236,14 +356,14 @@ void ui_load_mainpanel()
 
 /*****************************************************************************/
 
-  shell = ui_make_panel_and_form(Geomview, Root, True, NULL);
+  shell = ui_make_panel_and_form(Geomview, Root, True, True, NULL);
 
   XtVaSetValues(shell, XmNtitle, "Geomview - The Geometry Center", NULL);
 
 /*****************************************************************************/
 
   mainwindow = XtCreateManagedWidget("Geomview", xmMainWindowWidgetClass,
-		  shell, args, 0);
+				     shell, args, 0);
 
 /*****************************************************************************/
 
@@ -260,10 +380,10 @@ void ui_load_mainpanel()
 
   str[0] = XmStringCreateSimple("Space");
   cascade = XtVaCreateManagedWidget("button_3",	/* lesstif demands "button_<N>" */
-	xmCascadeButtonGadgetClass, mainmenu,
-	XmNlabelString, str[0],
-	XmNmnemonic,    'S',
-	NULL);
+				    xmCascadeButtonGadgetClass, mainmenu,
+				    XmNlabelString, str[0],
+				    XmNmnemonic, (XID)'S',
+				    NULL);
   XmStringFree(str[0]);
 
   str[0] = XmStringCreateSimple("Euclidean    [me]");
@@ -301,7 +421,7 @@ void ui_load_mainpanel()
 
 /*****************************************************************************/
 
-  motion = BuildMenu(mainmenu, XmMENU_PULLDOWN, "Motion", 'M', motion_menu);
+  BuildMenu(mainmenu, XmMENU_PULLDOWN, "Motion", 'M', motion_menu);
 
   for (n = 0; n < 3; n++)
   {
@@ -309,6 +429,10 @@ void ui_load_mainpanel()
     MotionToggle[n] = (Widget) motion_menu[n+2].subitems;
   }
 
+  help = BuildMenu(mainmenu, XmMENU_PULLDOWN, "Help", 'H', help_menu);
+
+  XtVaSetValues(mainmenu, XmNmenuHelpWidget, help, NULL);
+
   XtManageChild(mainmenu);
 
 /*****************************************************************************/
@@ -322,9 +446,12 @@ void ui_load_mainpanel()
   mainform = XmCreateForm(mainwindow, "Geomview", args, n);
 /*mainload = mib_load_interface(mainform, "interface/MainPanel.mib",
 		MI_FROMFILE);*/
-  mainload = mib_load_interface(mainform, Root,
-		MI_FROMSTRING);
- 
+  mainload = mib_load_interface(mainform, Root, MI_FROMSTRING);
+
+  /* Prevent automatic resizing; otherwise the main widget changes
+     everytime we start or stop a new module. */
+  XtVaSetValues(mainload->me, XmNresizePolicy, XmRESIZE_NONE, NULL);
+
   XtManageChild(mainform);
   XtInstallAccelerators(mainmenu, mainform);
 
@@ -335,12 +462,15 @@ void ui_load_mainpanel()
   EmodList   = mib_find_name(mainload, "EmoduleList")->me;
   KeyText    = mib_find_name(mainload, "KeyText")->me;
 
+  XtVaSetValues(EmodList, XmNselectionPolicy, XmSINGLE_SELECT, NULL);
+  XtVaSetValues(ObjectList, XmNselectionPolicy, XmSINGLE_SELECT, NULL);
+
   XtAddCallback(HideButton, XmNactivateCallback, (XtCallbackProc)ui_hide,
 		(XtPointer)Geomview);
 
-  XtAddCallback(ObjectList, XmNbrowseSelectionCallback,
+  XtAddCallback(ObjectList, XmNsingleSelectionCallback,
 		(XtCallbackProc)select_object, (XtPointer)NULL);
-  XtAddCallback(EmodList, XmNbrowseSelectionCallback,
+  XtAddCallback(EmodList, XmNsingleSelectionCallback,
 		(XtCallbackProc)select_module, (XtPointer)NULL);
 
   XtVaSetValues(KeyText, XmNeditable, False,
@@ -420,7 +550,7 @@ static void edit_menu_callbacks(Widget widget, char *text)
 
 static void inspect_menu_callbacks(Widget widget, XtPointer code)
 {
-  ui_showpanel((int)code, 1);
+  ui_showpanel((int)(long)code, 1);
 }
 
 /*****************************************************************************/
@@ -485,7 +615,6 @@ ui_keyboard(int ch)
 {
   static char kybd[12];
   static int nextc = 0;
-  XmString str;
   if(ch <= 0) {
     ui_keyboard(']');
     if(ch < 0) ui_keyboard('?');
@@ -538,9 +667,9 @@ static int id2menuindex(int id)
 
 static XmString *BuildObjectMenuList( int **menuindex, int *count )
 {
-  register int i;
+  int i;
   int n, camcount;
-  register DObject *obj, **objs;
+  DObject *obj, **objs;
   char buf[150];
   XmString *menulist;
 
@@ -595,7 +724,6 @@ static void
 BuildBrowserMenu()
 {
   XmString *menulist;
-  register int i;
 
   XmListDeleteAllItems(ObjectList);
 
@@ -641,3 +769,10 @@ static void select_module(Widget w, XtPointer data, XmListCallbackStruct *cbs)
 }
 
 /*****************************************************************************/
+
+/*
+ * Local Variables: ***
+ * mode: c ***
+ * c-basic-offset: 2 ***
+ * End: ***
+ */
