@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 #include "mg.h"
 #include "../common/drawer.h"
@@ -43,7 +45,7 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include "X11/IntrinsicP.h" /* for XtMoveWidget */
 
 #include "mibload.h"
@@ -175,13 +177,28 @@ char **argv;
     "gv*Save*fontList:    -adobe-helvetica-medium-r-normal--12-120-75-75-p-67-iso8859-1",
     "Commands*fontList:    -adobe-helvetica-medium-r-normal--12-120-75-75-p-67-iso8859-1",
     "Tools*fontList:    -adobe-helvetica-medium-r-normal--12-120-75-75-p-67-iso8859-1",
+
+    "*XmText*blinkRate:	0",
+    /* Encourage old (???) versions of Motif to interpret our keyboard
+     * shortcuts.  It works in some situations, depending on just what
+     * the cursor overlies.
+     */
+    "*XmForm*accelerators:	<KeyPress>: GVKey()",
+    "*XmFrame*translations: #override <KeyPress>: GVKey()",
+    "Geomview*KeyText*translations: #override <KeyPress>: GVKey()",
+    "Geomview*XmList*translations: #override <KeyPress>: GVKey()",
+    "Appearance*XmList*translations: #override <KeyPress>: GVKey()",
+    "Cameras*XmList*translations: #override <KeyPress>: GVKey()",
+    "Lights*XmList*translations: #override <KeyPress>: GVKey()",
+    "Credits*XmList*translations: #override <KeyPress>: GVKey()",
+
     NULL
   };
 
   static XtActionsRec actions[1] = { { "GVKey", key_action } };
   static String keyaccels = { "<KeyPress>: GVKey()" };
   
-
+#if 0
   static char *argvblah[] = { "Geomview",
 	/* We turn off blinking text cursors for two reasons:
 	 * - Motif grabs the main loop if the cursor sits in a text box
@@ -209,7 +226,7 @@ char **argv;
     "-xrm", "",
   };
   int argcblah = COUNT(argvblah);
-
+#endif
 
 /* Code for TopLevel */
 /*****************************************************************************/
@@ -218,12 +235,23 @@ char **argv;
 
   set_x_application_name();
 
+#if 1
+  TopLevel = XtAppInitialize(&App,
+			     x_application_name,
+			     NULL,
+			     0,
+			     (int *)&argc,
+			     argv,
+			     fallbacks,
+			     args,
+			     n);
+#else
   if(getenv("GV_NO_X_MAGIC")) {
     TopLevel = XtAppInitialize(&App,
 			       x_application_name,
 			       NULL,
 			       0,
-			       &argc,
+			       (int *)&argc,
 			       argv,
 			       fallbacks,
 			       args,
@@ -240,6 +268,7 @@ char **argv;
 			       args,
 			       n);
   }
+#endif
 
 #if 0
   if(getenv("GVXRM")) argvblah[COUNT(argvblah)-1] = getenv("GVXRM");
@@ -257,7 +286,7 @@ char **argv;
   ui_find_visual();
 
   geomicon = XCreatePixmapFromBitmapData(dpy, DefaultRootWindow(dpy),
-		oogl_bits, oogl_width, oogl_height,
+		(char *)oogl_bits, oogl_width, oogl_height,
 		WhitePixel(dpy, DefaultScreen(dpy)),
 		BlackPixel(dpy, DefaultScreen(dpy)),
 		DefaultDepth(dpy, DefaultScreen(dpy)));
@@ -305,7 +334,37 @@ void ui_mousefocus(int index)
   if (uistate.targetcam == INDEXOF(FOCUSID)) {
     ui_select(CAMID(uistate.mousefocus));
   }
+}
 
+LDEFINE(ui_cam_focus, LVOID,
+	"(ui-cam-focus [focus-change|mouse-cross])\n"
+	"Set the focus policy for the camera windows. The default is "
+	"\"mouse-cross\": a camera is made the active camera (for interactive "
+	"mouse events) when the mouse cursor crosses the window. Because "
+	"this means it can become "
+	"complicated to activate a specific camera (in the context of multiple "
+	"camera windows) there is also the option to only change the camera "
+	"focus when the window-manager decides to give it the focus for input "
+	"events. So, after specifying \"focus-change\" it depends on the "
+	"focus-change configuration of your window-manager when a camera "
+	"becomes the active camera for mouse-interaction.")
+{
+  Keyword policy = NO_KEYWORD;
+
+  LDECLARE(("ui-cam-focus", LBEGIN,
+	    LKEYWORD, &policy,
+	    LEND));
+  
+  if (policy != FOCUS_CHANGE_KEYWORD && policy != MOUSE_CROSS_KEYWORD) {
+    OOGLError(1,
+	      "Wrong keyword, expected either \"%s\" or \"%s\"\n",
+	      keywordname(FOCUS_CHANGE_KEYWORD),
+	      keywordname(MOUSE_CROSS_KEYWORD));
+    return Lnil;
+  }
+  set_ui_wm_focus(policy == FOCUS_CHANGE_KEYWORD);
+  
+  return Lt;
 }
 
 /*****************************************************************************/
@@ -418,20 +477,20 @@ LDEFINE(ui_target, LVOID,
         (geom or camera).  The default is \"yes\".  This command may\n\
         result in a change of motion modes based on target choice.")
 {
-  DObject *obj;
   int id, newtype;
-  int immediate=YES_KEYWORD;
+  Keyword immediate_kw = YES_KEYWORD;
+  bool immediate;
 
   LDECLARE(("ui-target", LBEGIN,
             LID, &id,
             LOPTIONAL,
-            LKEYWORD, &immediate,
+            LKEYWORD, &immediate_kw,
             LEND));
-  immediate = boolval("ui-target", immediate);
+  immediate = boolval("ui-target", immediate_kw);
   newtype = TYPEOF(id);
 
   if (id == uistate.targetid ) return Lt;
-  if ( !(obj = drawer_get_object(id))
+  if (drawer_get_object(id) == NULL
       || (id == TARGETID) || (id == CENTERID)
       || (id == SELF) || (id == UNIVERSE) || (id == PRIMITIVE)) {
     return Lt;
@@ -466,7 +525,7 @@ LDEFINE(ui_target, LVOID,
 }
 
 LDEFINE(ui_center, LVOID,
-       "(ui-center      ID)\n\
+       "(ui-center ID)\n\
         Set the center for user interface (i.e. mouse) controlled\n\
         motions to object ID.")
 {
@@ -484,16 +543,49 @@ LDEFINE(ui_center, LVOID,
   return Lt;
 }
 
+LDEFINE(ui_center_origin, LVOID,
+       "(ui-center-origin {origin|bbox-center})\n"
+	"Set the origin of the coordinate system of the \"CENTER\" object"
+	"for user interface (i.e. mouse) controlled motions."
+	"The keyword \"origin\" means to use the origin of the coordinate"
+	"system of the currently selected object, while \"bbox-center\" "
+	"means to use the center of the bouding box of the current object. "
+	"The keyword \"bbox-center\" makes no sense if the geometry is "
+	"non-Euclidean. Using either \"bbox-center\" or \"origin\" does "
+	"not make a difference if the center object is not a \"geometry\", "
+	"e.g. if it is a camera. Same holds if the World is the currently "
+	"selected object.")
+{
+  Keyword origin = NO_KEYWORD;
+
+  LDECLARE(("ui-center-origin", LBEGIN,
+            LKEYWORD, &origin,
+            LEND));
+
+  if (origin != BBOX_CENTER_KEYWORD && origin != ORIGIN_KEYWORD) {
+    OOGLError(1,
+	      "Wrong keyword, expected either \"%s\" or \"%s\"\n",
+	      keywordname(BBOX_CENTER_KEYWORD),
+	      keywordname(ORIGIN_KEYWORD));
+    return Lnil;
+  }
+  
+  set_ui_center_origin(origin == BBOX_CENTER_KEYWORD);
+  ui_tool_center_origin_set(origin == BBOX_CENTER_KEYWORD);
+
+  return Lt;
+}
+
 /*****************************************************************************/
 
 void ui_pickcolor(int val)
 {
   DView      *dv;
-  Color      *old;
-  char       *name;
+  Color      *old = NULL;
+  char       *name = NULL;
   Appearance *ap;
   DGeom      *dg;
-  int         index;
+  int         index = 0;
 
 
   ap = drawer_get_ap(GEOMID(uistate.targetgeom));
@@ -511,7 +603,7 @@ void ui_pickcolor(int val)
 
     case DRAWER_DIFFUSE:
 	name = "Faces"; index = FACEINDEX;
-	old = (Color *)&ap->mat->diffuse;
+	old = (Color *)(void *)&ap->mat->diffuse;
 	break;
 
     case DRAWER_EDGECOLOR:
@@ -623,8 +715,9 @@ void ui_init()
 
 /*****************************************************************************/
 
-Widget ui_make_panel_and_form(char *title, char *rootstr, Boolean wantkeys,
-				Widget *formp)
+Widget ui_make_panel_and_form(char *title, char *rootstr,
+			      Boolean wantkeys, Boolean resizable,
+			      Widget *formp)
 {
   Arg args[20];
   int n;
@@ -635,7 +728,7 @@ Widget ui_make_panel_and_form(char *title, char *rootstr, Boolean wantkeys,
   XtSetArg(args[n], XmNtitle, title); n++;
   XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING);n++;
   XtSetArg(args[n], XmNkeyboardFocusPolicy, XmPOINTER); n++;
-  XtSetArg(args[n], XmNallowShellResize, True); n++;
+  XtSetArg(args[n], XmNallowShellResize, resizable); n++;
 
   if(gvvisual->class & 1) {	/* If PseudoColor or other dynamic visual */
     XtSetArg(args[n], XmNvisual, gvvisual); n++;
@@ -652,8 +745,9 @@ Widget ui_make_panel_and_form(char *title, char *rootstr, Boolean wantkeys,
   XtSetArg(args[n], XmNmwmFunctions, MWM_FUNC_ALL); n++;
 
   set_x_application_name();
-  shell = XtAppCreateShell(title, x_application_name, topLevelShellWidgetClass, dpy,
-                                args, n);
+  shell = XtAppCreateShell(title, x_application_name,
+			   topLevelShellWidgetClass, dpy,
+			   args, n);
 
   /* Close when requested by WM */
   AProtocol = XmInternAtom(dpy, "WM_DELETE_WINDOW", False);
@@ -663,7 +757,7 @@ Widget ui_make_panel_and_form(char *title, char *rootstr, Boolean wantkeys,
   if(formp != NULL) {
     n = 0;
     XtSetArg(args[n], XmNrubberPositioning, True); n++;
-    XtSetArg(args[n], XmNresizable, True); n++;
+    XtSetArg(args[n], XmNresizable, resizable); n++;
     if(wantkeys) {
 	XtSetArg(args[n], XmNaccelerators, KeyAccels); n++;
     }
@@ -718,7 +812,8 @@ LDEFINE(ui_panel, LVOID,
         positioning.")
 {
   char *panelname;
-  int index, on;
+  int index;
+  Keyword on = NO_KEYWORD;
   struct panel *p;
   WindowStruct *ws=NULL;
   WnPosition wp;
@@ -970,3 +1065,9 @@ void ui_set_itext(Widget w, int val)
 }
 
 /*****************************************************************************/
+
+/*
+ * Local Variables: ***
+ * c-basic-offset: 2 ***
+ * End:
+ */
