@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 /*
  * anytopl.c
@@ -141,7 +143,7 @@ static HPoint3 *hpoints(PLData *PL)
 	if(PL->Tn) {
 	    hp.dim = v->hdim;
 	    hp.v = v->ndp;
-	    HPtNTransformComponents(&hp, PL->Tn, 4, xyzw, (float *)p);
+	    HPtNTransformComponents(PL->Tn, &hp, xyzw, p);
 	} else {
 	    /* Project to 3-space: just take first 3 and final components. */
 	    memcpy(p, v->ndp, 3*sizeof(float));
@@ -175,7 +177,7 @@ static float *ndpoints(PLData *PL)
 	    hp.dim = 4;
 	    hp.v = &v->p.x;
 	}
-	HPtNTransformComponents(&hp, PL->Tn, 4, comp, p);
+	HPtNTransformComponents(PL->Tn, &hp, comp, (HPoint3 *)p);
     } else {
 	if(v->hdim == hdim) {
 	    memcpy(p, v->ndp, hdim*sizeof(float));
@@ -258,7 +260,7 @@ Geom *PLDataToGeom(PLData *PL, int want, int unwant)
 		CR_POLYCOLOR, pc,
 		CR_END);
     if(want & PL_N)
-	PolyListComputeNormals((PolyList *)pl);
+	PolyListComputeNormals((PolyList *)pl, PL_HASPN|PL_HASPFL);
     return pl;
   }
 }
@@ -298,18 +300,18 @@ PLDataDelete(PLData *PL)
   }
 }
 
-static void *beziertoPL(int sel, Bezier *bez, va_list args) {
-  PLData *PL = va_arg(args, PLData *);
+static void *beziertoPL(int sel, Bezier *bez, va_list *args) {
+  PLData *PL = va_arg(*args, PLData *);
   if(PL->ap && (PL->ap->flag & APF_DICE))
     BezierDice( bez, PL->ap->dice[0], PL->ap->dice[1] );
-  if(bez->mesh == NULL || bez->flag & BEZ_REMESH)
+  if(bez->mesh == NULL || bez->geomflags & BEZ_REMESH)
     BezierReDice(bez);
   return GeomCall(sel, (Geom *)bez->mesh, PL);
 }
 
-static void *discgrptoPL(int sel, DiscGrp *dg, va_list args) {
+static void *discgrptoPL(int sel, DiscGrp *dg, va_list *args) {
   Geom *geom = NULL;
-  PLData *PL = va_arg(args, PLData *);
+  PLData *PL = va_arg(*args, PLData *);
   int i;
 
   if(dg->big_list == NULL || dg->big_list->el_list == NULL)
@@ -326,10 +328,10 @@ static void *discgrptoPL(int sel, DiscGrp *dg, va_list args) {
   return PL;
 }
 
-static void *insttoPL(int sel, Inst *inst, va_list args) {
+static void *insttoPL(int sel, Inst *inst, va_list *args) {
   GeomIter *it;
   Transform T;
-  PLData *PL = va_arg(args, PLData *);
+  PLData *PL = va_arg(*args, PLData *);
 
   it = GeomIterate((Geom *)inst, DEEP);
   while(NextTransform(it, T)) {
@@ -338,9 +340,9 @@ static void *insttoPL(int sel, Inst *inst, va_list args) {
   return PL;
 }
 
-static void *listtoPL(int sel, List *list, va_list args) {
+static void *listtoPL(int sel, List *list, va_list *args) {
   List *l;
-  PLData *PL = va_arg(args, PLData *);
+  PLData *PL = va_arg(*args, PLData *);
 
   for(l = list; l != NULL; l = l->cdr) {
     GeomCall(sel, l->car, PL);
@@ -388,20 +390,21 @@ static void putmesh(PLData *PL, int base, int nu, int nv, int uwrap, int vwrap)
 }
 
 
-static void *meshtoPL(int sel, Mesh *m, va_list args) {
+static void *meshtoPL(int sel, Mesh *m, va_list *args) {
   int base;
-  PLData *PL = va_arg(args, PLData *);
+  PLData *PL = va_arg(*args, PLData *);
 
   base = PLaddverts(PL, m->nu*m->nv, m->p, m->c, m->n);
-  putmesh(PL, base, m->nu, m->nv, m->flag & MESH_UWRAP, m->flag & MESH_VWRAP);
+  putmesh(PL, base, m->nu, m->nv,
+	  m->geomflags & MESH_UWRAP, m->geomflags & MESH_VWRAP);
   return PL;
 }
 
-static void *ndmeshtoPL(int sel, NDMesh *ndm, va_list args) {
+static void *ndmeshtoPL(int sel, NDMesh *ndm, va_list *args) {
   int nu = ndm->mdim[0];
   int nv = ndm->meshd > 1 ? ndm->mdim[1] : 1;
   int u, v, base = -1;
-  PLData *PL = va_arg(args, PLData *);
+  PLData *PL = va_arg(*args, PLData *);
   HPointN **pp = ndm->p;
   ColorA *c = ndm->c;
 
@@ -419,10 +422,11 @@ static void *ndmeshtoPL(int sel, NDMesh *ndm, va_list args) {
 }
 
 
-static void *npolylisttoPL(int sel, NPolyList *npl, va_list args) {
-  PLData *PL = va_arg(args, PLData *);
+static void *npolylisttoPL(int sel, NPolyList *npl, va_list *args)
+{
+  PLData *PL = va_arg(*args, PLData *);
   int base;
-  NPoly *p;
+  Poly *p;
   int i, vi;
 #define VMAX 100
   int face[VMAX];
@@ -436,18 +440,18 @@ static void *npolylisttoPL(int sel, NPolyList *npl, va_list args) {
 	vip = OOGLNewNE(int, p->n_vertices, "npolylist face");
 
     for(vi = 0; vi < p->n_vertices; vi++)
-	vip[vi] = base + npl->vi[vi + p->vi0];
+      vip[vi] = base + npl->vi[vi + npl->pv[i]];
 
     PLaddface(PL, p->n_vertices, vip,
-			npl->flags & PL_HASPCOL ? &p->pcol : NULL);
+			npl->geomflags & PL_HASPCOL ? &p->pcol : NULL);
     if(p->n_vertices > VMAX)
 	OOGLFree(vip);
   }
   return PL;
 }
 
-static void *polylisttoPL(int sel, PolyList *pl, va_list args) {
-  PLData *PL = va_arg(args, PLData *);
+static void *polylisttoPL(int sel, PolyList *pl, va_list *args) {
+  PLData *PL = va_arg(*args, PLData *);
   Poly *p;
   Vertex *v = pl->vl;
   int base, i, vi;
@@ -456,8 +460,8 @@ static void *polylisttoPL(int sel, PolyList *pl, va_list args) {
   base = PLnextvert(PL);
   for(i = 0; i < pl->n_verts; i++, v++) {
     PLaddverts(PL, 1, &v->pt,
-		pl->flags & PL_HASVCOL ? &v->vcol : NULL,
-		pl->flags & PL_HASVN ? &v->vn : NULL);
+		pl->geomflags & PL_HASVCOL ? &v->vcol : NULL,
+		pl->geomflags & PL_HASVN ? &v->vn : NULL);
   }
 
   p = pl->p;
@@ -467,15 +471,16 @@ static void *polylisttoPL(int sel, PolyList *pl, va_list args) {
 	vip = OOGLNewNE(int, p->n_vertices, "polylist face");
     for(vi = 0; vi < p->n_vertices; vi++)
 	vip[vi] = base + (p->v[vi] - pl->vl);
-    PLaddface(PL, p->n_vertices, vip, pl->flags & PL_HASPCOL ? &p->pcol : NULL);
+    PLaddface(PL, p->n_vertices, vip,
+	      pl->geomflags & PL_HASPCOL ? &p->pcol : NULL);
     if(p->n_vertices > VMAX)
 	OOGLFree(vip);
   }
   return PL;
 }
 
-static void *quadtoPL(int sel, Quad *q, va_list args) {
-  PLData *PL = va_arg(args, PLData *);
+static void *quadtoPL(int sel, Quad *q, va_list *args) {
+  PLData *PL = va_arg(*args, PLData *);
   int base, v, i;
 
   base = PLaddverts(PL, q->maxquad*4, *q->p, *q->c, *q->n);
@@ -490,17 +495,17 @@ static void *quadtoPL(int sel, Quad *q, va_list args) {
   return PL;
 }
 
-static void *skeltoPL(int sel, Skel *s, va_list args) {
-  PLData *PL = va_arg(args, PLData *);
+static void *skeltoPL(int sel, Skel *s, va_list *args) {
+  PLData *PL = va_arg(*args, PLData *);
   int base, i;
   int vert, edge[2];
   Skline *l = s->l;
 
   if(PL->ap && (PL->ap->flag & APF_VECTDRAW) == 0)
     return NULL;
-  base = s->dim == 4
+  base = s->pdim == 4
 	? PLaddverts(PL, s->nvert, (HPoint3 *)s->p, NULL, NULL)
-	: PLaddNDverts(PL, s->nvert, s->dim, s->p, NULL);
+	: PLaddNDverts(PL, s->nvert, s->pdim, s->p, NULL);
 
   vvneeds(&PL->vtable, VVCOUNT(PL->vtable) + 2*s->nvi);
 
@@ -525,11 +530,10 @@ static void *skeltoPL(int sel, Skel *s, va_list args) {
   return PL;
 }
 
-static void *vecttoPL(int sel, Vect *v, va_list args) {
-  PLData *PL = va_arg(args, PLData *);
+static void *vecttoPL(int sel, Vect *v, va_list *args) {
+  PLData *PL = va_arg(*args, PLData *);
   int base, i, vno;
   short *vip, *cip;
-  HPoint3 *vp;
   ColorA *cp, *lastc;
 
   if(PL->ap && (PL->ap->flag & APF_VECTDRAW) == 0)
@@ -539,7 +543,6 @@ static void *vecttoPL(int sel, Vect *v, va_list args) {
   vno = base;
   vip = v->vnvert;
   cip = v->vncolor;
-  vp = v->p;
   cp = v->c;
   lastc = (v->ncolor>0) ? cp : NULL;
   for(i = 0, vno = base; i < v->nvec; i++, vip++, cip++) {
@@ -573,7 +576,7 @@ static void *vecttoPL(int sel, Vect *v, va_list args) {
 static int
 PLaddverts(PLData *PL, int nv, HPoint3 *v, ColorA *c, Point3 *n)
 {
-  register Vert *vp;
+  Vert *vp;
   Material *mat;
   int i;
   int base = PLnextvert(PL);
@@ -614,7 +617,7 @@ PLaddverts(PLData *PL, int nv, HPoint3 *v, ColorA *c, Point3 *n)
 static int
 PLaddNDverts(PLData *PL, int nv, int hdim, float *v, ColorA *c)
 {
-  register Vert *vp;
+  Vert *vp;
   Material *mat;
   int i;
   int base = PLnextvert(PL);
@@ -706,10 +709,9 @@ PLaddseg(PLData *PL, int v0, int v1, ColorA *c)
 {
   Face *fp = VVAPPEND(PL->faces, Face);
   Material *mat = PL->ap->mat;
-  int has = PL_HASPC;
   if(mat && (mat->valid & MTF_EDGECOLOR)) {
     if(c == NULL || mat->override & MTF_EDGECOLOR) {
-	*(Color *)&fp->fcolor = mat->edgecolor;
+	    *(Color *)(void *)&fp->fcolor = mat->edgecolor;
 	fp->fcolor.a = mat->diffuse.a;
 	PL->some |= PL_HASPC;
     } else if(c) {
@@ -734,7 +736,6 @@ PLaddvect(PLData *PL, int nv, int verts[], ColorA *c)
   Material *mat;
   ColorA edgec;
   Face *fp;
-  int i;
 
   if(nv <= 0)
     return;
@@ -755,7 +756,7 @@ PLaddvect(PLData *PL, int nv, int verts[], ColorA *c)
     if(mat->override & MTF_EDGECOLOR)
 	c = NULL;
     if((mat->valid & MTF_EDGECOLOR) && c == NULL) {
-	*(Color *)&edgec = mat->edgecolor;
+	    *(Color *)(void *)&edgec = mat->edgecolor;
 	edgec.a = mat->diffuse.a;
 	c = &edgec;
     }
@@ -789,7 +790,7 @@ PLData *AnyGeomToPLData(Geom *g, Transform T, TransformN *Tn, Appearance *pap,
 			PLData *PL)
 {
   Transform Told;
-  TransformN *Tnold;
+  TransformN *Tnold = NULL;
   Appearance *apold = NULL;
 
   if(g == NULL)
