@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 
 /* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */
@@ -32,23 +34,17 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "meshP.h"
 
 Mesh *
-MeshDelete(m)
-Mesh           *m;
+MeshDelete(Mesh *m)
 {
-	if (m)
-	{
-		if (m->p)
-			GeomFree(m->p);
-		if (m->n)
-			GeomFree(m->n);
-		if (m->c)
-			GeomFree(m->c);
-		if (m->u)
-			GeomFree(m->u);
-		if (m->d)
-			GeomFree(m->d);
-		if (m->nd)
-			GeomFree(m->nd);
-	}
-	return NULL;
+  if (m) {
+    if (m->p)
+      GeomFree(m->p);
+    if (m->n)
+      GeomFree(m->n);
+    if (m->c)
+      GeomFree(m->c);
+    if (m->u)
+      GeomFree(m->u);
+  }
+  return NULL;
 }
