@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 
 /* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */
@@ -32,13 +34,12 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include "quadP.h"
 
 Quad *
-QuadDelete(q)
-	Quad *q;
+QuadDelete(Quad *q)
 {
-	if(q) {
-		if(q->p != NULL) GeomFree(q->p);
-		if(q->c != NULL) GeomFree(q->c);
-		if(q->n != NULL) GeomFree(q->n);
-	}
-	return NULL;
+  if(q) {
+    if(q->p != NULL) GeomFree(q->p);
+    if(q->c != NULL) GeomFree(q->c);
+    if(q->n != NULL) GeomFree(q->n);
+  }
+  return NULL;
 }
