@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 
 /* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips,
@@ -32,13 +34,28 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 
 #include "skelP.h"
 
-Skel *
-SkelTransform( Skel *s, Transform T )
+Skel *SkelTransform( Skel *s, Transform T, TransformN *TN )
 {
-  if(s->dim == 4) {
-    HPt3TransformN(T, (HPoint3 *)s->p, (HPoint3 *)s->p, s->nvert);
-  } else {
+#if 0
+  if (T && T != TM_IDENTITY) {
+
+    if(s->pdim == 4) {
+      HPt3TransformN(T, (HPoint3 *)s->p, (HPoint3 *)s->p, s->nvert);
+    } else if (s->pdim > 4) {
+      HPointN *tmp = HPtNCreate(s->dim, NULL);
+      float *tmp_data = tmp->v;
+      int i;
+      
+      for (tmp->v = s->p, i = 0; i < s->nvert; i++, tmp->v += s->dim) {
+	HPtNTransform(T, tmp, tmp);
+      }
+
+      tmp->v = tmp_data;
+      HPtNDelete(tmp);      
+    }
+  } else {   
     OOGLError(1, "Can't SkelTransform() %d-D Skel", s->dim);
   }
-  return(s);
+#endif
+  return s;
 }
