@@ -19,12 +19,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 /* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */
 
@@ -46,8 +48,10 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 # define  Window  int
 #endif
 
+#include "polylistP.h"
+
 mgcontext * mgopengl_ctxcreate(int a1, ...);
-void	    mgopengl_ctxset( int a1, ...  );
+int	    mgopengl_ctxset( int a1, ...  );
 int	    mgopengl_feature( int feature );
 void	    mgopengl_ctxdelete( mgcontext *ctx );
 int	    mgopengl_ctxget( int attr, void* valueptr );
@@ -64,24 +68,27 @@ void	    mgopengl_gettransform( Transform T );
 void	    mgopengl_settransform( Transform T );
 int	    mgopengl_pushappearance( void );
 int	    mgopengl_popappearance( void );
-Appearance *mgopengl_setappearance( Appearance* app, int merge );
-Appearance *mgopengl_getappearance( void );
+const Appearance *mgopengl_setappearance(const Appearance* app, int merge );
 int	    mgopengl_setcamera( Camera* cam );
 int	    mgopengl_setwindow( WnWindow *win, int final );
 mgopenglcontext *mgopengl_newcontext( mgopenglcontext *ctx );
+void mgopengl_taggedappearance(const void *tag);
 
 extern void mgopengl_polygon();
-extern void mgopengl_mesh();
 extern void mgopengl_line();
 extern void mgopengl_polyline();
-extern void mgopengl_polylist();
-extern void mgopengl_quads();
+extern void mgopengl_polylist(int np, Poly *p, int nv, Vertex *v, int plflags);
+extern void mgopengl_mesh(int wrap, int nu, int nv,
+			  HPoint3 *meshP, Point3 *meshN, Point3 *meshNQ,
+			  ColorA *meshC, TxST *meshST,
+			  int mflags);
+extern void mgopengl_quads(int count, HPoint3 *V, Point3 *N, ColorA *C,
+			   int qflags);
 
-
-
-void _mgopengl_ctxset(int a1, va_list *alist);
+int _mgopengl_ctxset(int a1, va_list *alist);
 static void mgopengl_choosewin(void), mgopengl_initwin(void);
 static void mgopengl_setviewport(void);
+static void mgimgfile2cambgimage(void);
 
 #ifndef GLUT
 static GLXContext mgopengl_sharectx(void);
@@ -95,23 +102,33 @@ struct mgfuncs mgopenglfuncs = {
   mgopengl_feature,
   (mgcontext *(*)())mgopengl_ctxcreate,
   mgopengl_ctxdelete,
-  (void (*)())mgopengl_ctxset,
+  (int (*)())mgopengl_ctxset,
   mgopengl_ctxget,
   mgopengl_ctxselect,
   mgopengl_sync,
   mgopengl_worldbegin,
   mgopengl_worldend,
   mgopengl_reshapeviewport,
+  /* geometry transform */
   mgopengl_settransform,
   mgopengl_gettransform,
   mgopengl_identity,
   mgopengl_transform,
   mgopengl_pushtransform,
   mgopengl_poptransform,
+  /* texture transform */
+  mg_settxtransform,
+  mg_gettxtransform,
+  mg_txidentity,
+  mg_txtransform,
+  mg_pushtxtransform,
+  mg_poptxtransform,
+  /* appearance stack */
   mgopengl_pushappearance,
   mgopengl_popappearance,
   mgopengl_setappearance,
-  mgopengl_getappearance,
+  mg_getappearance,
+  /*******************/
   mgopengl_setcamera,
   mgopengl_polygon,
   mgopengl_polylist,
@@ -119,11 +136,15 @@ struct mgfuncs mgopenglfuncs = {
   mgopengl_line,
   mgopengl_polyline,
   mgopengl_quads,
-  mg_bezier
-  };
+  mg_bezier,
+  mgopengl_bsptree,
+  mg_tagappearance,
+  mg_untagappearance,
+  mgopengl_taggedappearance
+};
 
 int
-  mgdevice_OPENGL()
+mgdevice_OPENGL()
 {
   _mgf = mgopenglfuncs;
   if (_mgc != NULL && _mgc->devno != MGD_OPENGL)
@@ -143,11 +164,13 @@ mgopengl_ctxcreate(int a1, ...)
   va_list alist;
 
 
-  _mgc =
-    (mgcontext*)mgopengl_newcontext( OOGLNewE(mgopenglcontext, "mgopengl_ctxcreate") );
+  _mgc = (mgcontext*)mgopengl_newcontext(OOGLNewE(mgopenglcontext,
+						  "mgopengl_ctxcreate"));
 
   va_start(alist, a1);
-  _mgopengl_ctxset(a1, &alist);
+  if (_mgopengl_ctxset(a1, &alist) == -1) {
+    mgopengl_ctxdelete(_mgc);
+  }
   va_end(alist);
   return _mgc;
 }
@@ -157,17 +180,16 @@ mgopengl_ctxcreate(int a1, ...)
  * Description:	internal ctxset routine
  * Args:	a1: first attribute
  *		*alist: rest of attribute-value list
- * Returns:	nothing
+ * Returns:	-1 on error, 0 on success
  * Author:	mbp
  * Date:	Fri Sep 20 11:08:13 1991
  * Notes:	mgopengl_ctxcreate() and mgopengl_ctxset() call this to actually
  *		parse and interpret the attribute list.
  */
-void
+int
 _mgopengl_ctxset(int a1, va_list *alist)
 {
   int attr;
-  WnWindow *owin;
   char **ablock = NULL;
 
 #define NEXT(type) OOGL_VA_ARG(type,alist,ablock)
@@ -215,7 +237,13 @@ _mgopengl_ctxset(int a1, va_list *alist)
     case MG_UNSETOPTIONS: _mgc->opts &= ~NEXT(int); break;
     case MG_SHOW: _mgc->shown = NEXT(int); break;
     case MG_PARENT: _mgc->parent = NEXT(mgcontext*); break;
-    case MG_BACKGROUND: _mgc->background = *NEXT(ColorA*); break;
+    case MG_BACKGROUND: {
+      _mgc->background = *NEXT(ColorA*);
+      if (_mgc->cam) {
+	CamSet(_mgc->cam, CAM_BGCOLOR, &_mgc->background, CAM_END);
+      }
+      break;
+    }
 
     case MG_SHADER: mgopengl_setshader( NEXT(mgshadefunc) ); break;
     case MG_SHADERDATA: _mgc->astk->shaderdata = NEXT(void*); break;
@@ -247,80 +275,65 @@ _mgopengl_ctxset(int a1, va_list *alist)
       }
       break;
 
-    case MG_NDINFO: _mgc->NDinfo = NEXT(void *); break;
-    case MG_NDMAP:  _mgc->NDmap = NEXT(mgmapfunc); break;
+    case MG_NDCTX: _mgc->NDctx = NEXT(mgNDctx *); break;
 
     case MG_WINCHANGE: _mgc->winchange = NEXT(mgwinchfunc); break;
     case MG_WINCHANGEDATA: _mgc->winchangeinfo = NEXT(void *); break;
 
     case MG_ZNUDGE:
-	_mgc->zfnudge = NEXT(double);
-	if(_mgopenglc->born) mgopengl_init_zrange();
-	break;
+      _mgc->zfnudge = NEXT(double);
+      if(_mgopenglc->born) mgopengl_init_zrange();
+      break;
 
     case MG_BITDEPTH:
-	/*ignored*/ NEXT(int);
-	break;
+      /*ignored*/ NEXT(int);
+      break;
 
     case MG_DITHER:
-	_mgopenglc->dither = NEXT(int);
-	break;
+      _mgopenglc->dither = NEXT(int);
+      break;
 
     case MG_DEPTHSORT:
-	/*ignored*/ NEXT(int);
-	break;
+      /*ignored*/ NEXT(int);
+      break;
 
-    /* Open GL-specific */
+      /* Open GL-specific */
     case MG_GLWINID:
-	_mgopenglc->win = NEXT(int);
-	break;
+      _mgopenglc->win = NEXT(int);
+      break;
 
 #ifndef GLUT
     case MG_GLXDISPLAY:
-	_mgopenglc->GLXdisplay = NEXT(void *);
-	break;
+      _mgopenglc->GLXdisplay = NEXT(void *);
+      break;
     case MG_GLXSINGLECTX:
-	_mgopenglc->cam_ctx[SGL] = NEXT(GLXContext);
-	break;
+      _mgopenglc->cam_ctx[SGL] = NEXT(GLXContext);
+      break;
     case MG_GLXDOUBLECTX:
-	_mgopenglc->cam_ctx[DBL] = NEXT(GLXContext);
-	break;
+      _mgopenglc->cam_ctx[DBL] = NEXT(GLXContext);
+      break;
 #endif
 
     case MG_GLXSINGLEWIN:
-	_mgopenglc->winids[SGL] = NEXT(int);
-	break;
+      _mgopenglc->winids[SGL] = NEXT(int);
+      break;
     case MG_GLXDOUBLEWIN:
-	_mgopenglc->winids[DBL] = NEXT(int);
-	break;
-
-    case MG_BGIMAGEFILE:
-	{
-	   char *fname = NEXT(char *);
-	   TxUser *tu = NULL;
-
-	   if(fname != NULL) {
-		/* Need to pad image to 4-channel RGBA, lest
-		 * Indy/Indigo graphics slow down by a factor of ~30!
-		 */
-		Texture *tex = TxCreate(TX_FILE, fname, TX_CHANNELS, 4, TX_END);
-		tu = mg_find_shared_texture(tex, MGD_OPENGL);
-		if(tu == NULL) {
-		    tu = TxAddUser(tex, 0, NULL, mgopengl_txpurge);
-		    tu->ctx = _mgc;
-		}
-	   }
-	   if(_mgopenglc->bgimage && (tu == NULL ||
-			!mg_same_texture(tu->tx, _mgopenglc->bgimage->tx)))
-		mg_remove_txuser(_mgopenglc->bgimage);
-	   _mgopenglc->bgimage = tu;
-	}
-	break;
+      _mgopenglc->winids[DBL] = NEXT(int);
+      break;
 
+    case MG_BGIMAGEFILE: {
+      char *fname = NEXT(char *);
 
+      if (_mgopenglc->bgimagefile) {
+	free(_mgopenglc->bgimagefile);
+      }
+      _mgopenglc->bgimagefile = fname ? strdup(fname) : NULL;
+      mgimgfile2cambgimage();
+      break;
+    }
     default:
       OOGLError (0, "_mgopengl_ctxset: undefined option: %d", attr);
-      return;
+      return -1;
     }
   }
 
@@ -340,6 +353,7 @@ _mgopengl_ctxset(int a1, va_list *alist)
 
 #undef NEXT
 
+  return 0;
 }
 
 
@@ -360,38 +374,38 @@ mgopengl_ctxget(int attr, void* value)
 
   switch (attr) {
 
-  /* Attributes common to all MG contexts: */
+    /* Attributes common to all MG contexts: */
   case MG_APPEAR: *VALUE(Appearance*) = &(_mgc->astk->ap); break;
   case MG_CAMERA: *VALUE(Camera*) = _mgc->cam; break;
   case MG_WINDOW:
-	/* In GLUT case, we demand that the caller use mgctxset(MG_WINDOW, ...)
-	 * whenever our window size changes.  Under GLX/X11, we do it ourselves.
-	 */
+    /* In GLUT case, we demand that the caller use mgctxset(MG_WINDOW, ...)
+     * whenever our window size changes.  Under GLX/X11, we do it ourselves.
+     */
 #ifndef GLUT
 
-	if(_mgopenglc->win > 0) {
-	    int x0, y0;
-	    WnPosition wp;
-	    unsigned int xsize, ysize, border_width, depth;
-	    Window dpyroot, toss;
+    if(_mgopenglc->win > 0) {
+      int x0, y0;
+      WnPosition wp;
+      unsigned int xsize, ysize, border_width, depth;
+      Window dpyroot, toss;
 
-	    XGetGeometry((Display *) (_mgopenglc->GLXdisplay),
-		(Window) _mgopenglc->win, &dpyroot, &x0, &y0, &xsize,
-		&ysize, &border_width, &depth);
-	    XTranslateCoordinates((Display *) (_mgopenglc->GLXdisplay),
-		(Window) _mgopenglc->win, dpyroot, 0, ysize-1,
-                                   &x0, &y0, &toss);
-	    y0 = HeightOfScreen(
-		DefaultScreenOfDisplay((Display *) (_mgopenglc->GLXdisplay)))
-		- y0 - 1;
+      XGetGeometry((Display *) (_mgopenglc->GLXdisplay),
+		   (Window) _mgopenglc->win, &dpyroot, &x0, &y0, &xsize,
+		   &ysize, &border_width, &depth);
+      XTranslateCoordinates((Display *) (_mgopenglc->GLXdisplay),
+			    (Window) _mgopenglc->win, dpyroot, 0, ysize-1,
+			    &x0, &y0, &toss);
+      y0 = HeightOfScreen(
+			  DefaultScreenOfDisplay((Display *) (_mgopenglc->GLXdisplay)))
+	- y0 - 1;
 
-	    wp.xmin = (long)x0; wp.xmax = (long)(x0+xsize-1);
-	    wp.ymin = (long)y0; wp.ymax = (long)(y0+ysize-1);
-	    WnSet(_mgc->win, WN_CURPOS, &wp, WN_END);
-	}
+      wp.xmin = (long)x0; wp.xmax = (long)(x0+xsize-1);
+      wp.ymin = (long)y0; wp.ymax = (long)(y0+ysize-1);
+      WnSet(_mgc->win, WN_CURPOS, &wp, WN_END);
+    }
 #endif
-	*VALUE(WnWindow*) = _mgc->win;
-	break;
+    *VALUE(WnWindow*) = _mgc->win;
+    break;
 
   case MG_PARENT:	*VALUE(mgcontext*) = _mgc->parent; break;
 
@@ -403,8 +417,7 @@ mgopengl_ctxget(int attr, void* value)
   case MG_SHADER:	*VALUE(mgshadefunc) = _mgc->astk->shader; break;
   case MG_SHADERDATA:	*VALUE(void *) = _mgc->astk->shaderdata; break;
   case MG_SPACE:	*VALUE(int) = _mgc->space; break;
-  case MG_NDINFO:	*VALUE(void *) = _mgc->NDinfo; break;
-  case MG_NDMAP:	*VALUE(mgmapfunc) = _mgc->NDmap; break;
+  case MG_NDCTX:	*VALUE(mgNDctx *) = _mgc->NDctx; break;
   case MG_ZNUDGE:	*VALUE(float) = _mgc->zfnudge; break;
 
   case MG_DEPTHSORT:	*VALUE(int) = MG_ZBUFFER; break;
@@ -414,7 +427,7 @@ mgopengl_ctxget(int attr, void* value)
   case MG_WINCHANGE:	*VALUE(mgwinchfunc) = _mgc->winchange; break;
   case MG_WINCHANGEDATA: *VALUE(void *) = _mgc->winchangeinfo; break;
 
-  /* Attributes specific to GL contexts: */
+    /* Attributes specific to GL contexts: */
   case MG_GLWINID: *VALUE(int) = _mgopenglc->win; break;
   case MG_GLBORN: *VALUE(int) = _mgopenglc->born; break;
   case MG_GLZMAX: *VALUE(double) = _mgopenglc->zmax; break;
@@ -432,8 +445,7 @@ mgopengl_ctxget(int attr, void* value)
 #endif
 
   case MG_BGIMAGEFILE:
-    *VALUE(char *) = _mgopenglc->bgimage && _mgopenglc->bgimage->tx
-		   ? _mgopenglc->bgimage->tx->filename : NULL;
+    *VALUE(char *) = _mgopenglc->bgimagefile;
     break;
 
   default:
@@ -476,8 +488,8 @@ mgopengl_sharectx(void)
   for(another = _mgclist; another != NULL; another = another->next) {
     if(another->devno == MGD_OPENGL) {
       if((shareable = ((mgopenglcontext *)another)->cam_ctx[SGL]) != NULL
-       || (shareable = ((mgopenglcontext *)another)->cam_ctx[DBL]) != NULL)
-	    break;
+	 || (shareable = ((mgopenglcontext *)another)->cam_ctx[DBL]) != NULL)
+	break;
     }
   }
   return shareable;
@@ -488,17 +500,19 @@ mgopengl_sharectx(void)
  * Function:	mgopengl_ctxset
  * Description:	set some context attributes
  * Args:	a1, ...: list of attribute-value pairs
- * Returns:	nothing
+ * Returns:	-1 on error, 0 on success
  * Author:	mbp
  * Date:	Fri Sep 20 12:00:18 1991
  */
-void mgopengl_ctxset( int a1, ...  )
+int mgopengl_ctxset( int a1, ...  )
 {
   va_list alist;
+  int result;
 
   va_start( alist, a1 );
-  _mgopengl_ctxset(a1, &alist);
+  result = _mgopengl_ctxset(a1, &alist);
   va_end(alist);
+  return result;
 }
 
 
@@ -535,30 +549,41 @@ void mgopengl_ctxdelete( mgcontext *ctx )
     mgctxselect(ctx);
     mgctxdelete(ctx);
     if(was != ctx)
-	mgctxselect(was);
+      mgctxselect(was);
   } else {
 #define mgoglc  ((mgopenglcontext *)ctx)
 
 #ifdef GLUT
     if(mgoglc->winids[0] > 0)
-	glutDestroyWindow(mgoglc->winids[0]);
+      glutDestroyWindow(mgoglc->winids[0]);
     if(mgoglc->winids[1] > 0)
-	glutDestroyWindow(mgoglc->winids[1]);
+      glutDestroyWindow(mgoglc->winids[1]);
 #else /*GLX*/
     if(mgoglc->born) {
       int i;
       for(i = SGL; i <= DBL; i++) {
-	  if(mgoglc->cam_ctx[i] && mgoglc->win > 0)
-	      glXDestroyContext( (Display *)(mgoglc->GLXdisplay),
-                         mgoglc->cam_ctx[i]);
+	if(mgoglc->cam_ctx[i] && mgoglc->win > 0)
+	  glXDestroyContext( (Display *)(mgoglc->GLXdisplay),
+			     mgoglc->cam_ctx[i]);
       }
     }
 #endif
 
     vvfree(&((mgopenglcontext *)ctx)->room);
+    if (mgoglc->light_lists)
+      free(mgoglc->light_lists);
+    if (mgoglc->texture_lists)
+      free(mgoglc->texture_lists);
+    if (mgoglc->translucent_lists)
+      free(mgoglc->translucent_lists);
+
+    if (mgoglc->bgimagefile) {
+      free(mgoglc->bgimagefile);
+    }
+
     mg_ctxdelete(ctx);
     if(ctx == _mgc)
-	_mgc = NULL;
+      _mgc = NULL;
   }
 }
 
@@ -581,9 +606,9 @@ mgopengl_ctxselect( mgcontext *ctx )
   if(_mgopenglc->win) {
 #ifdef GLUT
     if(_mgopenglc->win <= 0)
-	mgopengl_choosewin();
+      mgopengl_choosewin();
     if(_mgopenglc->win > 0)
-	glutSetWindow(_mgopenglc->win);
+      glutSetWindow(_mgopenglc->win);
 
 #else /* GLX */
     if(_mgopenglc->GLXdisplay != NULL) {
@@ -591,8 +616,8 @@ mgopengl_ctxselect( mgcontext *ctx )
       _mgopenglc->curctx = _mgopenglc->cam_ctx[ _mgc->opts & MGO_DOUBLEBUFFER ? DBL : SGL ];
       if(_mgopenglc->win > 0)
         glXMakeCurrent( (Display *)_mgopenglc->GLXdisplay,
-	 	      (Window) _mgopenglc->win,
-		      _mgopenglc->curctx);
+			(Window) _mgopenglc->win,
+			_mgopenglc->curctx);
     }
 #endif /*GLX*/
   }
@@ -611,7 +636,7 @@ void
 mgopengl_sync( void )
 { glFlush(); }
 
-static int glut_initted = 0;
+/*static int glut_initted = 0;*/
 
 static void
 mgopengl_makewin(int which)
@@ -620,9 +645,9 @@ mgopengl_makewin(int which)
 #define dpy (_mgopenglc->GLXdisplay)
 
   int xsize, ysize;
+#ifdef GLUT
   WnPosition wp;
 
-#ifdef GLUT
   if(!glut_initted) {
     static int unargc = 1;
     static char *unargv[] = { "mg_glut", NULL };
@@ -631,9 +656,9 @@ mgopengl_makewin(int which)
     glut_initted = 0;
   }
   if(WnGet(_mgc->win, WN_XSIZE, &xsize) <= 0 ||
-			WnGet(_mgc->win, WN_YSIZE, &ysize) <= 0) {
-	xsize = ysize = 200;
-	WnSet(_mgc->win, WN_XSIZE, xsize, WN_YSIZE, 200, WN_END);
+     WnGet(_mgc->win, WN_YSIZE, &ysize) <= 0) {
+    xsize = ysize = 200;
+    WnSet(_mgc->win, WN_XSIZE, xsize, WN_YSIZE, 200, WN_END);
   }
   if(WnGet(_mgc->win, WN_PREFPOS, &wp) > 0) {
     glutInitWindowPosition(wp.xmin, wp.ymin);
@@ -646,7 +671,7 @@ mgopengl_makewin(int which)
   static int dblBuf[] = {
     GLX_DOUBLEBUFFER,
     GLX_RGBA, GLX_DEPTH_SIZE, 23,
-    GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1,
+    GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, GLX_ALPHA_SIZE, 1,
     None
   };
   XSetWindowAttributes xswa;
@@ -660,32 +685,32 @@ mgopengl_makewin(int which)
   }
 
   _mgopenglc->cam_ctx[which] =
-	glXCreateContext(dpy, vi, mgopengl_sharectx(), GL_TRUE);
+    glXCreateContext(dpy, vi, mgopengl_sharectx(), GL_TRUE);
 
-    xswa.colormap = (vi->visual == XDefaultVisual(dpy, scr)) ?
-	XDefaultColormap(dpy, scr) :
-	XCreateColormap(dpy, root, vi->visual, AllocNone);
+  xswa.colormap = (vi->visual == XDefaultVisual(dpy, scr)) ?
+    XDefaultColormap(dpy, scr) :
+    XCreateColormap(dpy, root, vi->visual, AllocNone);
 
-    xswa.event_mask = KeyPressMask|ButtonPressMask
-                        |ButtonReleaseMask|ButtonMotionMask
-                        |EnterWindowMask|StructureNotifyMask|ExposureMask;
+  xswa.event_mask = KeyPressMask|ButtonPressMask
+    |ButtonReleaseMask|ButtonMotionMask
+    |EnterWindowMask|StructureNotifyMask|ExposureMask;
 
-    xswa.border_pixel = xswa.background_pixel = xswa.backing_pixel =
-      xswa.background_pixmap = None;
+  xswa.border_pixel = xswa.background_pixel = xswa.backing_pixel =
+    xswa.background_pixmap = None;
 
-    if(WnGet(_mgc->win, WN_XSIZE, &xsize) <= 0 ||
-			WnGet(_mgc->win, WN_YSIZE, &ysize) <= 0) {
-	xsize = ysize = 200;
-	WnSet(_mgc->win, WN_XSIZE, xsize, WN_YSIZE, 200, WN_END);
-    }
+  if(WnGet(_mgc->win, WN_XSIZE, &xsize) <= 0 ||
+     WnGet(_mgc->win, WN_YSIZE, &ysize) <= 0) {
+    xsize = ysize = 200;
+    WnSet(_mgc->win, WN_XSIZE, xsize, WN_YSIZE, 200, WN_END);
+  }
 
-    _mgopenglc->winids[which] = XCreateWindow( dpy, root,
-              0, 0, xsize, ysize, 0/*border*/, vi->depth,
-              InputOutput, vi->visual,
-              CWEventMask|CWColormap|CWBorderPixel
-              |CWBackPixel|CWBackPixmap, &xswa);
+  _mgopenglc->winids[which] = XCreateWindow( dpy, root,
+					     0, 0, xsize, ysize, 0/*border*/, vi->depth,
+					     InputOutput, vi->visual,
+					     CWEventMask|CWColormap|CWBorderPixel
+					     |CWBackPixel|CWBackPixmap, &xswa);
 
-    XMapWindow(dpy, _mgopenglc->winids[which]);
+  XMapWindow(dpy, _mgopenglc->winids[which]);
 
 #undef dpy
 
@@ -704,12 +729,12 @@ mgopengl_choosewin(void)
   if(_mgopenglc->winids[which] == 0) {
     mgopengl_makewin(which);
     if(_mgopenglc->winids[which] == 0)
-	which = 1-which;
+      which = 1-which;
   }
 
   if(_mgc->winchange)
     (*_mgc->winchange)(_mgc, _mgc->winchangeinfo,
-	MGW_DOUBLEBUF, (which == DBL));
+		       MGW_DOUBLEBUF, (which == DBL));
 
   _mgopenglc->win = _mgopenglc->winids[which];
   glutSetWindow(_mgopenglc->win);
@@ -718,8 +743,8 @@ mgopengl_choosewin(void)
 
   if(_mgopenglc->GLXdisplay == NULL && _mgopenglc->winids[which] == 0) {
     if((_mgopenglc->GLXdisplay = XOpenDisplay(NULL)) == NULL) {
-	OOGLError(1, "Can't open X display");
-	return;
+      OOGLError(1, "Can't open X display");
+      return;
     }
   }
 
@@ -727,26 +752,37 @@ mgopengl_choosewin(void)
   /* Switch back to the other buffer if need be */
   if(_mgopenglc->cam_ctx[which] == NULL) {
     if(_mgopenglc->cam_ctx[1-which] == NULL) {
-	mgopengl_makewin(which);
+      mgopengl_makewin(which);
     } else {
-	which = 1-which;
+      which = 1-which;
     }
   }
 
   if(_mgc->winchange)
-	(*_mgc->winchange)(_mgc, _mgc->winchangeinfo, MGW_DOUBLEBUF,
-				(which == DBL));
+    (*_mgc->winchange)(_mgc, _mgc->winchangeinfo, MGW_DOUBLEBUF,
+		       (which == DBL));
 
   _mgopenglc->curctx = _mgopenglc->cam_ctx[which];
   _mgopenglc->win = _mgopenglc->winids[which];
   if(_mgopenglc->win > 0) {
     XRaiseWindow( (Display *)_mgopenglc->GLXdisplay, (Window) _mgopenglc->win );
     glXMakeCurrent( (Display *)_mgopenglc->GLXdisplay,
-	(Window) _mgopenglc->win,
-	_mgopenglc->curctx);
+		    (Window) _mgopenglc->win,
+		    _mgopenglc->curctx);
   }
 
 #endif /* GLX */
+
+  if (_mgopenglc->n_light_lists == 0)
+    _mgopenglc->light_lists =
+      mgopengl_realloc_lists(NULL, &_mgopenglc->n_light_lists);
+  if (_mgopenglc->n_texture_lists == 0)
+    _mgopenglc->texture_lists =
+      mgopengl_realloc_lists(NULL, &_mgopenglc->n_texture_lists);
+  if (_mgopenglc->n_translucent_lists == 0)
+    _mgopenglc->translucent_lists = 
+      mgopengl_realloc_lists(NULL, &_mgopenglc->n_translucent_lists);
+
 }
 
 static void
@@ -758,11 +794,11 @@ mgopengl_setviewport()
   glViewport(vp.xmin, vp.ymin, vp.xmax-vp.xmin+1, vp.ymax-vp.ymin+1);
   glScissor(vp.xmin, vp.ymin, vp.xmax-vp.xmin+1, vp.ymax-vp.ymin+1);
   if(WnGet(_mgc->win, WN_CURPOS, &whole) <= 0 ||
-			vp.xmax-vp.xmin < whole.xmax-whole.xmin ||
-			vp.ymax-vp.ymin < whole.ymax-whole.ymin) {
-	glEnable(GL_SCISSOR_TEST);
+     vp.xmax-vp.xmin < whole.xmax-whole.xmin ||
+     vp.ymax-vp.ymin < whole.ymax-whole.ymin) {
+    glEnable(GL_SCISSOR_TEST);
   } else {
-	glDisable(GL_SCISSOR_TEST);
+    glDisable(GL_SCISSOR_TEST);
   }
   _mgc->win->changed &= ~WNF_HASVP;
 }
@@ -770,11 +806,7 @@ mgopengl_setviewport()
 static void
 mgopengl_initwin()
 {
-  WnPosition pos, vp;
-  int xsize, ysize, flag;
   GLdouble zrange[2];
-  char *name;
-  char gver[80];
   int i;
   LtLight **lp;
 
@@ -793,7 +825,7 @@ mgopengl_initwin()
 
   glClearDepth(_mgopenglc->zmax);
   glClearColor(_mgc->background.r, _mgc->background.g, _mgc->background.b,
-		_mgc->background.a);
+	       _mgc->background.a);
   glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);
 
   if((_mgc->opts&MGO_DOUBLEBUFFER) && !(_mgc->opts&MGO_INHIBITSWAP) ) {
@@ -801,7 +833,7 @@ mgopengl_initwin()
     glutSwapBuffers();
 #else
     if(_mgopenglc->win > 0)
-	glXSwapBuffers((Display *) (_mgopenglc->GLXdisplay), (Window) _mgopenglc->win);
+      glXSwapBuffers((Display *) (_mgopenglc->GLXdisplay), (Window) _mgopenglc->win);
 #endif
   }
 
@@ -827,84 +859,92 @@ mgopengl_initwin()
 void
 mgopengl_worldbegin( void )
 {
-  Transform V, S;
+  Transform V;
   int which = (_mgc->opts & MGO_DOUBLEBUFFER) ? DBL : SGL;
 
   mg_worldbegin();	/* Initialize W2C, C2W, W2S, S2W, etc. */
 
   if(
 #ifndef GLUT
-	_mgopenglc->curctx != _mgopenglc->cam_ctx[which] ||
+     _mgopenglc->curctx != _mgopenglc->cam_ctx[which] ||
 #endif
-		(_mgc->opts ^ _mgopenglc->oldopts) & MGO_DOUBLEBUFFER) {
+     (_mgc->opts ^ _mgopenglc->oldopts) & MGO_DOUBLEBUFFER) {
     mgopengl_choosewin();
     mgopengl_initwin();
     _mgopenglc->oldopts = _mgc->opts;
   }
 
-  glColorMask(  _mgc->opts & MGO_NORED ? GL_FALSE : GL_TRUE,
-		_mgc->opts & MGO_NOGREEN ? GL_FALSE : GL_TRUE,
-		_mgc->opts & MGO_NOBLUE ? GL_FALSE : GL_TRUE,
-		GL_TRUE );
+  glColorMask(_mgc->opts & MGO_NORED ? GL_FALSE : GL_TRUE,
+	      _mgc->opts & MGO_NOGREEN ? GL_FALSE : GL_TRUE,
+	      _mgc->opts & MGO_NOBLUE ? GL_FALSE : GL_TRUE,
+	      GL_TRUE );
 
-  if(_mgc->win->changed & WNF_HASVP)
+  if(_mgc->win->changed & WNF_HASVP) {
     mgopengl_setviewport();
+  }
 
   /* Erase to background color & initialize z-buffer */
   if (_mgc->opts & MGO_INHIBITCLEAR) {
-	glClearDepth(1.);
-	glClear(GL_DEPTH_BUFFER_BIT);
-  }
-  else { 
-    Texture *btx;
+    glClearDepth(1.);
+    glClear(GL_DEPTH_BUFFER_BIT);
+  } else {
+    Image *bgimg;
+
     glClearDepth( _mgopenglc->zmax);
     glClearColor(_mgc->background.r,
-	_mgc->background.g, _mgc->background.b, _mgc->background.a);
+		 _mgc->background.g, _mgc->background.b, _mgc->background.a);
 
     glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);
-    if(_mgopenglc->bgimage && (btx = _mgopenglc->bgimage->tx)) {
-	static GLint formats[] =
-	    { 0, GL_LUMINANCE, GL_LUMINANCE_ALPHA, GL_RGB, GL_RGBA };
-	if(btx->data == NULL)
-	    mg_inhaletexture(btx, TXF_RGBA);
-	if(btx->data != NULL) {
-	    static GLdouble pos[] = {0.0, 0.0, -1.0};
-	    int xsize, ysize, off;
-	    btx->flags |= TXF_USED;
-	    /* Could resize the image to fit the window here,
-	     * (or force the window to fit the image!)
-	     * but let's make that the user's problem.
-	     */
-	    glMatrixMode(GL_PROJECTION);
-	    glLoadIdentity();
-	    WnGet(_mgc->win, WN_XSIZE, &xsize);
-	    WnGet(_mgc->win, WN_YSIZE, &ysize);
-	    glOrtho(0, xsize, 0, ysize, -1, 1);
-	    glMatrixMode(GL_MODELVIEW);
-	    glLoadIdentity();
-	    if(xsize >= btx->xsize) {
-		pos[0] = (xsize - btx->xsize)/2;
-		off = 0;
-	    } else {
-		pos[0] = 0;
-		off = (btx->xsize - xsize) / 2;
-		glPixelStorei(GL_PACK_ROW_LENGTH, xsize);
-	    }
-	    if(ysize >= btx->ysize) {
-		pos[1] = (ysize - btx->ysize)/2;
-	    } else {
-		pos[1] = 0;
-		off += btx->xsize * ((btx->ysize - ysize) / 2);
-	    }
-	    glRasterPos3dv(pos);
-	    glDepthMask(GL_FALSE);
-	    glDisable(GL_DEPTH_TEST);
-	    glDrawPixels( btx->xsize, ysize < btx->ysize ? ysize : btx->ysize,
-			formats[btx->channels],
-			GL_UNSIGNED_BYTE, btx->data + btx->channels*off );
-	    glDepthMask(GL_TRUE);
-	    glPixelStorei(GL_PACK_ROW_LENGTH, 0);
-	}
+
+    if((bgimg = _mgc->bgimage)) {
+      static GLint formats[] =
+	{ 0, GL_LUMINANCE, GL_LUMINANCE_ALPHA, GL_RGB, GL_RGBA };
+      static GLdouble pos[] = {0.0, 0.0, -1.0};
+      int xsize, ysize, off;
+
+      /* Could resize the image to fit the window here,
+       * (or force the window to fit the image!)
+       * but let's make that the user's problem.
+       */
+      glMatrixMode(GL_PROJECTION);
+      glLoadIdentity();
+      WnGet(_mgc->win, WN_XSIZE, &xsize);
+      WnGet(_mgc->win, WN_YSIZE, &ysize);
+      glOrtho(0, xsize, 0, ysize, -1, 1);
+      glMatrixMode(GL_MODELVIEW);
+      glLoadIdentity();
+      if(xsize >= bgimg->width) {
+	pos[0] = (xsize - bgimg->width)/2;
+	off = 0;
+      } else {
+	pos[0] = 0;
+	off = (bgimg->width - xsize) / 2;
+	glPixelStorei(GL_UNPACK_ROW_LENGTH, bgimg->width);
+      }
+      if(ysize >= bgimg->height) {
+	pos[1] = (ysize - bgimg->height)/2;
+      } else {
+	pos[1] = 0;
+	off += bgimg->width * ((bgimg->height - ysize) / 2);
+      }
+      glRasterPos3dv(pos);
+      glDepthMask(GL_FALSE);
+      glDisable(GL_DEPTH_TEST);
+      if (bgimg->channels % 1 == 0) {
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+      }
+      glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+      glDrawPixels(xsize < bgimg->width ? xsize : bgimg->width,
+		   ysize < bgimg->height ? ysize : bgimg->height,
+		   formats[bgimg->channels],
+		   GL_UNSIGNED_BYTE, bgimg->data + bgimg->channels*off );
+      if (bgimg->channels % 1 == 0) {
+	glBlendFunc(GL_SRC_ALPHA, GL_ZERO);
+	glDisable(GL_BLEND);
+      }
+      glDepthMask(GL_TRUE);
+      glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
     }
   }
 
@@ -912,19 +952,22 @@ mgopengl_worldbegin( void )
 
   glRenderMode(GL_RENDER);
 
-  if(_mgopenglc->dither) glEnable(GL_DITHER);
-  else glDisable(GL_DITHER);
+  if(_mgopenglc->dither) {
+    glEnable(GL_DITHER);
+  } else {
+    glDisable(GL_DITHER);
+  }
 
   _mgc->has = 0;
 
   /* Interpret the camera: load the proper matrices onto the GL matrix
      stacks.  */
   if(!(_mgc->opts & MGO_INHIBITCAM)) {
-      glMatrixMode(GL_PROJECTION);
-      CamViewProjection( _mgc->cam, V );
-      glLoadMatrixf( (GLfloat *) V);
-      glMatrixMode(GL_MODELVIEW);
-      glLoadMatrixf( (GLfloat *) _mgc->W2C);
+    glMatrixMode(GL_PROJECTION);
+    CamViewProjection( _mgc->cam, V );
+    glLoadMatrixf( (GLfloat *) V);
+    glMatrixMode(GL_MODELVIEW);
+    glLoadMatrixf( (GLfloat *) _mgc->W2C);
 
   }
 
@@ -945,6 +988,20 @@ mgopengl_worldbegin( void )
 void
 mgopengl_worldend( void )
 {
+  GLuint list;
+  
+  /* Execute and delete any pending translucent display lists. */
+  for (list = 0; list < _mgopenglc->translucent_seq; list++) {
+    glCallList(_mgopenglc->translucent_lists[list]);
+  }
+
+  for (list = 0; list < _mgopenglc->translucent_seq; list++) {
+    /* Destroy the contents of the dpy lists for translucent objects. */
+    glNewList(_mgopenglc->translucent_lists[list], GL_COMPILE);
+    glEndList();
+  }
+  _mgopenglc->translucent_seq = 0;
+
   if((_mgc->opts&MGO_DOUBLEBUFFER) && !(_mgc->opts&MGO_INHIBITSWAP)) {
 #ifdef GLUT
     glutSwapBuffers();
@@ -954,6 +1011,7 @@ mgopengl_worldend( void )
 #endif
   }
   glFlush();
+
 }
 
 /*-----------------------------------------------------------------------
@@ -969,7 +1027,6 @@ mgopengl_worldend( void )
 void
 mgopengl_reshapeviewport( void )
 {
-  long w, h;
   float pixasp = 1;
   WnPosition vp;
 
@@ -977,9 +1034,9 @@ mgopengl_reshapeviewport( void )
   WnGet(_mgc->win, WN_VIEWPORT, &vp);
   mgopengl_setviewport();
   CamSet(_mgc->cam, CAM_ASPECT,
-	pixasp * (double)(vp.xmax-vp.xmin+1) /
-		 (double)(vp.ymax-vp.ymin+1),
-	CAM_END);
+	 pixasp * (double)(vp.xmax-vp.xmin+1) /
+	 (double)(vp.ymax-vp.ymin+1),
+	 CAM_END);
 }
 
 /*-----------------------------------------------------------------------
@@ -1101,6 +1158,59 @@ mgopengl_settransform( Transform T )
   _mgc->has = _mgc->xstk->hasinv = 0;
 }
 
+/*-----------------------------------------------------------------------
+ * Function:	mgopengl_new_translucent
+ * Description: start a display-list which stores a translucent object,
+ *              the dpy-list will be called later after all opaque objects
+ *              have been drawn.
+ * Returns:	the display list.
+ * Author:	cH
+ * Date:	Fri Nov 17 2006
+ */
+GLuint mgopengl_new_translucent(Transform T)
+{
+  
+  if (_mgopenglc->n_translucent_lists <= _mgopenglc->translucent_seq) {
+    _mgopenglc->translucent_lists =
+      mgopengl_realloc_lists(_mgopenglc->translucent_lists,
+			     &_mgopenglc->n_translucent_lists);
+  }
+
+  glNewList(_mgopenglc->translucent_lists[_mgopenglc->translucent_seq],
+	    GL_COMPILE /* do not execute now */);
+
+  /* include the side-effects into the displaylist */
+  mgopengl_pushtransform();
+  mgopengl_pushappearance();
+
+  /* Load the current transformation matrix into the display list */
+  mgopengl_settransform(T);
+
+  /* alpha-blending will be enabled on-demand in mgopengl_bsptree(). */
+
+  return _mgopenglc->translucent_lists[_mgopenglc->translucent_seq++];
+}
+
+/*-----------------------------------------------------------------------
+ * Function:	mgopengl_end_translucent
+ * Description: end the definition of a translucent object.
+ * Returns:	the display list.
+ * Author:	cH
+ * Date:	Fri Nov 17 2006
+ */
+void mgopengl_end_translucent(void)
+{
+
+  /* include the side-effects into the displaylist */
+  mgopengl_popappearance();
+  mgopengl_poptransform();
+
+  glEndList();
+  /* No need to disable alpha-blending because the corresponding
+   * enable-commands are wrapped into a GL_COMPILE display list.
+   */
+}
+
 /*-----------------------------------------------------------------------
  * Function:	mgopengl_pushappearance
  * Description:	push the MG context appearance stack
@@ -1128,8 +1238,8 @@ mgopengl_pushappearance( void )
 int
 mgopengl_popappearance( void )
 {
-  register struct mgastk *mastk = _mgc->astk;
-  register struct mgastk *mastk_next;
+  struct mgastk *mastk = _mgc->astk;
+  struct mgastk *mastk_next;
 
   if (! (mastk_next=mastk->next)) {
     OOGLError(0, "mgopengl_popappearance: appearance stack has only 1 entry.");
@@ -1138,17 +1248,19 @@ mgopengl_popappearance( void )
 
   if ( ( (mastk->light_seq != mastk_next->light_seq)	/* lighting changed */
 #ifndef TRUE_EMISSION
-	|| ((mastk->mat.valid ^ mastk_next->mat.valid) & MTF_EMISSION)
-							/* GL_LIGHT_MODEL_TWO_SIDE changed */
+	 || ((mastk->mat.valid ^ mastk_next->mat.valid) & MTF_EMISSION)
+	 /* GL_LIGHT_MODEL_TWO_SIDE changed */
 #endif
-       ) && IS_SHADED(mastk->next->ap.shading))    /* lighting on */
-      mgopengl_lighting(mastk_next, mastk_next->lighting.valid);
+	 ) && IS_SHADED(mastk->next->ap.shading))    /* lighting on */
+    mgopengl_lighting(mastk_next, mastk_next->lighting.valid);
   mgopengl_appearance(mastk_next, mastk_next->ap.valid);
 
   mg_popappearance();
 
-  if(_mgopenglc->curtex && _mgopenglc->curtex->tx != mastk_next->ap.tex)
-    mgopengl_notexture();           /* Unbind our no-longer-needed texture */
+  if (_mgopenglc->tevbound && _mgopenglc->curtex->tx != mastk_next->ap.tex) {
+    mgopengl_notexture(); /* Unbind our no-longer-needed texture */
+  }
+
   return 1;
 }
 
@@ -1170,8 +1282,8 @@ mgopengl_popappearance( void )
  *		things not set here: normals (drawing, scaling,
  *		  everting), drawing faces vs. edges
  */
-Appearance *
-mgopengl_setappearance( Appearance* ap, int mergeflag )
+const Appearance *
+mgopengl_setappearance(const Appearance *ap, int mergeflag )
 {
   int changed, mat_changed, lng_changed;
   struct mgastk *mastk = _mgc->astk;
@@ -1187,7 +1299,7 @@ mgopengl_setappearance( Appearance* ap, int mergeflag )
       ap->mat ? ap->mat->valid &~ (ma->mat->override &~ ap->mat->override) : 0;
     lng_changed =
       ap->lighting ? ap->lighting->valid &~ 
-	(ma->lighting->override &~ ap->lighting->override) : 0;
+      (ma->lighting->override &~ ap->lighting->override) : 0;
   }
   else {
     changed = ap->valid;
@@ -1219,61 +1331,95 @@ mgopengl_setappearance( Appearance* ap, int mergeflag )
      */
     if ((ap->lighting) && (mastk->next)) {
       if (mastk->light_seq == mastk->next->light_seq) {
-	mastk->light_seq++;
+	mastk->light_seq = next_light_seq(_mgc, mastk);	
+	if (_mgopenglc->n_light_lists <= mastk->light_seq) {
+	  _mgopenglc->light_lists =
+	    mgopengl_realloc_lists(_mgopenglc->light_lists,
+				   &_mgopenglc->n_light_lists);
+	}	
 	/*
 	 * We need a new lighting model.
 	 * To ensure we don't have any leftover garbage in GL's copy of this
 	 * lighting model, we force GL to reset to defaults, then
 	 * reinitialize everything.
 	 */
-	glNewList( mastk->light_seq, GL_COMPILE); glMaterialf(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, * nullarray); glEndList();;
+	glNewList(_mgopenglc->light_lists[mastk->light_seq], GL_COMPILE);
+	glMaterialf(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, *nullarray);
+	glEndList();;
 	lng_changed |= ma->lighting->valid;	/* "All fields changed" */
       }
     }
     if (ma->shading != APF_CONSTANT &&
-		(ap->lighting != NULL 
+	(ap->lighting != NULL 
 #ifndef TRUE_EMISSION
-			/* Hack: must enable GL_LIGHT_MODEL_TWO_SIDE lighting if so */
-		|| (mastk->next && 
-		    (mastk->mat.valid ^ mastk->next->mat.valid) & MTF_EMISSION)
+	 /* Hack: must enable GL_LIGHT_MODEL_TWO_SIDE lighting if so */
+	 || (mastk->next && 
+	     (mastk->mat.valid ^ mastk->next->mat.valid) & MTF_EMISSION)
 #endif
-					)) {
+	 )) {
       mgopengl_lighting( mastk, lng_changed );
     }
 
-	/* Let mgopengl_material() decide if we need a new material */
-    if (ap->mat)
+    /* Let mgopengl_material() decide if we need a new material */
+    if (ap->mat) {
       mgopengl_material( mastk, mat_changed );
+    }
 
     mgopengl_appearance( mastk, changed );
 
-        /* Ensure that, at least, we don't have the *wrong* texture bound. */
-    if(ap->tex) {
-        static int dbgtex;
-        if(_mgopenglc->tevbound)
-            mgopengl_notexture();
+    if (_mgopenglc->tevbound && _mgopenglc->curtex->tx != ap->tex) {
+      mgopengl_notexture(); /* Unbind our no-longer-needed texture */
     }
-
   }
-  return ap;
+
+  return &_mgc->astk->ap;
 }
 
-
-/*-----------------------------------------------------------------------
- * Function:	mgopengl_getappearance
- * Description:	return a ptr to current appearance
- * Returns:	ptr to current appearance
- * Author:	mbp
- * Date:	Fri Sep 20 13:00:41 1991
- * Notes:	Applications should not modify the returned appearance
- *		in any way.
- */
-Appearance *
-mgopengl_getappearance()
+void mgopengl_taggedappearance(const void *tag)
 {
-    return &(_mgc->astk->ap);
+  struct mgastk *astk = (struct mgastk *)tag;
+  Appearance *ap = &astk->ap;
+  LmLighting *lm = &astk->lighting;
+
+  mg_setappearance(ap, 0);
+
+  if (lm->valid) {
+    glCallList(_mgopenglc->light_lists[astk->light_seq]);
+  }
+
+  glMatrixMode(GL_MODELVIEW);
+  glPushMatrix();
+  glLoadMatrixf(&_mgc->W2C[0][0] );
+  mgopengl_lights(lm, astk);
+  glPopMatrix();
+
+  /* Let mgopengl_material() decide if we need a new material */
+  if (ap->mat) {
+    mgopengl_material(astk, ap->mat->valid);
+  }
+
+  mgopengl_appearance(astk, ap->valid);
+
+  if (_mgopenglc->tevbound && _mgopenglc->curtex->tx != ap->tex) {
+    mgopengl_notexture(); /* Unbind our no-longer-needed texture */
+  }
 }
 
+static void mgimgfile2cambgimage(void)
+{
+  Image *bgimage;
+
+  if (_mgc->cam && _mgopenglc->bgimagefile) {
+    bgimage =
+      ImgCreate(IMG_CHANNELS, IMGF_RGBA,
+		IMG_DATA_CHAN_FILE, IMGF_RGB, NULL, _mgopenglc->bgimagefile,
+		IMG_END);
+    if (bgimage) {
+      CamSet(_mgc->cam, CAM_BGIMAGE, bgimage);
+      ImgDelete(bgimage);
+    }
+  }
+}
 
 /*-----------------------------------------------------------------------
  * Function:	mgopengl_setcamera
@@ -1285,13 +1431,17 @@ mgopengl_getappearance()
  * Notes:	The context stores a pointer to the camera, not a copy
  *		of it.
  */
-int
-mgopengl_setcamera( Camera* cam )
+int mgopengl_setcamera(Camera* cam)
 {
-  if (_mgc->cam) CamDelete(_mgc->cam);
-  _mgc->cam = cam;
-  RefIncr((Ref*) cam);
-  return 0;
+  if (_mgc->cam != cam) {
+    CamDelete(_mgc->cam);
+    _mgc->cam = REFGET(Camera, cam);
+    if (_mgc->cam) {
+      CamGet(_mgc->cam, CAM_BGCOLOR, &_mgc->background);
+      mgimgfile2cambgimage();
+    }
+  }
+  return true;
 }
 
 /*
@@ -1302,13 +1452,9 @@ mgopengl_setcamera( Camera* cam )
 int
 mgopengl_setwindow( WnWindow *win, int final )
 {
-  WnPosition pos, vp;
-  int xsize, ysize, flag, reconstrain;
-  int positioned = 0;
-  char *name, *oname;
-
-  if(win == NULL)
+  if(win == NULL) {
     return 0;
+  }
 
   /* We can't do anything on our own.  Leave it up to 
    * the (possibly) user-supplied winchange callback.
@@ -1317,14 +1463,29 @@ mgopengl_setwindow( WnWindow *win, int final )
     (*_mgc->winchange)(_mgc, _mgc->winchangeinfo, MGW_WINCHANGE, win);
 
   if(win != _mgc->win) {
-    RefIncr((Ref *)win);
     WnDelete(_mgc->win);
-    _mgc->win = win;
+    _mgc->win = REFGET(WnWindow, win);
   }
   return 1;
 }
 
+GLuint *mgopengl_realloc_lists(GLuint *lists, int *n_lists)
+{
+  int i;
+  GLuint new_lists;
 
+  if ((new_lists = glGenLists(DPYLIST_INCR)) == 0) {
+    OOGLError(0, "mgopengl_realloc_lists: no new lists available.");
+    return NULL;
+  }
+  lists = realloc(lists, (*n_lists + DPYLIST_INCR)*sizeof(GLuint));
+  for (i = *n_lists; i < *n_lists + DPYLIST_INCR; i++) {
+    lists[i] = new_lists++;
+  }
+  *n_lists = i;
+
+  return lists;
+}
 
 /*-----------------------------------------------------------------------
  * Function:	mgopengl_newcontext
@@ -1340,9 +1501,9 @@ mgopengl_newcontext( mgopenglcontext *ctx )
   mg_newcontext(&(ctx->mgctx));
   ctx->mgctx.devfuncs = &mgopenglfuncs;
   ctx->mgctx.devno = MGD_OPENGL;
-  ctx->mgctx.astk->ap_seq = 1;
-  ctx->mgctx.astk->mat_seq = 1;
-  ctx->mgctx.astk->light_seq = 1;
+  ctx->mgctx.astk->ap_seq = 0;
+  ctx->mgctx.astk->mat_seq = 0;
+  ctx->mgctx.astk->light_seq = 0;
   ctx->mgctx.zfnudge = 40.e-6;
   ctx->born = 0;
   ctx->win = 0;
@@ -1365,7 +1526,17 @@ mgopengl_newcontext( mgopenglcontext *ctx )
 #endif
   ctx->should_lighting = ctx->is_lighting = 0;
   ctx->dither = 1;
-  ctx->bgimage = NULL;
+  ctx->bgimagefile = NULL;
+
+  /* reserve a number of display list numbers */
+  ctx->n_light_lists = 0;
+  ctx->light_lists = NULL;
+  ctx->n_texture_lists = 0;
+  ctx->texture_lists = NULL;
+  ctx->n_translucent_lists = 0;
+  ctx->translucent_lists = NULL;
+  ctx->translucent_seq = 0;
+  
   return ctx;
 }
 
@@ -1380,11 +1551,17 @@ mgopengl_newcontext( mgopenglcontext *ctx )
 mgcontext *
 mgopengl_findctx( int winid )
 {
-  register struct mgcontext *mgc;
+  struct mgcontext *mgc;
 
   for(mgc = _mgclist; mgc != NULL; mgc = mgc->next) {
     if(mgc->devno == MGD_OPENGL && ((mgopenglcontext *)mgc)->win == winid)
-	return mgc;
+      return mgc;
   }
   return NULL;
 }
+
+/*
+ * Local Variables: ***
+ * c-basic-offset: 2 ***
+ * End: ***
+ */
