@@ -1,5 +1,6 @@
 /* Copyright (C) 1992-1998 The Geometry Center
  * Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips
+ * Copyright (C) 2007 Claus-Justus Heine
  *
  * This file is part of Geomview.
  * 
@@ -19,12 +20,14 @@
  * USA, or visit http://www.gnu.org.
  */
 
-#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#if 0
 static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
 Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
+#endif
 
 #include "mg.h"
 #include "mgP.h"
@@ -37,9 +40,15 @@ Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#ifdef HAVE_LIBGEN_H
+# include <libgen.h>
+#endif
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 mgcontext * mgrib_ctxcreate(int a1, ...);
-void	    mgrib_ctxset( int a1, ...  );
+int	    mgrib_ctxset( int a1, ...  );
 int	    mgrib_feature( int feature );
 void	    mgrib_ctxdelete( mgcontext *ctx );
 int	    mgrib_ctxget( int attr, void* valueptr );
@@ -55,8 +64,7 @@ int	    mgrib_poptransform( void );
 void	    mgrib_settransform( Transform T );
 int	    mgrib_pushappearance( void );
 int	    mgrib_popappearance( void );
-Appearance  *mgrib_setappearance( Appearance* app, int merge);
-Appearance  *mgrib_getappearance( void );
+const Appearance *mgrib_setappearance(const Appearance* app, int merge);
 int	    mgrib_setcamera( Camera* cam );
 mgribcontext *mgrib_newcontext( mgribcontext *ctx );
 
@@ -67,7 +75,7 @@ extern void  mgrib_polyline();
 extern void  mgrib_polylist();
 extern void  mgrib_bezier();
 
-void _mgrib_ctxset(int a1, va_list *alist);
+int _mgrib_ctxset(int a1, va_list *alist);
 
 WnWindow *mgribwindow(WnWindow *win);
 
@@ -77,23 +85,33 @@ struct mgfuncs mgribfuncs = {
   mgrib_feature,
   (mgcontext *(*)())mgrib_ctxcreate,
   mgrib_ctxdelete,
-  (void (*)())mgrib_ctxset,
+  (int (*)())mgrib_ctxset,
   mgrib_ctxget,
   mgrib_ctxselect,
   mgrib_sync,
   mgrib_worldbegin,
   mgrib_worldend,
   mgrib_reshapeviewport,
+  /* geometry transform */
   mgrib_settransform,
   mg_gettransform,	/* Generic */
   mgrib_identity,
   mgrib_transform,
   mgrib_pushtransform,
   mgrib_poptransform,
+  /* texture transform */
+  mg_settxtransform,
+  mg_gettxtransform,
+  mg_txidentity,
+  mg_txtransform,
+  mg_pushtxtransform,
+  mg_poptxtransform,
+  /* appearance stack */
   mgrib_pushappearance,
   mgrib_popappearance,
   mgrib_setappearance,
-  mgrib_getappearance,
+  mg_getappearance,
+  /********************/
   mgrib_setcamera,
   mgrib_polygon,
   mgrib_polylist,
@@ -101,10 +119,14 @@ struct mgfuncs mgribfuncs = {
   mgrib_line,
   mgrib_polyline,
   mg_quads,
-  mgrib_bezier
-  };
+  mgrib_bezier,
+  mg_bsptree,
+  mg_tagappearance,
+  mg_untagappearance,
+  mg_taggedappearance
+};
 
-static mgribcontext *MGRIB;	/* For debugging */
+mgribcontext *_MGRIB_;	/* For debugging */
 
 int
 mgdevice_RIB()
@@ -129,11 +151,11 @@ mgrib_ctxcreate(int a1, ...)
   time_t timedate = (time_t)time(0);
     
   _mgc =
-    (mgcontext*)(MGRIB = mgrib_newcontext( OOGLNewE(mgribcontext,
-    	"mgrib_ctxcreate") ));
+    (mgcontext*)(_MGRIB_ = mgrib_newcontext( OOGLNewE(mgribcontext,
+						      "mgrib_ctxcreate") ));
 
   /* Ensure some sensible default Window */
-  WnSet(_mgc->win, WN_XSIZE, 300, WN_YSIZE, 200, WN_END);
+  WnSet(_mgc->win, WN_XSIZE, 450, WN_YSIZE, 450, WN_END);
 
   /* Default RIB1.0 structure info */
   sprintf(_mgribc->ribscene, "Generic RIB file");
@@ -141,10 +163,14 @@ mgrib_ctxcreate(int a1, ...)
   sprintf(_mgribc->ribfor, getenv("USER"));
   sprintf(_mgribc->ribdate, ctime(&timedate));
   _mgribc->ribdate[24] = '\0'; /* remove the line feed */
-  
+  _mgribc->world = 0;
+
   va_start(alist, a1);
-  _mgrib_ctxset(a1, &alist);
+  if (_mgrib_ctxset(a1, &alist) == -1) {
+    mgrib_ctxdelete(_mgc);
+  }
   va_end(alist);
+
   return _mgc;
 }
 
@@ -153,28 +179,36 @@ mgrib_ctxcreate(int a1, ...)
  * Description:	internal ctxset routine
  * Args:	a1: first attribute
  *		*alist: rest of attribute-value list
- * Returns:	nothing
+ * Returns:	-1 on error, 0 on success
  * Author:	mbp
  * Date:	Fri Sep 20 11:08:13 1991
  * Notes:	mgrib_ctxcreate() and mgrib_ctxset() call this to actually
  *		parse and interpret the attribute list.
  */
-void
+int
 _mgrib_ctxset(int a1, va_list *alist)
 {
   int attr;
+  FILE *ribfile = NULL;
+  const char *ribdpy = NULL;
+  struct stat st;
+  char *dot;
+
+  /* secure copy to fixed-length string */
+#undef strNcpy
+#define strNcpy(dst, src)						\
+  strncpy((dst), (src), sizeof(dst)); (dst)[sizeof(dst)-1] = '\0'
 
   for (attr = a1; attr != MG_END; attr = va_arg (*alist, int)) {
     switch (attr) {
-    case MG_ApSet:
-      {
-	Appearance *ap;
-
-	ap = _ApSet(NULL, va_arg(*alist, int), alist);
-	mgrib_setappearance(ap, MG_MERGE);
-	ApDelete(ap);
-      }
+    case MG_ApSet: {
+      Appearance *ap;
+      
+      ap = _ApSet(NULL, va_arg(*alist, int), alist);
+      mgrib_setappearance(ap, MG_MERGE);
+      ApDelete(ap);
       break;
+    }
     case MG_WnSet:
       _WnSet( _mgc->win, va_arg(*alist, int), alist);
       break;
@@ -208,81 +242,144 @@ _mgrib_ctxset(int a1, va_list *alist)
       break;
     case MG_ZNUDGE:
       /* for now, read nothing */
-      va_arg(*alist, double);
+      (void)va_arg(*alist, double);
+      break;
+    case MG_SPACE:
+      _mgc->space = va_arg(*alist, int);
+      break;
+    case MG_NDCTX:
+      _mgc->NDctx = va_arg(*alist, mgNDctx *);
       break;
 
-    case MG_SPACE:   _mgc->space = va_arg(*alist, int); break;
-    case MG_NDINFO:  _mgc->NDinfo = va_arg(*alist, void *); break;
-    case MG_NDMAP:   _mgc->NDmap = va_arg(*alist, mgmapfunc); break;
-
-
-    /* kind of RIB-specific */
+      /* kind of RIB-specific */
     case MG_RIBFILE:
-/*        if(_mgribc->rib) fclose(_mgribc->rib); */
-	_mgribc->rib = va_arg(*alist, FILE*);
-	break;
+      /*        if(_mgribc->rib) fclose(_mgribc->rib); */
+      ribfile = va_arg(*alist, FILE*);
+      break;
 
-    /* really RIB-specific */
+      /* really RIB-specific */
     case MG_RIBLINEMODE:
-	_mgribc->line_mode = va_arg(*alist, int);
-	break;
+      _mgribc->line_mode = (enum line_mode)va_arg(*alist, int);
+      break;
     case MG_RIBFORMAT:
-	switch( va_arg(*alist, int) ) {
-	case MG_RIBASCII:
-	    _mgribc->render_device |= RMD_ASCII;
-	    _mgribc->render_device &= ~RMD_BINARY; break;
-	case MG_RIBBINARY:
-	    _mgribc->render_device |= RMD_BINARY;
-	    _mgribc->render_device &= ~RMD_ASCII; break;
-	}
-	break;
-    case MG_RIBFILEPATH:
-	if(_mgribc->rib) fclose(_mgribc->rib);
-	strcpy(_mgribc->filepath, va_arg(*alist, char*));
-	_mgribc->rib = fopen(_mgribc->filepath,"w+");
-	break;
+      switch( va_arg(*alist, int) ) {
+      case MG_RIBASCII:  _mgribc->render_device = RMD_ASCII; break;
+      case MG_RIBBINARY: _mgribc->render_device = RMD_BINARY; break;
+      }
+      break;
     case MG_RIBDISPLAY:
-        _mgribc->display = va_arg(*alist, int);
-        break;
+      _mgribc->display = (enum display)va_arg(*alist, int);
+      break;
     case MG_RIBDISPLAYNAME:
-	strcpy(_mgribc->displayname, va_arg(*alist, char*));
-        break;
+      ribdpy = va_arg(*alist, char*);
+      break;
     case MG_RIBBACKING:
-        _mgribc->backing = va_arg(*alist, int);
-	break;
+      _mgribc->backing = (enum backing)va_arg(*alist, int);
+      break;
     case MG_RIBSHADER:
-        _mgribc->shader = va_arg(*alist, int);
-	break;
+      _mgribc->shader = (enum shader)va_arg(*alist, int);
+      break;
     case MG_RIBSCENE:
-    	strcpy(_mgribc->ribscene, va_arg(*alist, char*));
-	break;
+      strNcpy(_mgribc->ribscene, va_arg(*alist, char*));
+      break;
     case MG_RIBCREATOR:
-    	strcpy(_mgribc->ribcreator, va_arg(*alist, char*));
-	break;
+      strNcpy(_mgribc->ribcreator, va_arg(*alist, char*));
+      break;
     case MG_RIBFOR:
-    	strcpy(_mgribc->ribfor, va_arg(*alist, char*));
-	break;
+      strNcpy(_mgribc->ribfor, va_arg(*alist, char*));
+      break;
     case MG_RIBDATE:
-    	strcpy(_mgribc->ribdate, va_arg(*alist, char*));
-	break;
+      strNcpy(_mgribc->ribdate, va_arg(*alist, char*));
+      break;
     default:
-    
-    	OOGLError (0, "_mgrib_ctxset: undefined option: %d\n", attr);
-    	return;
-    	break;
+      OOGLError(0, "_mgrib_ctxset: undefined option: %d\n", attr);
+      return -1;
     }
   }
 
-  if (_mgc->shown && !_mgribc->born) {
+  if (ribfile != NULL && ribdpy == NULL) {
+    OOGLError(0,
+	      "_mgrib_ctxset: must not specify a file without a displayname");
+    return -1;
+  }
 
+  if (ribdpy != NULL) {
+    bool rfopened = false;
+
+    if (ribfile == NULL) {
+      ribfile = fopen(ribdpy, "w+");
+      rfopened = true;
+    }
+    if (ribfile == NULL) {
+      OOGLError(0, "_mgrib_ctxset: unable to open file for rib display \"%s\"",
+		ribdpy);
+      return -1;
+    }
+    if (fstat(fileno(ribfile), &st) < 0) {
+      OOGLError(0, "_mgrib_ctxset: unable to stat file for rib display \"%s\"",
+		ribdpy);
+      if (rfopened) {
+	fclose(ribfile);
+      }
+      return -1;
+    }
+    /* ok, now insert the stuff into the context */
+    if (_mgribc->rib && _mgribc->rib_close) {
+      fclose(_mgribc->rib);
+    }
+    _mgribc->rib = ribfile;
+    strNcpy(_mgribc->displayname, ribdpy);
+  }
+
+  /* Now decide where to put texture data (anything else?)
+   *
+   * If ribfile is a regular file, then we asssume that
+   * dirname(ribdpy) is its path-component; texture data goes there,
+   * under the name basename(ribdpy)-txSEQ.tiff, where "SEQ" is an
+   * ever increasing number (during the life-time of the context).
+   *
+   * If ribfile is not a regular file and dirname(ribdpy) equals ".",
+   * then texture data goes to /tmp or ${TMPDIR}, in order not to
+   * bloat the current directory which piping data around.
+   *
+   * If dirname(ribdpy) != ".", but dirname(ribdpy) does not exist,
+   * then we loose. So what.
+   */
+
+  /* Extract the path-component from displayname */
+  strcpy(_mgribc->displaypath, _mgribc->displayname);
+  strcpy(_mgribc->displaypath, dirname(_mgribc->displaypath));
+  if (!S_ISREG(st.st_mode) && strcmp(_mgribc->displaypath, ".") == 0) {
+    strNcpy(_mgribc->displaypath, _mgribc->tmppath);
+  }
+
+  /* If displayname really is a path-name, then include only its
+   * basename component into the rib-output. Otherwise one has to edit
+   * the RIB-file each time one moves it around.
+   */
+  strcpy(_mgribc->displaybase, _mgribc->displayname);
+  strcpy(_mgribc->displaybase, basename(_mgribc->displaybase));
+  /* Strip the suffix; the RIB-file will contain displaybase.tiff,
+   * texturenames will be generated as displaybase-SEQ.tiff, where SEQ
+   * is an ever increasing number.
+   */
+  if ((dot = strrchr(_mgribc->displaybase, '.')) != NULL &&
+      (strcmp(dot, ".tiff") == 0 || strcmp(dot, ".rib") == 0)) {
+    *dot = '\0';
+  }
+  /* Finished. The displayname emitted to the RIB file now will always
+   * be without path-component.
+   */
+
+  if (_mgc->shown && !_mgribc->born) {
     /* open the window */
     mgribwindow(_mgc->win);
     /* rib state is *not* in accordance with appearance state:
        don't set the appearance until worldbegin time */
   }
+  return 0;
 }
 
-
 /*-----------------------------------------------------------------------
  * Function:	mgrib_ctxget
  * Description:	get a context attribute value
@@ -300,7 +397,7 @@ mgrib_ctxget(int attr, void* value)
 
   switch (attr) {
 
-  /* Attributes common to all MG contexts: */
+    /* Attributes common to all MG contexts: */
   case MG_APPEAR:
     *VALUE(Appearance*) = &(_mgc->astk->ap);
     break;
@@ -309,7 +406,7 @@ mgrib_ctxget(int attr, void* value)
     break;
   case  MG_WINDOW:
     if(_mgribc->born) {
-    /* IT DOESN'T MAKE SENSE TO DO THIS, DOES IT? */
+      /* IT DOESN'T MAKE SENSE TO DO THIS, DOES IT? */
     }
     *VALUE(WnWindow*) = _mgc->win;
     break;
@@ -327,52 +424,50 @@ mgrib_ctxget(int attr, void* value)
     /* not implemented now */
     *VALUE(int) = 0;
   case MG_SPACE: *VALUE(int) = _mgc->space; break;
-  case MG_NDINFO: *VALUE(void *) = _mgc->NDinfo; break;
-  case MG_NDMAP:  *VALUE(mgmapfunc) = _mgc->NDmap; break;
+  case MG_NDCTX: *VALUE(mgNDctx *) = _mgc->NDctx; break;
 
-  /* Attributes specific to RIB contexts: */
+    /* Attributes specific to RIB contexts: */
   case MG_RIBWINID:
     *VALUE(int) = _mgribc->win;
     break;
   case MG_RIBFILE:
     *VALUE(FILE*) = _mgribc->rib;
-     break;
+    break;
 
-  /* Attributes really specific to RI contexts: */
+    /* Attributes really specific to RI contexts: */
   case MG_RIBLINEMODE:
-     *VALUE(int) = _mgribc->line_mode;
-	 break;
+    *VALUE(int) = _mgribc->line_mode;
+    break;
   case MG_RIBFORMAT:
-     if(_mgribc->render_device & RMD_ASCII) *VALUE(int) = MG_RIBASCII;
-     if(_mgribc->render_device & RMD_BINARY) *VALUE(int) = MG_RIBBINARY;
-	 break;
-  case MG_RIBFILEPATH:
-     *VALUE(char *) = _mgribc->filepath;
-	 break;
+    switch (_mgribc->render_device) {
+    case RMD_ASCII: *VALUE(int) = MG_RIBASCII; break;
+    case RMD_BINARY: *VALUE(int) = MG_RIBBINARY; break;
+    }
+    break;
   case MG_RIBDISPLAY:
-     *VALUE(int) = _mgribc->display;
-	 break;
+    *VALUE(int) = _mgribc->display;
+    break;
   case MG_RIBDISPLAYNAME:
-     *VALUE(char *) = _mgribc->displayname;
-	 break;
+    *VALUE(char *) = _mgribc->displayname;
+    break;
   case MG_RIBBACKING:
-     *VALUE(int) = _mgribc->backing;
-     break;
+    *VALUE(int) = _mgribc->backing;
+    break;
   case MG_RIBSHADER:
-     *VALUE(int) = _mgribc->shader;
-     break;
+    *VALUE(int) = _mgribc->shader;
+    break;
   case MG_RIBSCENE:
-     *VALUE(char *) = _mgribc->ribscene;
-     break;
+    *VALUE(char *) = _mgribc->ribscene;
+    break;
   case MG_RIBCREATOR:
-     *VALUE(char *) = _mgribc->ribcreator;
-     break;
+    *VALUE(char *) = _mgribc->ribcreator;
+    break;
   case MG_RIBFOR:
-     *VALUE(char *) = _mgribc->ribfor;
-     break;
+    *VALUE(char *) = _mgribc->ribfor;
+    break;
   case MG_RIBDATE:
-     *VALUE(char *) = _mgribc->ribdate;
-     break;
+    *VALUE(char *) = _mgribc->ribdate;
+    break;
   default:
     OOGLError (0, "mgrib_ctxget: undefined option: %d\n", attr);
     return -1;
@@ -398,66 +493,68 @@ WnWindow *
 mgribwindow(WnWindow *win)
 {
   WnPosition wp;
-  int xsize, ysize, flag, reconstrain;
-  char *name;
-  char gver[80];
-  double pxloc, pyloc, pxsize, pysize;
+  int xsize, ysize;
   char fullscene[280], fullcreator[280], fullfor[280], fulldate[280];
+  char dpyname[PATH_MAX];
 
-    /* RIB 1.0 structure comments */
-    sprintf(fullscene,"Scene %s",_mgribc->ribscene);
-    sprintf(fullcreator,"Creator %s",_mgribc->ribcreator);
-    sprintf(fullfor,"For %s",_mgribc->ribfor);
-    sprintf(fulldate,"CreationDate %s",_mgribc->ribdate);
-    mrti(mr_header,"RenderMan RIB-Structure 1.0", mr_nl,
-    	 mr_header, fullscene, mr_nl,
-	 mr_header, fullcreator, mr_nl,
-	 mr_header, fulldate, mr_nl,
-	 mr_header, fullfor, mr_nl,
-	 mr_header, "Frames 1", mr_nl,	/* always 1 frame*/
-	 mr_NULL);
+  /* RIB 1.0 structure comments */
+  sprintf(fullscene,"Scene %s",_mgribc->ribscene);
+  sprintf(fullcreator,"Creator %s",_mgribc->ribcreator);
+  sprintf(fullfor,"For %s",_mgribc->ribfor);
+  sprintf(fulldate,"CreationDate %s",_mgribc->ribdate);
+  mrti(mr_header,"RenderMan RIB-Structure 1.0", mr_nl,
+       mr_header, fullscene, mr_nl,
+       mr_header, fullcreator, mr_nl,
+       mr_header, fulldate, mr_nl,
+       mr_header, fullfor, mr_nl,
+       mr_header, "Frames 1", mr_nl,	/* always 1 frame*/
+       mr_NULL);
 
-    /* options */
-    if(_mgribc->shader && _mgribc->shadepath) {
-	mrti(mr_header, "CapabilitiesNeeded ShadingLanguage", mr_nl,
-	     mr_embed, "version 3.03", mr_nl,
-	     mr_option, mr_string, "searchpath", mr_string, "shader",
-		mr_nl, mr_embed, "[", mr_string, _mgribc->shadepath,
-		mr_embed, "]", mr_nl, mr_nl, mr_NULL);
-    } else {
-        mrti(mr_embed, "version 3.03", mr_nl, mr_nl, mr_NULL);
-    }
+  /* options */
+  if(_mgribc->shader != RM_STDSHADE && _mgribc->shadepath) {
+    mrti(mr_header, "CapabilitiesNeeded ShadingLanguage", mr_nl,
+	 mr_embed, "version 3.03", mr_nl,
+	 mr_option, mr_string, "searchpath", mr_string, "shader",
+	 mr_nl, mr_embed, "[", mr_string, _mgribc->shadepath,
+	 mr_embed, "]", mr_nl, mr_nl, mr_NULL);
+  } else {
+    mrti(mr_embed, "version 3.03", mr_nl, mr_nl, mr_NULL);
+  }
         
-    /* set display characteristics...*/
-    mrti(mr_display, mr_string, _mgribc->displayname, 
-         (_mgribc->display == MG_RIBFRAME) ? mr_framebuffer : mr_file, 
-         (_mgribc->backing == MG_RIBDOBG) ? mr_rgb : mr_rgba, mr_NULL);
+  /* set display characteristics...*/
+  snprintf(dpyname, PATH_MAX, "%s%s", _mgribc->displaybase, 
+	   _mgribc->display == MG_RIBTIFF ? ".tiff" : ".rib");
+  mrti(mr_display, mr_string, dpyname, 
+       (_mgribc->display == MG_RIBFRAME) ? mr_framebuffer : mr_file, 
+       (_mgribc->backing == MG_RIBDOBG) ? mr_rgb : mr_rgba, mr_NULL);
 
-    /* window size */
-    WnGet(_mgc->win, WN_CURPOS, &wp);
-    xsize = wp.xmax - wp.xmin + 1;
-    ysize = wp.ymax - wp.ymin + 1;
-    mrti(mr_format, mr_int, xsize, mr_int, ysize, mr_float, 1., mr_NULL);
+  /* window size */
+  WnGet(_mgc->win, WN_CURPOS, &wp);
+  xsize = wp.xmax - wp.xmin + 1;
+  ysize = wp.ymax - wp.ymin + 1;
+  mrti(mr_format, mr_int, xsize, mr_int, ysize, mr_float, 1., mr_NULL);
 
-    _mgribc->born = 1;
-    return(win);
+  _mgribc->born = true;
+  return win;
 }
 
 /*-----------------------------------------------------------------------
  * Function:	mgrib_ctxset
  * Description:	set some context attributes
  * Args:	a1, ...: list of attribute-value pairs
- * Returns:	nothing
+ * Returns:	-1 on error, 0 on success
  * Author:	mbp
  * Date:	Fri Sep 20 12:00:18 1991
  */
-void mgrib_ctxset( int a1, ...  )
+int mgrib_ctxset( int a1, ...  )
 {
   va_list alist;
-
+  int result;
+  
   va_start( alist, a1 );
-  _mgrib_ctxset(a1, &alist);
+  result = _mgrib_ctxset(a1, &alist);
   va_end(alist);
+  return result;
 }
 
 
@@ -489,20 +586,27 @@ int mgrib_feature( int feature )
  */
 void mgrib_ctxdelete( mgcontext *ctx )
 {
-/*  if(_mgribc->rib) fclose(_mgribc->rib); */
+  /*  if(_mgribc->rib) fclose(_mgribc->rib); */
 
   if(ctx->devno != MGD_RIB) {
     mgcontext *was = _mgc;
     mgctxselect(ctx);
     mgctxdelete(ctx);
-    if(was != ctx)
-	mgctxselect(was);
+    if(was != ctx) {
+      mgctxselect(was);
+    }
   } else {
-    if(((mgribcontext *)ctx)->shadepath)
-        free(((mgribcontext *)ctx)->shadepath);
+    if(((mgribcontext *)ctx)->shadepath) {
+      free(((mgribcontext *)ctx)->shadepath);
+    }
+    if (_mgribc->tx) {
+      OOGLFree(_mgribc->tx);
+    }
+    mrti_delete(&_mgribc->worldbuf);
+    mrti_delete(&_mgribc->txbuf);
     mg_ctxdelete(ctx);
     if(ctx == _mgc)
-	_mgc = NULL;
+      _mgc = NULL;
   }
 }
 
@@ -522,7 +626,7 @@ mgrib_ctxselect( mgcontext *ctx )
   }
 
   _mgc = ctx;
-  MGRIB = (mgribcontext *)ctx;
+  _MGRIB_ = (mgribcontext *)ctx;
 
   return(0);
 }
@@ -550,122 +654,145 @@ void
 mgrib_worldbegin( void )
 {
   static Transform cam2ri = {{1, 0,0,0}, {0,1,0,0}, {0,0,-1,0},{0,0,0,1}};
-  int persp;
   float halfxfield, halfyfield, aspect, cnear, cfar, fov;
-  WnWindow *win;
-  int xsize, ysize;
-  Transform T;
-  WnPosition wp;
   char str[256];
-  float focallen;
   HPoint3 look;
   Point3 lookat;
   LtLight **lp;
   int i;
 
-    /* first, check to see if we need to open the default rib file    */
-    /* IT'S NOW POSSIBLE THAT THIS WON'T GET SENT TO A FILE (streams) */
-    /* IN WHICH CASE IT WOULD BE WRONG TO OPEN A FILE. MOVE THIS!!    */
-    if(!_mgribc->rib)
-    	_mgribc->rib = fopen(DEFAULT_RIB_FILE, "w+");
-	
-    /* interpret options...(none exist now) */
-
-
-    mg_worldbegin();
-    mg_findcam();
-
-    LM_FOR_ALL_LIGHTS( _mgc->astk->ap.lighting, i,lp ) {
-	(*lp)->changed = 1;	/* We'll need to emit all the lights once */
+  /* first, check to see if we need to open the default rib file    */
+  /* IT'S NOW POSSIBLE THAT THIS WON'T GET SENT TO A FILE (streams) */
+  /* IN WHICH CASE IT WOULD BE WRONG TO OPEN A FILE. MOVE THIS!!    */
+  if(!_mgribc->rib) {
+    if (mgrib_ctxset(MG_RIBDISPLAYNAME, DEFAULT_RIB_FILE, MG_END) == -1) {
+      OOGLError(0, "mgrib_worldbeging(): unable to open default file \"%s\"",
+		DEFAULT_RIB_FILE);
     }
+  }
+	
+  /* interpret options...(none exist now) */
 
+  mg_worldbegin();
+  mg_findcam();
 
-    /* Camera is at (in its own coordinates)  (0,0,0).
-     * Looking toward vector (0,0,-1)
-     * nominally looking at a point (0,0,-focallen)
-     */
-    CamGet(_mgc->cam, CAM_FOCUS, &focallen);
-    look.x = look.y = 0;  look.z = -focallen;   look.w = 1;
-    /* Transform to world coordinates */
-    HPt3TransPt3(_mgc->C2W, &look, &lookat);    
-    /* Now the camera is at _mgc->cpos (this is a Point3), */
-    /* looking at lookat (another Point3) */
+  LM_FOR_ALL_LIGHTS( _mgc->astk->ap.lighting, i,lp ) {
+    (*lp)->changed = 1;	/* We'll need to emit all the lights once */
+  }
+
+  /* Camera is at (in its own coordinates)  (0,0,0).
+   * Looking toward vector (0,0,-1)
+   * nominally looking at a point (0,0,-focallen)
+   */
+  CamGet(_mgc->cam, CAM_FOCUS, &_mgribc->focallen);
+  look.x = look.y = 0;  look.z = -_mgribc->focallen;   look.w = 1;
+  /* Transform to world coordinates */
+  HPt3TransPt3(_mgc->C2W, &look, &lookat);    
+  /* Now the camera is at _mgc->cpos (this is a Point3), */
+  /* looking at lookat (another Point3) */
    
-    /* interpret camera ...*/
-    CamGet( _mgc->cam, CAM_NEAR, &cnear);
-    CamGet( _mgc->cam, CAM_FAR, &cfar);
-    mrti(mr_clipping, mr_float, cnear, mr_float, cfar, mr_NULL);
-    CamGet( _mgc->cam, CAM_PERSPECTIVE, &persp);
-    mrti(mr_projection, mr_string, persp?"perspective":"orthographic", mr_NULL);
-    CamGet( _mgc->cam, CAM_ASPECT, &aspect );
-    CamGet( _mgc->cam, CAM_HALFYFIELD, &halfyfield );
-    halfxfield = halfyfield * aspect;
-    mrti(mr_screenwindow, mr_float, -halfxfield, mr_float, halfxfield,
-    	mr_float, -halfyfield, mr_float, halfyfield, mr_NULL);
-    CamGet( _mgc->cam, CAM_FOV, &fov);
-    sprintf(str, "Field of view %g", fov);
-    mrti(mr_comment, str, mr_NULL);
-    mrti(mr_framebegin, mr_int, 1, mr_nl, mr_NULL);
+  /* interpret camera ...*/
+  CamGet( _mgc->cam, CAM_NEAR, &cnear);
+  CamGet( _mgc->cam, CAM_FAR, &cfar);
+
+  /* make our buffer the current one */
+  mrti_makecurrent(&_mgribc->worldbuf);
+
+  mrti(mr_clipping, mr_float, cnear, mr_float, cfar, mr_NULL);
+  CamGet( _mgc->cam, CAM_PERSPECTIVE, &_mgribc->persp);
+  mrti(mr_projection, mr_string,
+       _mgribc->persp ? "perspective" : "orthographic", mr_NULL);
+  CamGet( _mgc->cam, CAM_ASPECT, &aspect );
+  CamGet( _mgc->cam, CAM_HALFYFIELD, &halfyfield );
+  halfxfield = halfyfield * aspect;
+  mrti(mr_screenwindow, mr_float, -halfxfield, mr_float, halfxfield,
+       mr_float, -halfyfield, mr_float, halfyfield, mr_NULL);
+  CamGet( _mgc->cam, CAM_FOV, &fov);
+  sprintf(str, "Field of view %.8g", fov);
+  mrti(mr_comment, str, mr_NULL);
+  mrti(mr_framebegin, mr_int, 1, mr_nl, mr_NULL);
     
-    sprintf(str, "CameraOrientation %.4g %.4g %.4g %.4g %.4g %.4g",
-    	    _mgc->cpos.x,_mgc->cpos.y,_mgc->cpos.z,
+  if (_mgc->cpos.w != 0.0 && _mgc->cpos.w != 1.0) {
+    sprintf(str, "CameraOrientation %.8g %.8g %.8g %.8g %.8g %.8g",
+	    _mgc->cpos.x/_mgc->cpos.w,
+	    _mgc->cpos.y/_mgc->cpos.w,
+	    _mgc->cpos.z/_mgc->cpos.w,
 	    lookat.x,lookat.y,lookat.z);
-    mrti(mr_header, str, mr_nl, mr_NULL);
-    mrti(mr_identity, mr_NULL);
-    mgrib_printmatrix(cam2ri);
+  } else {
+    sprintf(str, "CameraOrientation %.8g %.8g %.8g %.8g %.8g %.8g",
+	    _mgc->cpos.x,_mgc->cpos.y,_mgc->cpos.z,
+	    lookat.x,lookat.y,lookat.z);
+  }
+  mrti(mr_header, str, mr_nl, mr_NULL);
+  mrti(mr_identity, mr_NULL);
+  mgrib_printmatrix(cam2ri);
 
-    mgrib_printmatrix(_mgc->W2C);
+  mgrib_printmatrix(_mgc->W2C);
 
+  /* otherwise explicitly specified normals would be inverted */
+  mrti(mr_reverseorientation, mr_NULL);
 
-    /* RiWorldBegin...*/
-    mrti(mr_nl, mr_nl, mr_worldbegin, mr_NULL);
+  /* declare "bgcolor" as uniform float, needed by the GVblend filters
+   * (GL_BLEND texturing).
+   *
+   * We also declare At (\alpha_{texture}), which is used to switch
+   * off texture introduced transparency when the appearance has
+   * transparency disabled.
+   */
+  mrti(mr_declare, mr_string, "bgcolor", mr_string, "uniform color", mr_NULL);
+  mrti(mr_declare, mr_string, "At", mr_string, "uniform float", mr_NULL);
 
-    /* if the option is selected, add the background */
-    if(_mgribc->backing == MG_RIBDOBG) {
-        float halfxbg = cfar * halfxfield, halfybg = cfar * halfyfield;
-	float farz = -.99 * cfar;
-	Point3 bg[4];
+  /* RiWorldBegin...*/
+  mrti(mr_nl, mr_nl, mr_worldbegin, mr_NULL);
 
-	bg[0].x = -halfxbg; bg[0].y = -halfybg; bg[0].z = farz;
-	bg[1].x = -halfxbg; bg[1].y =  halfybg; bg[1].z = farz;
-	bg[2].x =  halfxbg; bg[2].y =  halfybg; bg[2].z = farz;
-	bg[3].x =  halfxbg; bg[3].y = -halfybg; bg[3].z = farz;
+  _mgribc->world = 1;
+    
+  /* if the option is selected, add the background */
+  if(_mgribc->backing == MG_RIBDOBG) {
+    float halfxbg = cfar * halfxfield, halfybg = cfar * halfyfield;
+    float farz = -.99 * cfar;
+    Point3 bg[4];
+
+    bg[0].x = -halfxbg; bg[0].y = -halfybg; bg[0].z = farz;
+    bg[1].x = -halfxbg; bg[1].y =  halfybg; bg[1].z = farz;
+    bg[2].x =  halfxbg; bg[2].y =  halfybg; bg[2].z = farz;
+    bg[3].x =  halfxbg; bg[3].y = -halfybg; bg[3].z = farz;
 	
-	mrti(mr_comment, "simulate background color via polygon", mr_NULL);
-	mrti(mr_attributebegin, mr_NULL);
-	mgrib_printmatrix(_mgc->C2W);
-	mrti(mr_surface, mr_constant, mr_NULL);
-	mrti(mr_color, mr_parray, 3, &_mgc->background, mr_NULL);
-	mrti(mr_polygon, mr_P, mr_parray, 4*3, bg, mr_NULL);
-	mrti(mr_attributeend, mr_NULL);
-    }			
+    mrti(mr_comment, "simulate background color via polygon", mr_NULL);
+    mrti(mr_attributebegin, mr_NULL);
+    mgrib_printmatrix(_mgc->C2W);
+    mrti(mr_surface, mr_constant, mr_NULL);
+    mrti(mr_color, mr_parray, 3, &_mgc->background, mr_NULL);
+    mrti(mr_polygon, mr_P, mr_parray, 4*3, bg, mr_NULL);
+    mrti(mr_attributeend, mr_NULL);
+  }			
 
-    /* bring ri state into accordance with appearance state */
+  /* bring ri state into accordance with appearance state */
   
-    {
-      Appearance *ap = ApCopy( &(_mgc->astk->ap), NULL );
-      mgrib_setappearance( ap, MG_SET );
-      ApDelete(ap);
-    }
+  {
+    Appearance *ap = ApCopy( &(_mgc->astk->ap), NULL );
+    mgrib_setappearance( ap, MG_SET );
+    ApDelete(ap);
+  }
 
-    /* NOW DONE BY mg_worldbegin()
-     *CamView(_mgc->cam, _mgribc->W2S);
-     *TmTranslate(T, 1.0, 1.0, 0);
-     *TmConcat(_mgribc->W2S,T, _mgribc->W2S);
-     */
+  /* NOW DONE BY mg_worldbegin()
+   *CamView(_mgc->cam, _mgribc->W2S);
+   *TmTranslate(T, 1.0, 1.0, 0);
+   *TmConcat(_mgribc->W2S,T, _mgribc->W2S);
+   */
      
 
-    /* NOW DONE BY mg_worldbegin()
-     *mgrib_ctxget(MG_WINDOW, &win);
-     *WnGet(_mgc->win, WN_CURPOS, &wp);
-     *xsize = wp.xmax - wp.xmin + 1;
-     *ysize = wp.ymax - wp.ymin + 1;
-     *
-     *TmScale(T, (double)xsize*.5, (double)ysize*.5, 1.0);
-     *TmConcat(_mgribc->W2S,T, _mgribc->W2S);
-     *
-     *TmCopy(_mgribc->W2S, _mgribc->O2S);
-     */
+  /* NOW DONE BY mg_worldbegin()
+   *mgrib_ctxget(MG_WINDOW, &win);
+   *WnGet(_mgc->win, WN_CURPOS, &wp);
+   *xsize = wp.xmax - wp.xmin + 1;
+   *ysize = wp.ymax - wp.ymin + 1;
+   *
+   *TmScale(T, (double)xsize*.5, (double)ysize*.5, 1.0);
+   *TmConcat(_mgribc->W2S,T, _mgribc->W2S);
+   *
+   *TmCopy(_mgribc->W2S, _mgribc->O2S);
+   */
 }
 
 /*-----------------------------------------------------------------------
@@ -678,11 +805,11 @@ mgrib_worldbegin( void )
 void
 mgrib_worldend( void )
 {
-    unsigned char *buffer = tokenbuffer;
-    mrti(mr_worldend, mr_nl, mr_NULL);
-    mrti(mr_frameend, mr_nl, mr_NULL);
-    /* now flush the buffer, if appropriate */
-    /* mgrib_flushbuffer(); NOW DONE AT HIGHER LEVEL */
+  _mgribc->world = 0;
+  mrti(mr_worldend, mr_nl, mr_NULL);
+  mrti(mr_frameend, mr_nl, mr_NULL);
+  /* now flush the buffer, if appropriate */
+  /* mgrib_flushbuffer(); NOW DONE AT HIGHER LEVEL */
 
 }
 
@@ -699,18 +826,18 @@ mgrib_worldend( void )
 void
 mgrib_reshapeviewport( void )
 {
-    WnWindow *win;
-    WnPosition wp;
-    int xsize, ysize;
+  WnWindow *win;
+  WnPosition wp;
+  int xsize, ysize;
 
-    mgrib_ctxget(MG_WINDOW, &win);	/* Get window; force it to ask
+  mgrib_ctxget(MG_WINDOW, &win);	/* Get window; force it to ask
 					 * NeXTStep how big the window is
 					 */
-    WnGet(win, WN_CURPOS, &wp);
-    xsize = wp.xmax - wp.xmin + 1;
-    ysize = wp.ymax - wp.ymin + 1;
+  WnGet(win, WN_CURPOS, &wp);
+  xsize = wp.xmax - wp.xmin + 1;
+  ysize = wp.ymax - wp.ymin + 1;
 
-    CamSet(_mgc->cam, CAM_ASPECT, (double)xsize/(double)ysize, CAM_END);
+  CamSet(_mgc->cam, CAM_ASPECT, (double)xsize/(double)ysize, CAM_END);
 }
 
 /*-----------------------------------------------------------------------
@@ -725,7 +852,7 @@ mgrib_reshapeviewport( void )
 void
 mgrib_identity( void )
 {
-    mgrib_settransform( TM_IDENTITY );
+  mgrib_settransform( TM_IDENTITY );
 }
 
 /*-----------------------------------------------------------------------
@@ -742,9 +869,9 @@ mgrib_identity( void )
 void
 mgrib_transform( Transform T )
 {
-    mg_transform(T);
-    mgrib_printmatrix(T);
-    TmConcat(_mgc->xstk->T, _mgc->W2S, _mgc->O2S);
+  mg_transform(T);
+  mgrib_printmatrix(T);
+  TmConcat(_mgc->xstk->T, _mgc->W2S, _mgc->O2S);
 }
 
 /*-----------------------------------------------------------------------
@@ -761,9 +888,9 @@ mgrib_transform( Transform T )
 int
 mgrib_pushtransform( void )
 {
-    mg_pushtransform();
-    mrti(mr_transformbegin, mr_NULL);
-    return 0;
+  mg_pushtransform();
+  mrti(mr_transformbegin, mr_NULL);
+  return 0;
 }
 
 /*-----------------------------------------------------------------------
@@ -780,10 +907,10 @@ mgrib_pushtransform( void )
 int
 mgrib_poptransform( void )
 {
-    mg_poptransform();
-    mrti(mr_transformend, mr_NULL);
-    TmConcat(_mgc->xstk->T, _mgc->W2S, _mgc->O2S);
-    return 0;
+  mg_poptransform();
+  mrti(mr_transformend, mr_NULL);
+  TmConcat(_mgc->xstk->T, _mgc->W2S, _mgc->O2S);
+  return 0;
 }
 
 
@@ -799,11 +926,11 @@ mgrib_poptransform( void )
 void
 mgrib_settransform( Transform T )
 {
-    mrti(mr_identity, mr_NULL);
-    mg_settransform( T );
-    TmConcat(_mgc->xstk->T, _mgc->W2S, _mgc->O2S);
-    if(T != TM_IDENTITY)
-	mgrib_printmatrix(T);
+  mrti(mr_identity, mr_NULL);
+  mg_settransform( T );
+  TmConcat(_mgc->xstk->T, _mgc->W2S, _mgc->O2S);
+  if(T != TM_IDENTITY)
+    mgrib_printmatrix(T);
 }
 
 /*-----------------------------------------------------------------------
@@ -817,7 +944,8 @@ int
 mgrib_pushappearance( void )
 {
   mg_pushappearance();
-  mrti(mr_comment, "push appearance", mr_attributebegin, mr_NULL);
+  if (_mgribc->world)
+    mrti(mr_comment, "push appearance", mr_attributebegin, mr_NULL);
   return 0;
 }
 
@@ -834,15 +962,15 @@ mgrib_pushappearance( void )
 int
 mgrib_popappearance( void )
 {
-  register struct mgastk *mastk = _mgc->astk;
-  register struct mgastk *mastk_next;
+  struct mgastk *mastk = _mgc->astk;
 
-  if (! (mastk_next=mastk->next)) {
+  if (mastk->next == NULL) {
     OOGLError(0, "mggl_popappearance: appearance stack has only 1 entry.\n");
     return 0;
   }
 
-  mrti(mr_comment, "pop appearance", mr_attributeend, mr_NULL);
+  if (_mgribc->world)
+    mrti(mr_comment, "pop appearance", mr_attributeend, mr_NULL);
   mg_popappearance();
   return 0;
 }
@@ -855,13 +983,12 @@ mgrib_popappearance( void )
  *		since shaders depend on both appearance and material 
  *		settings. (4/1/93 TMM)
  */
-Appearance *
-mgrib_setappearance( Appearance* ap, int mergeflag )
+const Appearance *
+mgrib_setappearance(const Appearance* ap, int mergeflag )
 {
   int changed, mat_changed, lng_changed;
   struct mgastk *mastk = _mgc->astk;
   Appearance *ma;
-  static float nullarray[] = { 0.0 };
 
   ma = &(mastk->ap);
 
@@ -889,23 +1016,8 @@ mgrib_setappearance( Appearance* ap, int mergeflag )
     /* interpret lights ... */
     mgrib_lighting(_mgc->astk, lng_changed);
   }
-  return ap;
-}
 
-
-/*-----------------------------------------------------------------------
- * Function:	mgrib_getappearance
- * Description:	return a ptr to current appearance
- * Returns:	ptr to current appearance
- * Author:	mbp
- * Date:	Fri Sep 20 13:00:41 1991
- * Notes:	Applications should not modify the returned appearance
- *		in any way.
- */
-Appearance *
-mgrib_getappearance()
-{
-    return &(_mgc->astk->ap);
+  return &_mgc->astk->ap;
 }
 
 /*-----------------------------------------------------------------------
@@ -938,34 +1050,42 @@ mgrib_setcamera( Camera* cam )
 mgribcontext *
 mgrib_newcontext( mgribcontext *ctx )
 {
-  static char stdshaderpaths[] =
-	".:shaders:/usr/local/prman/prman/lib/shaders:/NextLibrary/Shaders";
-
+  static const char stdshaderpaths[] =
+    ".:shaders:/usr/local/prman/prman/lib/shaders:/NextLibrary/Shaders";
+  
   char *geomdata = getenv("GEOMDATA");
 
+  memset(ctx, 0, sizeof(mgribcontext));
+
   mg_newcontext(&(ctx->mgctx));
   ctx->mgctx.devfuncs = &mgribfuncs;
   ctx->mgctx.devno = MGD_RIB;
   ctx->mgctx.astk->ap_seq = 1;
   ctx->mgctx.astk->mat_seq = 1;
   ctx->mgctx.astk->light_seq = 1;
-  ctx->born = 0;
+  ctx->born = false;
   ctx->rib = NULL;
-  ctx->backing = MG_RIBDOBG;
-  ctx->shader = MG_RIBSTDSHADE;
+  ctx->rib_close = false;
+  ctx->backing = RB_DOBG;
+  ctx->shader = RM_EXTSHADE;
   ctx->shadepath = NULL;		/* should add context field */
-  if(geomdata) {
+  if (geomdata) {
     char path[512];
-    sprintf(path, "%s:%s/shaders", stdshaderpaths, geomdata);
+    sprintf(path, "%s:%s/shaders:&", stdshaderpaths, geomdata);
     ctx->shadepath = strdup(path);
   }
 
+  if ((ctx->tmppath = getenv("TMPDIR")) == NULL) {
+    ctx->tmppath = "/tmp";
+  }
+
   /* initalize the token interface */
-  if(!tokenbuffer) mrti_init();
-  mrti_reset();
+  mrti_init(&ctx->worldbuf);
+  mrti_init(&ctx->txbuf);
+  mrti_makecurrent(&ctx->worldbuf);
 
   ctx->render_device = RMD_ASCII;
-  ctx->line_mode = MG_RIBCYLINDER;
+  ctx->line_mode = RM_CYLINDER;
 
   return ctx;
 }
@@ -981,11 +1101,11 @@ mgrib_newcontext( mgribcontext *ctx )
 mgcontext *
 mgrib_findctx( long winid )
 {
-  register struct mgcontext *mgc;
+  struct mgcontext *mgc;
 
   for(mgc = _mgclist; mgc != NULL; mgc = mgc->next) {
     if(mgc->devno == MGD_RIB && ((mgribcontext *)mgc)->win == winid)
-	return mgc;
+      return mgc;
   }
   return NULL;
 }
@@ -1001,41 +1121,46 @@ mgrib_findctx( long winid )
 void
 mgrib_flushbuffer()
 {
-    unsigned char *buffer = tokenbuffer;
-    /* do we even want a buffer anymore? why? */
-    if(!_mgribc->rib)
-    	_mgribc->rib = fopen(DEFAULT_RIB_FILE, "w+");
-    while(buffer<ptr) putc(*(buffer++), _mgribc->rib);
-    fflush(_mgribc->rib);
-    mrti_reset(_mgribc->rib);
+  TokenBuffer *wbuf = &_mgribc->worldbuf;
+  size_t size;
+
+  if(!_mgribc->rib) {
+    if (mgrib_ctxset(MG_RIBDISPLAYNAME, DEFAULT_RIB_FILE, MG_END) == -1) {
+      return;
+    }
+  }
+  if (_mgribc->tx) {
+    TokenBuffer *txbuf = &_mgribc->txbuf;
+
+    size = (size_t)wbuf->tkb_worldptr - (size_t)wbuf->tkb_buffer;
+    if (size && fwrite(wbuf->tkb_buffer, size, 1, _mgribc->rib) != 1) {
+      OOGLError(1, "Error flushing RIB tokenbuffer (prologue)");
+    }
+    mrti_makecurrent(txbuf);
+    mrti(mr_nl, mr_nl, mr_NULL);
+    size = (size_t)txbuf->tkb_ptr - (size_t)txbuf->tkb_buffer;
+    if (size && fwrite(txbuf->tkb_buffer, size, 1, _mgribc->rib) != 1) {
+      OOGLError(1, "Error flushing RIB tokenbuffer (textures)");
+    }
+    size = (size_t)wbuf->tkb_ptr - (size_t)wbuf->tkb_worldptr;
+    if (size && fwrite(wbuf->tkb_worldptr, size, 1, _mgribc->rib) != 1) {
+      OOGLError(1, "Error flushing RIB tokenbuffer (world)");
+    }
+    mrti_reset();
+  } else {
+    size = (size_t)wbuf->tkb_ptr - (size_t)wbuf->tkb_buffer;
+    if (size && fwrite(wbuf->tkb_buffer, size, 1, _mgribc->rib) != 1) {
+      OOGLError(1, "Error flushing RIB tokenbuffer");
+    }
+  }
+  fflush(_mgribc->rib);
+  mrti_makecurrent(wbuf);
+  mrti_reset();
 }
 
-/*-----------------------------------------------------------------------
- * Function:	mgrib_terminatebuffer
- * Description: NULL terminates the tokenbuffer, required to support
- 		NeXTs render panel 
- * Returns:	nothing
- * Author:	wisdom
- * Date:	Mon Jul 26 12:35:45 CDT 1993
- * Notes:	This is a public routine, prototyped in mgrib.h
+/*
+ * Local Variables: ***
+ * mode: c ***
+ * c-basic-offset: 2 ***
+ * End: ***
  */
-void
-mgrib_terminatebuffer()
-{
-    *(ptr++) = 0;
-}
-
-/*-----------------------------------------------------------------------
- * Function:	mgrib_tokenbuffer
- * Description: returns through indirection the tokenbuffer an size 
- * Returns:	char pointer to tokenbuffer
- * Author:	wisdom
- * Date:	Mon Jul 26 12:35:45 CDT 1993
- * Notes:	This is a public routine, prototyped in mgrib.h
- */
-void
-mgrib_tokenbuffer(char **buffer, int *size)
-{
-    *buffer = (char *)tokenbuffer;
-    *size = ptr - tokenbuffer;
-}
